#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>

#include <unistd.h>
#include <fcntl.h>

#include "jalbTerrain.h"
#include "jalbEntity.h"

#include "jalbClient.h"

#define BUFSIZE 15000

/** Networking */

// Returns the "Socket File Descriptor"
int myConnect(char *address) {
    struct addrinfo hints;
    struct addrinfo *result, *rp;
    int s;
//    ssize_t nread;
    /* Obtain address(es) matching host/port */

    memset(&hints, 0, sizeof(struct addrinfo));
    hints.ai_family = AF_UNSPEC;    /* Allow IPv4 or IPv6 */
    hints.ai_socktype = SOCK_DGRAM; /* Datagram socket */
    hints.ai_flags = 0;
    hints.ai_protocol = 0;          /* Any protocol */

    char *myPor = "2300";

    s = getaddrinfo(address, myPor, &hints, &result);
    if (s != 0) {
        fprintf(stderr, "getaddrinfo: %s\n", gai_strerror(s));
        exit(EXIT_FAILURE);
    }

    /* getaddrinfo() returns a list of address structures.
       Try each address until we successfully connect(2).
       If socket(2) (or connect(2)) fails, we (close the socket
       and) try the next address. */

    int sfd = -1;

    for (rp = result; rp != NULL; rp = rp->ai_next) {
        sfd = socket(rp->ai_family, rp->ai_socktype,
                     rp->ai_protocol);
        if (sfd == -1)
            continue;

        if (connect(sfd, rp->ai_addr, rp->ai_addrlen) != -1)
            break;                  /* Success */

        close(sfd);
    }

    int flags = fcntl(sfd, F_GETFL);
    flags |= O_NONBLOCK;
    fcntl(sfd, F_SETFL, flags);

    if (rp == NULL) {               /* No address succeeded */
        fprintf(stderr, "Could not connect\n");
        exit(EXIT_FAILURE);
    }

    freeaddrinfo(result);           /* No longer needed */

    return sfd;
}

// Returns sfd.
int sendWakeUp(struct jalbServer *ser, int sfd, unsigned char *buf) {
    // Wakeup
    if(sfd != -1) {
        close(sfd);
    }
    sfd = myConnect(ser->address);

//            printf("connectAttempt: %d\n", *sfd);

    if(sfd != -1) {
        // Send the user and password. If its good start the loadscreen and go into Ogl.

        // If its bad go to the connection screen.
        char sendBuff[1];

        sendBuff[0] = coWakeup;
        if (write(sfd, sendBuff, 1) != 1) {
            fprintf(stderr, "partial/failed write\n");
//                            exit(EXIT_FAILURE);
        }
    }
    return sfd;
}

void sendCreateAccount(struct jalbServer *ser, int sfd, unsigned char *buf) {
    // Create Account
    // Header, name, username.
    buf[0] = coNewAccount;
    strncpy((char*)&buf[1], ser->username, 256);
    strncpy((char*)&buf[257], ser->password, 256);
    if (write(sfd, buf, 513) != 513) {
        fprintf(stderr, "partial/failed write\n");
        // exit(EXIT_FAILURE);
    }
}

void sendLogIn(struct jalbServer *ser, int sfd, unsigned char *buf, int chunkRadius) {
    // Log In
    // Header, name, username, chunkRadius.
    buf[0] = coLogin;
    strncpy((char*)&buf[1], ser->username, 256);
    strncpy((char*)&buf[257], ser->password, 256);
    intToBuffer(chunkRadius, &buf[513]);
    if (write(sfd, buf, 517) != 517) {
        fprintf(stderr, "partial/failed write\n");
        // exit(EXIT_FAILURE);
    }
}

void sendUpdate(int sfd, unsigned char *buf, unsigned char *key) {
//    printf("sendUpdate()\n");
    buf[0] = coUpdate;
    memcpy(&buf[1], key, 16);
    if (write(sfd, buf, 17) != 17) {
        fprintf(stderr, "partial/failed write\n");
        // exit(EXIT_FAILURE);
    }
}

void intToBuffer(int i, unsigned char *buff) {
    *(int*)buff = htonl(i);
}

int buffToInt(unsigned char *buff) {
    return ntohl(*(int*)buff);
}

float bufferMult = 1.5;

int recvTerList(unsigned char *buff, int length, ArrayList *terList) {
    int index = 0;
    int x = buffToInt(&buff[index]);
    index += sizeof(int);
    int y = buffToInt(&buff[index]);
    index += sizeof(int);

    int numTers = arrayListGetLength(terList);
    int i;
    struct jalbTerrainList *ter;
    for(i = 0; i < numTers; i++) {
        ter = arrayListGetPointer(terList, i);
        if(ter->chunkCoords[0] == x && ter->chunkCoords[1] == y) {
            // Overwrite
            free(ter->faces);
            free(ter->verts);
            break;
        }
    }
    if( i == numTers) {
        ter = malloc(sizeof(struct jalbTerrainList));
        arrayListAddEndPointer(terList, ter);
    }

    printf("%d, %d\n", x, y);
    printf("length: %d\tindex: %d\n", length, index);

    // x, y, numVerts, numFaces, faces, verts, texis
    ter->chunkCoords[0] = x;
    ter->chunkCoords[1] = y;
    ter->numVerts = buffToInt(&buff[index]);
    index += sizeof(int);
    ter->numFaces = buffToInt(&buff[index]);
    index += sizeof(int);

    printf("verts, faces (%d, %d)\n", ter->numVerts, ter->numFaces);

    ter->maxVerts = ter->numVerts * bufferMult;
    ter->maxFaces = ter->numFaces * bufferMult;

    ter->verts = calloc(ter->maxVerts, sizeof(struct vertTex));
    ter->faces = malloc(ter->maxFaces * 3 * sizeof(int));

    memcpy(ter->verts, &buff[index], ter->numVerts * sizeof(struct vertTex));
    index += ter->numVerts * sizeof(struct vertTex);

    memcpy(ter->faces, &buff[index], ter->numFaces * 3 * sizeof(int));
    index += ter->numFaces * 3 * sizeof(float);

    printf("index: %d\n", index);

    return index;
}

int recvEnts(unsigned char *buff, int length, ArrayList *entList) {
    printf("recvEnts()\n");
    int index = 0;
    while(index < length) {
        struct jalbEntity *ent = malloc(sizeof(struct jalbEntity));
        // Id, model, scale, pos, chunkCoords
        ent->globalId = buffToInt(&buff[index]);
        index += sizeof(int);
        ent->modelIndex = buffToInt(&buff[index]);
        index += sizeof(int);
        memcpy(&ent->scale, &buff[index], sizeof(float));
        index += sizeof(float);
        memcpy(ent->pos, &buff[index], 3 * sizeof(float));
        index += 3 * sizeof(float);
        memcpy(ent->rots, &buff[index], 3 * sizeof(float));
        index += 3 * sizeof(float);
        ent->chunkCoords[0] = buffToInt(&buff[index]);
        index += sizeof(int);
        ent->chunkCoords[1] = buffToInt(&buff[index]);
        index += sizeof(int);

        sayEnt(ent);

        arrayListAddEndPointer(entList, ent);
    }
    return index;
}

void sendDisconnect(unsigned char *buf, int sfd, unsigned char *key) {
    buf[0] = coDisconnect;
    int bufIndex = 1;
    memcpy(&buf[bufIndex], key, 16);
    bufIndex += 16;
    if (write(sfd, buf, bufIndex) != bufIndex) {
        printf("partial send\n");
    }
}
