#include <gl.h>

#include <stdlib.h>
#include <stdio.h>

#include <math.h>

#include "jalbModel.h"

/** Model */

struct jalbModel *jalbInitModel() {
    struct jalbModel *model;
    model = (struct jalbModel*)malloc(sizeof(struct jalbModel));

    printf("model: %p\n", model);

    model->dimensions = 0;

    model->numVerts = 0;
    model->numFaces = 0;

    model->faces = NULL;
    model->verts = NULL;
    model->UVs = NULL;

    return model;
}

void jalbFreeModel(struct jalbModel *model) {
    if(model->verts != NULL) {
        free(model->verts );
        model->verts = NULL;
    }

    if(model->faces != NULL) {
        free(model->faces );
        model->faces = NULL;
    }

    if(model->UVs != NULL) {
        free(model->UVs );
        model->UVs = NULL;
    }

    free(model);
}

// reads wavefront (,obj) files.
void jalbReadModel(char* filename, struct jalbModel *model) {
// This is going to load up data types required by opengl with model data from and obj file.

// Make 2 parses through the object.
// Figure the amount of information to make space for.
// Fill that space with the information.

    printf("Load Model: %s\n", filename);

    FILE*     file;
    // open the file
    file = fopen(filename, "r");

    char      buf[128];

    GLuint numVerts = 0;
    GLuint numFaces = 0;
    GLuint numUVs = 0;

    int indexDimension;

    // Count the number of verts and faces
    while(fscanf(file, "%s", buf) != EOF) {
        switch(buf[0]) {
        case '#':				/* comment */
            /* eat up rest of line */
            fgets(buf, sizeof(buf), file);
            break;
        case 'v':
            switch(buf[1]) {
            case '\0':
                // model.numVerts = model.numVerts + 1;
                numVerts++;
                break;
            case 't':
                numUVs++;
                break;
            }
            /* eat up rest of line */
            fgets(buf, sizeof(buf), file);
            break;
        case 'f':
            /* eat up rest of line */
            fgets(buf, sizeof(buf), file);
            numFaces++;
            break;
        default:
            /* eat up rest of line */
            fgets(buf, sizeof(buf), file);
        }
    }

    GLuint *tempIndexes = NULL;
    if(numUVs > 0) {
        tempIndexes = malloc(sizeof(GLuint) * numFaces * 2 * 3);
        indexDimension = 2;
        model->dimensions = 2;
    } else {
        tempIndexes = malloc(sizeof(GLuint) * numFaces * 3);
        indexDimension = 1;
        model->dimensions = 1;
    }

    GLfloat *tempVerts;
    tempVerts = malloc(sizeof(GLfloat) * numVerts * 3);

    GLfloat *tempUVs;
    tempUVs = malloc(sizeof(GLfloat) * numUVs * 2);

    model->numFaces = numFaces;

    numVerts = 0;
    numFaces = 0;
    numUVs = 0;


    rewind(file);

    int v, t;

    while(fscanf(file, "%s", buf) != EOF) {
        switch(buf[0]) {
        case '#':				/* comment */
            /* eat up rest of line */
            fgets(buf, sizeof(buf), file);
            break;
        case 'v':
            switch(buf[1]) {
            case '\0':
                fscanf(file, "%f %f %f",
                       &tempVerts[3 * numVerts + 0],
                       &tempVerts[3 * numVerts + 1],
                       &tempVerts[3 * numVerts + 2]);
                numVerts++;
                break;
            case 't':
                fscanf(file, "%f %f",
                       &tempUVs[2 * numUVs + 0],
                       &tempUVs[2 * numUVs + 1]);
                numUVs++;
                break;
            default:
                /* eat up rest of line */
                fgets(buf, sizeof(buf), file);
                break;
            }
            break;
        case 'f':
            fscanf(file, "%s", buf);
            // printf("sscanf: %d\n", sscanf(buf, "%d/%d", &v, &t));
            /* can be one of %d, %d//%d, %d/%d, %d/%d/%d %d//%d */
            if (sscanf(buf, "%d/%d", &v, &t) == 2) {
                v--;
                t--;
                tempIndexes[3*2*numFaces] = v;
                tempIndexes[3*2*numFaces+1] = t;
                fscanf(file, "%d/%d", &v, &t);
                v--;
                t--;
                tempIndexes[3*2*numFaces+2] = v;
                tempIndexes[3*2*numFaces+3] = t;
                fscanf(file, "%d/%d", &v, &t);
                v--;
                t--;
                tempIndexes[3*2*numFaces+4] = v;
                tempIndexes[3*2*numFaces+5] = t;
            } else if (sscanf(buf, "%d", &v) == 1) {
                // printf("v: %d\n", v);
                v--;
                tempIndexes[3*numFaces] = v;
                fscanf(file, "%d", &v);
                v--;
                tempIndexes[3*numFaces+1] = v;
                fscanf(file, "%d", &v);
                v--;
                tempIndexes[3*numFaces+2] = v;
            }
            numFaces++;
            break;
        default:
//            printf("%s\n", buf);
            /* eat up rest of line */
            fgets(buf, sizeof(buf), file);
//            printf("%s\n", buf);
        }
    }

    // sayVectVectDimensions("tempIndexes", tempIndexes, numFaces, 3, 2);

    int size = 1;
    // run through indexes
    int i = 1;
    int j;
    int isUnique;
    if(indexDimension == 1) {
        /// Set the actual to the temps.
        // SIZE is the number of unique indexes.
        model->verts = malloc(sizeof(GLfloat) * numVerts * 3);
        model->faces = malloc(sizeof(GLuint) * numFaces * 3);
        model->numFaces = numFaces;
        model->numVerts = numVerts;
    } else if(indexDimension == 2) {
        for (; i < numFaces * 3; i++) {
            isUnique = 1;
            for (j = i - 1; j >= 0; j-=1 ) {
                if (tempIndexes[i * indexDimension] == tempIndexes[j * indexDimension]) {
                    if(tempIndexes[i * indexDimension + 1] == tempIndexes[j * indexDimension + 1]) {
                        isUnique = 0;
                        break;
                    }
                }
            }
            if (isUnique) {
                size++;
            }
        }
        // printf("size1: %d\n", size);

        // SIZE is the number of unique indexes.
        model->verts = malloc(sizeof(GLfloat) * size * 3);
        model->UVs = malloc(sizeof(GLuint) * size * 2);
        model->faces = malloc(sizeof(GLuint) * numFaces * 3);

        model->numVerts = size;

        size = 0;

        // Add the first index then start with i = 1.
        i = 0;

        model->faces[0] = 0;
        model->verts[0] = tempVerts[tempIndexes[i * indexDimension] * 3];
        model->verts[1] = tempVerts[tempIndexes[i * indexDimension] * 3 + 1];
        model->verts[2] = tempVerts[tempIndexes[i * indexDimension] * 3 + 2];
        model->UVs[0] = tempUVs[tempIndexes[i * indexDimension + 1] * 2];
        model->UVs[1] = tempUVs[tempIndexes[i * indexDimension + 1] * 2 + 1];
        size++;

        // sayVectVectFloat("tempVerts", tempVerts, numVerts, 3);

        for(i = 1 ; i < numFaces * 3; i++) {
            isUnique = 1;
            for (j = i-1; j >= 0; j-=1 ) {
                if (tempIndexes[i * indexDimension] == tempIndexes[j * indexDimension]) {
                    if(tempIndexes[i * indexDimension + 1] == tempIndexes[j * indexDimension + 1]) {
                        isUnique = 0;
                        break;
                    }
                }
            }
            if (isUnique) {
                // I should be making copyVect methods or whatever and use that.
                model->faces[i] = size;
                model->verts[size * 3] = tempVerts[tempIndexes[i * indexDimension] * 3];
                model->verts[size * 3 + 1] = tempVerts[tempIndexes[i * indexDimension]* 3 + 1];
                model->verts[size * 3 + 2] = tempVerts[tempIndexes[i * indexDimension] * 3 + 2];
                model->UVs[size * 2] = tempUVs[tempIndexes[i * indexDimension + 1] * 2];
                model->UVs[size * 2 + 1] = tempUVs[tempIndexes[i * indexDimension + 1] * 2 + 1];
                size++;
            } else {
                // Repeat, so add the repeat index to the index list.

                // Use this is parsing forwardly.
                // model->faces[i] = j;

                // Should use this with reverse parsing.
                model->faces[i] = model->faces[j];
            }
        }
    } else {

    }

    free(tempIndexes);
    free(tempUVs);
    free(tempVerts);

}

struct jalbModel *jalbMakeModel(char* filename, int tex) {
    struct jalbModel *m = jalbInitModel();
    jalbReadModel(filename, m);
    m->tex = tex;
    return m;
}

/** Camera Mats */

// Mat does need to be zeroed.
void genCamTransMatrix3rd(struct jalbCamera *cam, float *matTrans) {
    // start with a vect of {0, 0, dist}
    // Rotate it by xAngle then yAngle.

    // First get the Z and Y of the X rotation.
    // Then using the origional X and new Z get X and Z of the Y rotation.
    int i;
    for(i = 0; i < 16; i++) {
        matTrans[i] = 0;
    }
    for(i = 0; i < 4; i++) {
        matTrans[i * 4 + i] = 1;
    }

    float x;
    float y;
    float z;

    z = cos(cam->rots[0]) * cam->distance;
    y = sin(cam->rots[0]) * cam->distance;

    x = sin(cam->rots[1]) * z;
    z = cos(cam->rots[1]) * z;

    // Equals
    matTrans[0 * 4 + 3] = x;
    matTrans[1 * 4 + 3] = y;
    matTrans[2 * 4 + 3] = z;

    // Plus Equals
    matTrans[0 * 4 + 3] += -cam->followVect[0];
    matTrans[1 * 4 + 3] += -cam->followVect[1];
    matTrans[2 * 4 + 3] += -cam->followVect[2];
}

void genCamTransVect3rd(struct jalbCamera *cam, float *vect) {
    // start with a vect of {0, 0, dist}
    // Rotate it by xAngle then yAngle.

    // First get the Z and Y of the X rotation.
    // Then using the origional X and new Z get X and Z of the Y rotation.

    vect[2] = -cos(cam->rots[0]) * cam->distance;
    vect[1] = -sin(cam->rots[0]) * cam->distance;

    vect[0] = sin(cam->rots[1]) * vect[2];
    vect[2] = cos(cam->rots[1]) * vect[2];

    vect[0] += cam->followVect[0];
    vect[1] += cam->followVect[1];
    vect[2] += cam->followVect[2];
//
//    vect[0] += cam->position[0];
//    vect[1] += cam->position[1];
//    vect[2] += cam->position[2];
}

/** Camera */

// mod is what model the camera follows.
// xAng is at -30.
struct jalbCamera *initCam() {
    struct jalbCamera *cam = malloc(sizeof(struct jalbCamera));
    cam->rots[1] = 0.0;
//    cam->yAngle = M_PI/4.0;

//    cam->xAngle = -M_PI/2.0;
    cam->rots[0] = -M_PI/4.0;
    // cam->xAngle = 0.0;
    cam->rots[2] = 0.0;

    cam->distance = 10.0;

//    cam->followVect = character->position;

    return cam;
}


