#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <math.h>

#include "jalbArrayList.h"
#include "jalbTerrain.h"
/** Terrain */

struct jalbTerrainList *initTerrain(int edge, int x, int y, float scale) {
    struct jalbTerrainList *ter = malloc(sizeof(struct jalbTerrainList));

float bufferMult = 1.5;

    ter->chunkCoords[0] = x;
    ter->chunkCoords[1] = y;

    ter->numVerts = numHexVerts(edge);
    ter->numFaces = numHexFaces(edge);

    printf("numVerts, numFaces: %d, %d\n", ter->numVerts, ter->numFaces);

    ter->maxVerts = ter->numVerts * bufferMult;
    ter->maxFaces = ter->numFaces * bufferMult;

    ter->verts = calloc(ter->maxVerts, sizeof(struct vertTex));
    ter->faces = malloc(ter->maxFaces * 3 * sizeof(int));

    genHexVerts(edge, scale, ter->verts);
    genHexFace(edge, ter->faces);

    printf("sizeof(ter) :%lu\n", sizeof(*ter));
    printf("sizeof(ter->verts) :%lu\n", sizeof(*ter->verts) * ter->numVerts);
    printf("sizeof(ter->faces) :%lu\n", sizeof(*ter->faces) * ter->numFaces);

    return ter;
}

/// Indexing + Total.
// Radius is the same as the number of faces on any edge.
int numHexVerts(int radius) {
    return (radius * radius + radius) * 3 + 1;
}

int numHexFaces(int radius) {
    return radius * radius * 6;
}

int *genHexFace(int radius, int *faces) {
//    int numFaces = numHexFaces(radius);
//    int *faces = calloc(3 * numFaces, sizeof(int));

    int width;
    int faceIndex = 0;

    int y;

    int top = 0;
    int bot = radius + 1;

    // Increasing width.
    for(y = 0; y < radius ; y++) {
        width = (radius + y)* 2 + 1;
        int x;
        for(x = 0; x < width; x++) {
            if(x % 2 == 0) {
                // btb
                faces[faceIndex *3] = bot;
                bot++;
                faces[faceIndex *3 +1] = top;
                faces[faceIndex *3 +2] = bot;
            } else {
                // ttb
                faces[faceIndex *3] = top;
                top++;
                faces[faceIndex *3 +1] = top;
                faces[faceIndex *3 +2] = bot;
            }
            faceIndex++;
        }
        top++;
        bot++;
    }

    for(y = radius-1; y >=0; y--) {
        width = (radius + y)* 2 + 1;
        int x;
        for(x = 0; x < width; x++) {
            if(x % 2 == 0) {
                // ttb
                faces[faceIndex *3] = top;
                top++;
                faces[faceIndex *3 +1] = top;
                faces[faceIndex *3 +2] = bot;
            } else {
                // btb
                faces[faceIndex *3] = bot;
                bot++;
                faces[faceIndex *3 +1] = top;
                faces[faceIndex *3 +2] = bot;
            }
            faceIndex++;
        }
        top++;
        bot++;
    }
    return faces;
}

struct vertTex *genHexVerts(int radius, float scale, struct vertTex *verts) {
    // Not the most efficient loopage, but visualy the most sensable.
    // Because this should only be an init function i sacraficed effeciency for readability.

//    int numVerts = numHexVerts(radius);
//    printf("numVerts: %d\n", numVerts);
//    struct vertTex *verts = calloc(numVerts, sizeof(struct vertTex));

    int ex = 0;

    int vertIndex = 0;
    int y;
    int x;
    for(y = radius; y >= -radius; y--) {
        for (x = -y -ex; x <= ex; x++) {
            float wx = 0;
            float wz = 0;
            terToWorld(y, x, &wz, &wx, scale);

            verts[vertIndex].loc[0] = wx;
            verts[vertIndex].loc[1] = 0.0;
            verts[vertIndex].loc[2] = wz;

//            printf("world (%f, %f)\n", wx, wz);
            vertIndex++;
        }
        if(y > 0) {
            // Decreasing.
            ex++;
        }
    }
    return verts;
}

void worldToTer(float wx, float wz, int *terX, int *terZ, float scale) {
    int negx = wx < 0 ? -1 : 1;
//    int negz = wz < 0 ? -1 : 1;
//    x *= negx;
//    z *= negz;

    // Denominator to make grid.
    float dx = scale / 2;
    float dz = cos(M_PI/6) * scale;

    // Grid values
    int gx = (wx / dx);
    int gz =(int) (wz / dz);

    // remainder
    wx -= dx * gx;
    wz -= dz * gz;

    // scalar
    wx /= dx;
    wz /= dz;

    if(wz < 0) {
        gz--;
    }

    // Get the abs of scalar x.
    wx *= negx;
    // Add 1 to scalar z if negative.
    if(wz < 0) {
        wz += 1;
    }

    int tx = (gx + 1 * negx) / 3;
    int tz = 0;

    gz -= tx;
    if(gz > 0 ) {
        tz = (gz + 1) / 2;
    } else {
        tz = gz / 2;
    }

    // If gx < 0 then do the opposite.
    // if (gx > then do this) [ != (gx < 0) ]
    if(abs(gx) % 3 == 1) {
        // z is even or odd
        if( gz % 2 == 0) {
            // Top left to bottom right.
            if(wx > 1 - wz) {
                // Checks to see if the x is negative.
                if(gx > 0) {
                    tx++;
                } else {
                    tx --;
                    tz++;
                }
            }
        } else {
            if(wx > wz) {
                if(gx > 0) {
                    tx++;
                    tz--;
                } else {
                    tx--;
                }
            }
        }
    }

    /// Returned values set.
    *terX = tx;
    *terZ = tz;
}

// Terrain coords to world coords. Having a method would be annoying because i need to return an x and y..
// Why do we have return values, why are return values only primative data.
void terToWorld(int tx, int ty, float *wx, float *wy, float scale) {
//    printf("terToWorld (%d, %d)\n", tx, ty);
// 12 * pow(3, .5) = that 20.7 number.
    *wx += tx * cos(M_PI * 30 / 180) * scale;
    *wy += tx * sin(M_PI * 30 / 180) * scale;
    *wy += 2 * ty * sin(M_PI * 30 / 180) * scale;
}

// world loc doesnt need to be zeroed
void terToWorldNew(int tx, int ty, float *wx, float *wy, float scale) {
    *wx = tx * 3 * cos(M_PI/3) * scale;
    *wy = tx * sin(M_PI/3) * scale;
    *wy += ty * 2 * sin(M_PI/3) * scale;
}

/** World */
// Returns 0 if unsuccessful, or numChunks if successful.
//int loadWorld(char *dir, ArrayList *chunkList) {
//    FILE *file;
//    file = fopen(dir, "r");
//    if(file == NULL ) {
//        printf("error opening file\n");
//        return 0;
//    }
//
//    int numChunks;
//    fread(&numChunks, sizeof(int), 1, file);
//
//    int i;
//    for(i = 0; i < numChunks; i++) {
//        struct jalbTerrain *chunk = malloc(sizeof(struct jalbTerrain));
//        fread(&chunk->numVerts, sizeof(int), 1, file);
//        fread(&chunk->numFaces, sizeof(int), 1, file);
//        fread(&chunk->chunkCoords[0], sizeof(int), 1, file);
//        fread(&chunk->chunkCoords[1], sizeof(int), 1, file);
//        fread(&chunk->verts, sizeof(float), chunk->numVerts * 3, file);
//        fread(&chunk->faces, sizeof(int), chunk->numFaces * 3, file);
//    }
//    return numChunks;
//}

// Embed
//void embedSpawnTer(ArrayList *terList, int radius, int coX, int coY, float scale, int edge) {
//    int i = 0;
//    int ex = 0;
//    int x = radius - 1;
//    int y;
//    for(; x > -radius; x--) {
//        for (y = -x -ex; y <= ex; y++) {
////            printf("action (%d, %d)\n", x, y);
//            /// Action, have (x, y)
//            checkAndSpawn(terList, x + coX, y + coY, scale, edge);
//            i++;
//        }
//        if(x > 0) {
//            ex++;
//        }
//    }
//}

//void checkAndSpawn(ArrayList *terList, int coX, int coY, float scale, int edge) {
//    // Iterate the terList to see if the chunk is spawned, if not then spawn one.
//    int numTer = arrayListGetLength(terList);
//    int i;
//    for(i = 0; i < numTer; i++) {
//        struct jalbTerrain *ter = arrayListGetPointer(terList, i);
//        if(ter->chunkCoords[0] == coX && ter->chunkCoords[1] == coY) {
//            return;
//        }
//    }
//    // There has not been a conflict yet, so spawn a ter. This means i need to know scale, and edge.
//    struct jalbTerrain *ter = initTerrain(edge, coX, coY, scale/edge);
//    arrayListAddEndPointer(terList, ter);
//    printf("ter: %p\n", ter);
//    printf("ter->faces[0]: %d\n", ter->faces[0]);
//}

int terRadius(int x, int y) {
//    printf("terRadius(%d, %d)\n", x, y);
    if(x > 0) {
        x = -x;
        y = -y;
    }
    if(y < 0) {
        return -x - y;
    } else if (y > -x) {
        return y;
    } else {
        return -x;
    }
}


void sayTerrain(struct jalbTerrainList *ter) {
//     GLuint      numVerts;
//    GLuint      numFaces;
//
//    int         x;
//    int         y;
//
//    GLuint      *faces;
//    GLfloat     *verts;
//    GLfloat     *UVs;
    printf("{%d\t%d\t%d\t%d}\n", ter->numVerts, ter->numFaces, ter->chunkCoords[0], ter->chunkCoords[1]);
    int i;
    printf("ter->verts {");
    for(i = 0; i < ter->numVerts; i++) {
        printf("( %f, %f, %f )[%d], ", ter->verts[i].loc[0], ter->verts[i].loc[1], ter->verts[i].loc[2], ter->verts[i].texi);
    }
    printf("}\n");

    printf("myTer[0]->faces {");
    for(i = 0; i < ter->numFaces * 3; i++) {
        printf("%d, ", ter->faces[i]);
    }
    printf("}\n");

//    printf("myTer->texis {");
//    for(i = 0; i < ter->numVerts; i++) {
//        printf("%d, ", ter->texis[i]);
//    }
//    printf("}\n");
}

void freeTerrain(struct jalbTerrainList *ter) {
    printf("freeTer: %p\n", ter);
    if(ter == NULL) {
        return;
    }

    printf("free: %p\n", ter->verts);
    free(ter->verts);
    free(ter->faces);

    free(ter);
}
