#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "jama.h"

/** Matrix Math */

// Sets mat1 to mat1 * mat2;
void matMatMult(float *mat1, float *mat2) {
    float temp[4];
    int i = 0;
    int j = 0;
    for(; i < 4; i++) {
        for(j = 0; j < 4; j++) {
            temp[j] = mat1[i * 4 + 0] * mat2[0 + j] + mat1[i * 4 + 1] * mat2[4 + j] + mat1[i * 4 + 2] * mat2[8 + j] + mat1[i * 4 + 3] * mat2[12 + j];
        }
        for(j = 0; j < 4; j++) {
            mat1[i * 4 + j] = temp[j];
        }
    }
}

/** Rotation Matrices */
// Doesnt need to be zeroed.
void xRotMatrix4(float *mat, float thea) {
    mat[0] = 1;
    mat[1] = 0;
    mat[2] = 0;
    mat[3] = 0;

    mat[1 * 4 + 0] = 0;
    mat[1 * 4 + 1] = cos(thea);
    mat[1 * 4 + 2] = sin(thea);
    mat[1 * 4 + 3] = 0;

    mat[2 * 4 + 0] = 0;
    mat[2 * 4 + 1] = -sin(thea);
    mat[2 * 4 + 2] = cos(thea);
    mat[2 * 4 + 3] = 0;

    mat[3 * 4 + 0] = 0;
    mat[3 * 4 + 1] = 0;
    mat[3 * 4 + 2] = 0;
    mat[3 * 4 + 3] = 1;
}

void yRotMatrix4(float *mat, float thea) {
    mat[0 * 4 + 0] = cos(thea);
    mat[0 * 4 + 1] = 0;
    mat[0 * 4 + 2] = -sin(thea);
    mat[0 * 4 + 3] = 0;

    mat[1 * 4 + 0] = 0;
    mat[1 * 4 + 1] = 1;
    mat[1 * 4 + 2] = 0;
    mat[1 * 4 + 3] = 0;

    mat[2 * 4 + 0] = sin(thea);
    mat[2 * 4 + 1] = 0;
    mat[2 * 4 + 2] = cos(thea);
    mat[2 * 4 + 3] = 0;

    mat[3 * 4 + 0] = 0;
    mat[3 * 4 + 1] = 0;
    mat[3 * 4 + 2] = 0;
    mat[3 * 4 + 3] = 1;
    //return mat;
}

void zRotMatrix4(float *mat, float thea) {
    mat[0 * 4 + 0] = cos(thea);
    mat[0 * 4 + 1] = sin(thea);
    mat[0 * 4 + 2] = 0;
    mat[0 * 4 + 3] = 0;

    mat[1 * 4 + 0] = -sin(thea);
    mat[1 * 4 + 1] = cos(thea);
    mat[1 * 4 + 2] = 0;
    mat[1 * 4 + 3] = 0;

    mat[2 * 4 + 0] = 0;
    mat[2 * 4 + 1] = 0;
    mat[2 * 4 + 2] = 1;
    mat[2 * 4 + 3] = 0;

    mat[3 * 4 + 0] = 0;
    mat[3 * 4 + 1] = 0;
    mat[3 * 4 + 2] = 0;
    mat[3 * 4 + 3] = 1;
    //return mat;
}

void matVectMult(float *mat, float *vect, float *newVect) {
    float temp[4];
    int i = 0;
    for(; i < 4; i++) {
        temp[i] = mat[i * 4 + 0] * vect[0] + mat[i * 4 + 1] * vect[1] + mat[i * 4 + 2] * vect[2] + mat[i * 4 + 3] * vect[3];
    }
    for(i = 0; i < 4; i++) {
        newVect[i] = temp[i];
    }
}

void matVect3Mult(float *mat, float *vect, float *newVect) {
    float temp[4];
    int i = 0;
    for(; i < 3; i++) {
        temp[i] = mat[i * 4 + 0] * vect[0] + mat[i * 4 + 1] * vect[1] + mat[i * 4 + 2] * vect[2];
    }
    for(i = 0; i < 3; i++) {
        newVect[i] = temp[i];
    }
}

/** Vector math */

void vectNormalize(float *vec, int length) {
    float sum = 0;
    int i;
    for(i = 0; i < length; i++) {
        sum += vec[i] * vec[i];
    }
    if(sum == 0) {
        return;
    }
    sum = pow(sum, .5);
    for(i = 0; i < length; i++) {
        vec[i] /= sum;
    }
}

/** Saying */

void sayMat4(char *str, float *mat) {
    printf("%s {\n", str);
    int i = 0;
    int j;
    for(; i < 4; i++) {
        for(j = 0; j < 4; j++) {
            printf("%f", mat[i * 4 + j]);
            if(j < 3) {
                printf(", ");
            }
        }
        if(i < 3) {
            printf("},\n");
        }
    }
    printf("}\n");
}

void sayVect(char *str, float *vect, int length) {
    printf("%s (", str);
    int i = 0;
    for(; i < length; i++) {
        printf("%f, ", vect[i]);
    }
    printf("}\n");
}

