#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <SDL.h>
#include <SDL_image.h>
#include "SDL_ttf.h"

#include <glew.h>
#include <SDL_opengl.h>
#include <glu.h>
#include <SOIL.h>

#include <ft2build.h>
#include FT_FREETYPE_H

// Sleep
#include <unistd.h>

#include <netinet/in.h>
#include <arpa/inet.h>

#include "jalbArrayList.h"
#include "jalbScrollList.h"
#include "jalbButtonArray.h"

#include "jalbClient.h"

#include "jalbAnimMap.h"

#include "jalbTerrain.h"
#include "jalbEntity.h"
#include "jalbModel.h"

#include "jama.h"

#include "reader.h"

#define BUFSIZE 12000
#define KEYSIZE 16

/** Functions */
int init();
void initUI();

void loadDefaultKeyMap(SDL_Keycode *keyMaps);
void loadDefaultServer(ArrayList *servList);

int loadCubemap();
void loadSkyBoxArrays();

void closingUI();

void gLoop();
void inputEvents(SDL_Event e);
void inputEventsGame(SDL_Event e);
void handleKeyDown(SDL_Keycode c);
void handleMouse(int x, int y);

void tic();

void closing();

/** Game Function */
void gameTic();

/** Ticly Networking */
void ticlyIn(int nread);
void ticlyOut();

void drawGameUI();

void makeProjMatrix(struct jalbCamera *cam);

/** Networking In */
void recvEntsTicly();
void recvTerTicly(unsigned char *buf);

void renderUI();

void stageUpdateText(char *str);

void keyPress(char c);
void specialKeys(int key);
void editCharArray(char* str, int maxLength, char c, int cursorIndex);
void backSpaceCharArray(char* str, int cursorIndex);

char *stageToStr();

// Loading
SDL_Texture* loadTexture( char *path, int *retWidth, int *retHeight);

void saveServs(char *dir, ArrayList *servList);
int loadServs(char *dir, ArrayList *servList);
void saveServ(struct jalbServer *ser, FILE *file);
void loadServ(struct jalbServer *ser, FILE *file);

void saveKeyMap(char *dir);
int loadKeyMap(char *dir);

// Init Ogl.
void initGlew();
void initGL();
void initShaders();
void initUniformLocations();
void initTextures();
GLuint loadShaders(char * vertex_file_path, char * fragment_file_path);
void oldToNew();

/** Rendering */
void renderTerList(struct jalbTerrainList *ter, struct jalbCamera *cam, int *chunkCoords);
void renderEntity(struct jalbEntity *ent, struct jalbCamera *cam);
void render2d(int x, int y, int width, int height, GLuint tex);

void gameRenderString(char *str, int x, int y);
void renderRect(int x, int y, int width, int height, float r, float g, float b, float a);
void saveBmp(char *dir, int width, int height, unsigned char *buffer);

void drawWindow(int x, int y, int width, int height);
void makeClickVect(int x, int y, float *ret);

/** Variables */

//Screen dimension constants
const int SCREEN_WIDTH = 1240;
const int SCREEN_HEIGHT = 800;

//The window we'll be rendering to
SDL_Window* gWindow = NULL;

// Sdl rendering.
SDL_Renderer *gRenderer = NULL;
// Ogl rendering.
SDL_GLContext gContext = NULL;

/** Vars for gLoop Below */
int ticksPerSecond = 20;
int mpt;
unsigned int lastSec = 0;
unsigned int nextTick = 0;
unsigned int ticksThisSecond = 0;
int running = 0;
/** Vars for gLoop Above */

/** TTF Vars*/
TTF_Font *font;
int textH = 20;
int textW = 10;

SDL_Color color = {0, 0, 0, 1};
SDL_Color backColor = {255, 255, 255, 1};
SDL_Color colorRed = {159, 15, 15, 1};
SDL_Color colorOrng = {255, 127, 00, 1};
SDL_Color colorGre = {15, 200, 15, 1};

SDL_Texture *theme;
int themeW, themeH;

int edgeW = 4;

// Input vars
int mouseDownLeft = 0;
int mouseDownRight = 0;
int mouseDownLeftOld = 0;
int mouseDownRightOld = 0;
int mouseLoc[2];

struct scrollList *sl;
struct buttonArray *mainBa;
struct buttonArray *editBa0;
struct buttonArray *editBa1;
struct buttonArray *editBa2;

struct buttonArray *optionBaKeys;
struct buttonArray *optionBa1;

int servSelected = -1;

int txtBoxSelected;
int cursorIndex;
int cursorW = 3;
int cursorH = 18;

int stage = csMain;

int optionMaxYscroll;
//int optionCurYscroll;

int scrollBarPressed = 0;

struct scrollBar *optionSB;

ArrayList *servList;

char *dirKeyMap = "res/keyMap";
char *dirServList = "res/servList";

// Networking
int sfd = -1;

unsigned int lastRecv;

int returnWaitTime = 2000;
int returnFailMax = 3;

int failCount = 0;

GLfloat projMatrix[16];

enum keyMap {
    kmReturn = 0,
    //Text Editing
    kmScrollLeft,
    kmScrollRight,
    kmScrollUp,
    kmScrollDown,
    kmHome,
    kmEnd,
    kmBackSpace,
    kmDelete,
    kmNextBox,
    kmShift,

    // Ogl Controls
    kmWalkForward, // First index to be part of gameInputs. + Direct map to gameInputs.
    kmWalkBackwards,
    kmStrafeLeft,
    kmStrafeRight,
    kmRotateleft,
    kmRotateright,
    kmJump,

    kmHotBar0,
    kmHotBar1,
    kmHotBar2,
    kmHotBar3,
    kmHotBar4,
    kmHotBar5,

    numKeyMaps,
};

int numGameInputs = numKeyMaps - kmWalkForward;
char allKeyInputs[numKeyMaps]; // Booleans for weather the key is being pressed or not.
char gameInputs[numKeyMaps - kmWalkForward]; // 0 unchanged, 1 pressed, 2 released (this turn).
// I can either keep track of the changes during the pressed part.
// Or i can save the old and when sending compare the 2.

int keysPerInput = 2;
SDL_Keycode *keyMaps;

struct jalbAnimMap *loadingMap;

int bufIndex = 0;
unsigned char buf[BUFSIZE];

// Max is server chunkRadius-1 (10)
int chunkRadius = 10;

unsigned char key[16];

// ogl vars.
ArrayList *terList;
ArrayList *entList;
ArrayList *modList;
ArrayList *minList;

/** OpenGL Vars */
// ProgramIDs
GLuint skyBoxProgramID = 0;
GLuint statModelProgramID = 0; // Static model
GLuint animModelProgramID = 0; // Animated model
GLuint flatImgProgramId = 0;
GLuint singleColProgId = 0;
GLuint terProgramID = 0;

// buffers
GLuint vertexBuffer;
GLuint vertBuf;
GLuint indsBuf;
GLuint indexBuffer;
GLuint uvBuffer0;
GLuint uvBuffer1;
GLuint uvBuffer2;
GLuint vbo_triangle_colors;

// Uniform IDs
GLuint skyBoxMatID;
GLuint skyBoxTexID;

GLuint statModelMatID;
GLuint statModelTexID;

GLuint animModelMatID;
GLuint animModelBoneID;
GLuint animModelTexID;
GLuint animModelIndexID;
GLuint animModelWeightID;

GLuint terMatID;
GLuint terTexID;

GLuint ter1MatID;
GLuint ter1TexID;

GLuint singleColUniformId;

GLuint flatImgTexID;

GLuint SSBOMatID;
GLuint SSBOUvID;
GLuint SSBOTexID;


GLuint ssboVerts = 0;
GLuint ssboIndex = 0;

// Textures
GLuint terAtlas = 0;
GLuint demonTex = 0;
GLuint treeTex = 0;
GLuint wallTex = 0;
GLuint portrait0 = 0;
GLuint hotbar0 = 0;
GLuint map0 = 0;

GLuint cubemapTexture = 0;

GLuint skyboxVAO = 0;
GLuint skyboxVBO = 0;

int near = 4;
int far = 400;

struct jalbCamera *myCam;

float terrainScale = 20;
int entId = -1;
GLfloat pos[3] = {.05, .3, 0};

int myEntChunkCoordsOld[2];

/// Client Game UI Vars
int iconWidth = 64;
// Hotbar
int hotBarLoc[2] = {0, 0};
int hotBarDraw[2] = {709, 60};
int hotBarDim[2] = {709, 60};
int hotBarBoarderDim[2] = {3, 3};
int hotBarGap = 1;

int portraitLoc[2] = {0, 0};
int portraitDraw[2] = {256, 128};
int portraitDim[2] = {256, 128};
int healthOffset[2] = {99, 76};
int manaOffset[2] = {0, 23};
int barDim[2] = {150, 20};

int nameOffset[2] = {97, 18};

int mapLoc[2] = {0, 0};
int mapDraw[2] = {256, 256};
int mapDim[2] = {256, 256};

int windowMinWidth = 360;
int windowMinHeight = 420;

// Free Type Text
FT_Library  ftLib;
FT_Face ftFace;

/// Other

enum entStats {
    health = 0,
    strength,
    dexterity,
    agility,
    endurance,
    arcane,
    numStats,
};

int playerStats[numStats * 2];

int main(int argc, char *args[]) {
    if(!init()) {
        printf("Init Failed\n");
    } else {
//        SSBOtest3();
        gLoop();
        if(stage == csGame) {
            sendDisconnect(buf, sfd, key);
        }
        closing();
    }

    return 0;
}

/** Inits */

int init() {
    //Initialize SDL
    if( SDL_Init( SDL_INIT_VIDEO ) < 0 ) {
        printf( "SDL could not initialize! SDL Error: %s\n", SDL_GetError() );
        return 0;
    }
    // Is this only necessary for 3d rendering?
    //Use OpenGL 3.1 core
    SDL_GL_SetAttribute( SDL_GL_CONTEXT_MAJOR_VERSION, 4 );
    SDL_GL_SetAttribute( SDL_GL_CONTEXT_MINOR_VERSION, 1 );
    SDL_GL_SetAttribute( SDL_GL_CONTEXT_PROFILE_MASK, SDL_GL_CONTEXT_PROFILE_CORE );

    gWindow = SDL_CreateWindow( "Footh Incremental: Client 10", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, SCREEN_WIDTH, SCREEN_HEIGHT, SDL_WINDOW_OPENGL |SDL_WINDOW_SHOWN );

    if( gWindow == NULL ) {
        printf( "Window could not be created! SDL Error: %s\n", SDL_GetError() );
        return 0;
    }

    keyMaps = calloc(numKeyMaps * 2, sizeof(SDL_Keycode));

    int success = loadKeyMap(dirKeyMap);
    if(!success) {
        loadDefaultKeyMap(keyMaps);
    }

    initUI();

    return 1;
}

void initUI() {
    printf("initUI()\n");

    // SDL_SetWindowFullscreen(gWindow, SDL_WINDOW_FULLSCREEN);

    gRenderer = SDL_CreateRenderer(gWindow, 0, SDL_RENDERER_ACCELERATED);

    /// Init TTF.
    if(TTF_Init()==-1) {
        printf("TTF_Init: %s\n", TTF_GetError());
        exit(2);
    }

    font = TTF_OpenFont("res/UbuntuMono-R.ttf", textH);

    TTF_SizeText(font,"a",&textW, NULL);

    if(!font) {
        printf("TTF_OpenFont: %s\n", TTF_GetError());
        // handle error
    }

    // Init theme image.
    theme = loadTexture("res/Footh.png", &themeW, &themeH);

    // init server list.
    int listW = SCREEN_WIDTH * 70 / 100;
    int listX = (SCREEN_WIDTH - listW) / 2;
    int listY = 10 + themeH + 40;

    sl = initScrollList(listX, listY, listW, 200, textW, textH, 4, 2, 1);

    servList = initArrayList(10, sizeof(void*));

    int success = loadServs(dirServList, servList);

    if(!success) {
        loadDefaultServer(servList);
    }

    // Button Array
    int buttHeight = (edgeW + textH) * 2;
    mainBa = initButtonArray(listX, 0, listW, textW, textH, buttHeight, edgeW, 2, 2);
    mainBa->y = SCREEN_HEIGHT - 40 - buttHeight * 2 - edgeW * 3;

    /// Finally set scrollList height.
    sl->height = mainBa->y - sl->y - 10;

    // Adds to loaded servers to the scrollList.
    int length = arrayListGetLength(servList);
    int i;
    for(i = 0; i < length; i++) {
        struct jalbServer *ser = arrayListGetPointer(servList, i);

        scrollListAddButt(sl);

        scrollListSetText(sl, ser->name, i, 0, &color, gRenderer, font);
        scrollListSetText(sl, ser->address, i, 1, &color, gRenderer, font);
    }

    // Finish filling out buttArray.
    buttArSetText(mainBa, gRenderer, "Log In", 0, 0, &color, font);
    buttArSetText(mainBa, gRenderer, "Options", 1, 0, &color, font);
    buttArSetText(mainBa, gRenderer, "Add", 0, 1, &color, font);
    buttArSetText(mainBa, gRenderer, "Quit", 1, 1, &color, font);

    // Init Button Arrays for the server edit screen.
    editBa0 = initButtonArray(listX, 0, listW, textW, textH, buttHeight, edgeW, 1, 2);
    editBa1 = initButtonArray(listX, 0, listW, textW, textH, buttHeight, edgeW, 2, 2);
    editBa2 = initButtonArray(listX, 0, listW, textW, textH, buttHeight, edgeW, 1, 1);

    editBa0->y = themeH + 10 + 40;
    editBa1->y = editBa0->y + (buttHeight + edgeW) * 2 + edgeW;
    editBa2->y = editBa1->y + (buttHeight + edgeW) * 2 + edgeW;

    buttArSetText(editBa1, gRenderer, "Create Account", 0, 1, &color, font);
    buttArSetText(editBa1, gRenderer, "Delete", 1, 1, &color, font);

    buttArSetText(editBa2, gRenderer, "Done", 0, 0, &color, font);

    /// Options screen.

    listX = (SCREEN_WIDTH - 600) / 2;
    optionBaKeys = initButtonArray(listX, listY, 600, textW, textH, buttHeight, edgeW, 3, numKeyMaps);
    int ik = 0;
    buttArSetText(optionBaKeys, gRenderer, "Return", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Scroll Left", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Scroll Right", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Scroll Up", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Scroll Down", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Home", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "End", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Backspace", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Delete", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Next Box", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Shift", 0, ik, &color, font);

    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Walk Forward", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Walk Backwards", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Strafe Left", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Strafe Right", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Rotate Left", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Rotate Right", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Jump", 0, ik, &color, font);

    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Hot-Bar: 0", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Hot-Bar: 1", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Hot-Bar: 2", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Hot-Bar: 3", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Hot-Bar: 4", 0, ik, &color, font);
    ik++;
    buttArSetText(optionBaKeys, gRenderer, "Hot-Bar: 5", 0, ik, &color, font);

    for(i = 0; i < numKeyMaps * 2; i++) {
        const char *keyName = SDL_GetKeyName(keyMaps[i]);
        int row = i / 2;
        int col = i - row * 2 + 1;
        buttArSetText(optionBaKeys, gRenderer, (char*) keyName, col, row, &color, font);
    }

    listY += numKeyMaps * (buttHeight + edgeW) + edgeW;

    // This includes the edgeWs on each side, because they get removed later.
    int buttonWidth = (600 - mainBa->edgeW * 4) / 3 + mainBa->edgeW * 2;

    listX = (SCREEN_WIDTH - buttonWidth) / 2;

    // What width to go with?
    optionBa1 = initButtonArray(listX, listY, buttonWidth, textW, textH, buttHeight, edgeW, 1, 1);
    buttArSetText(optionBa1, gRenderer, "Done", 0, 0, &color, font);

    // theme
    int optionScreenHeight = 10 + themeH + 40;
    // key butons.
    optionScreenHeight += (buttHeight + edgeW) * numKeyMaps + edgeW;
    // return button.
    optionScreenHeight += (buttHeight + edgeW) + edgeW;
    // Bottom space.
    optionScreenHeight += 40;
    optionMaxYscroll = optionScreenHeight - SCREEN_HEIGHT;

    printf("screen_height: %d\n", SCREEN_HEIGHT);
    printf("optionScreenHeight: %d\n", optionScreenHeight);

    optionSB = initScrollBar(SCREEN_HEIGHT, optionScreenHeight, SCREEN_WIDTH - 10, 0);

    // init loading animation.
    int animW;
    int animH;
    SDL_Texture *tex = loadTexture("res/LoadMap0.png", &animW, &animH);
    loadingMap = initAnimMap(tex, animW, animH, 3, 3, 8);
}

// Id like this to be scripted. Not exactly sure why.
void loadDefaultKeyMap(SDL_Keycode *keyMaps){
        int ik = 0;
        keyMaps[ik * 2] = SDLK_ESCAPE;
        ik++;
        keyMaps[ik * 2] = SDLK_LEFT;
        ik++;
        keyMaps[ik * 2] = SDLK_RIGHT;
        ik++;
        keyMaps[ik * 2] = SDLK_UP;
        ik++;
        keyMaps[ik * 2] = SDLK_DOWN;
        ik++;
        keyMaps[ik * 2] = SDLK_HOME;
        ik++;
        keyMaps[ik * 2] = SDLK_END;
        ik++;
        keyMaps[ik * 2] = SDLK_BACKSPACE;
        ik++;
        keyMaps[ik * 2] = SDLK_DELETE;
        ik++;
        keyMaps[ik * 2] = SDLK_TAB;
        ik++;

        keyMaps[ik * 2] = SDLK_LSHIFT;
        keyMaps[ik * 2 + 1] = SDLK_RSHIFT;
        ik++;

        keyMaps[ik * 2] = SDLK_w;
        ik++;
        keyMaps[ik * 2] = SDLK_s;
        ik++;
        keyMaps[ik * 2] = SDLK_q;
        ik++;
        keyMaps[ik * 2] = SDLK_e;
        ik++;
        keyMaps[ik * 2] = SDLK_a;
        ik++;

        keyMaps[ik * 2] = SDLK_d;
        ik++;
        keyMaps[ik * 2] = SDLK_SPACE;

        ik++;
        keyMaps[ik * 2] = SDLK_1;
        ik++;
        keyMaps[ik * 2] = SDLK_2;
        ik++;
        keyMaps[ik * 2] = SDLK_3;
        ik++;
        keyMaps[ik * 2] = SDLK_4;
        ik++;
        keyMaps[ik * 2] = SDLK_5;
        ik++;
        keyMaps[ik * 2] = SDLK_6;
}

void loadDefaultServer(ArrayList *servList) {
    struct jalbServer *ser = malloc(sizeof(struct jalbServer));
    ser->name[0] = 'a';
    ser->name[1] = '\0';

    ser->address[0] = 'a';
    ser->address[1] = '\0';

    ser->username[0] = '\0';
    ser->password[0] = '\0';

    arrayListAddEndPointer(servList, ser);

    ser = malloc(sizeof(struct jalbServer));
    ser->name[0] = 'a';
    ser->name[1] = 'b';
    ser->name[2] = 'c';
    ser->name[3] = '\0';

    ser->address[0] = 'd';
    ser->address[1] = 'e';
    ser->address[2] = 'f';
    ser->address[3] = '\0';

    ser->username[0] = '\0';
    ser->password[0] = '\0';

    arrayListAddEndPointer(servList, ser);
}

// Log in has been confirmed, so free UI shit and init oGL shit.
// Also init my lists/ structs.
void initGame() {
    terList = initArrayList(10, sizeof(void*));
    entList = initArrayList(10, sizeof(void*));
    modList = initArrayList(10, sizeof(void*));
    minList = initArrayList(10, sizeof(void*));

    // Set Game UI locs.
    // See port and map can fit with hotbar centered.
    int sideSpace = (SCREEN_WIDTH - hotBarDraw[0]) / 2;
    if(sideSpace >= portraitDraw[0] && sideSpace >= mapDraw[0]) {
        // Can fit all on bot.
        hotBarLoc[0] = (SCREEN_WIDTH - hotBarDraw[0]) / 2;
        hotBarLoc[1] = SCREEN_HEIGHT - hotBarDraw[1];

        portraitLoc[0] = 0;
        portraitLoc[1] = SCREEN_HEIGHT - portraitDraw[1];

        mapLoc[0] = SCREEN_WIDTH - mapDraw[0];
        mapLoc[1] = SCREEN_HEIGHT - mapDraw[1];
    } else {
        hotBarLoc[0] = (SCREEN_WIDTH - hotBarDraw[0]) / 2;
        hotBarLoc[1] = SCREEN_HEIGHT - hotBarDraw[1];

        portraitLoc[0] = 0;
        portraitLoc[1] = 0;

        mapLoc[0] = SCREEN_WIDTH - mapDraw[0];
        mapLoc[1] = 0;
    }
}

int loadCubemap() {
    char *faces[] = {"res/texture/skyBox1/Right.png", "res/texture/skyBox1/Left.png", "res/texture/skyBox1/Top.png","res/texture/skyBox1/Bottom.png" , "res/texture/skyBox1/Front.png", "res/texture/skyBox1/Back.png"};

    GLuint textureID;
    glGenTextures(1, &textureID);
    glActiveTexture(GL_TEXTURE0);

    int width,height;
    unsigned char* image;

    glBindTexture(GL_TEXTURE_CUBE_MAP, textureID);
    int i;
    for(i = 0; i < 6; i++) {
        image = SOIL_load_image(faces[i], &width, &height, 0, SOIL_LOAD_RGB);
        glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X + i, 0, GL_RGB, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE, image);
        SOIL_free_image_data(image);
    }
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
    glBindTexture(GL_TEXTURE_CUBE_MAP, 0);

    return textureID;
}

void loadSkyBoxArrays() {
    GLfloat skyboxVertices[] = {
        // Positions
        -1.0f,  1.0f, -1.0f,
        -1.0f, -1.0f, -1.0f,
        1.0f, -1.0f, -1.0f,
        1.0f, -1.0f, -1.0f,
        1.0f,  1.0f, -1.0f,
        -1.0f,  1.0f, -1.0f,

        -1.0f, -1.0f,  1.0f,
        -1.0f, -1.0f, -1.0f,
        -1.0f,  1.0f, -1.0f,
        -1.0f,  1.0f, -1.0f,
        -1.0f,  1.0f,  1.0f,
        -1.0f, -1.0f,  1.0f,

        1.0f, -1.0f, -1.0f,
        1.0f, -1.0f,  1.0f,
        1.0f,  1.0f,  1.0f,
        1.0f,  1.0f,  1.0f,
        1.0f,  1.0f, -1.0f,
        1.0f, -1.0f, -1.0f,

        -1.0f, -1.0f,  1.0f,
        -1.0f,  1.0f,  1.0f,
        1.0f,  1.0f,  1.0f,
        1.0f,  1.0f,  1.0f,
        1.0f, -1.0f,  1.0f,
        -1.0f, -1.0f,  1.0f,

        -1.0f,  1.0f, -1.0f,
        1.0f,  1.0f, -1.0f,
        1.0f,  1.0f,  1.0f,
        1.0f,  1.0f,  1.0f,
        -1.0f,  1.0f,  1.0f,
        -1.0f,  1.0f, -1.0f,

        -1.0f, -1.0f, -1.0f,
        -1.0f, -1.0f,  1.0f,
        1.0f, -1.0f, -1.0f,
        1.0f, -1.0f, -1.0f,
        -1.0f, -1.0f,  1.0f,
        1.0f, -1.0f,  1.0f
    };
    // Setup skybox VAO
    glGenVertexArrays(1, &skyboxVAO);
    glGenBuffers(1, &skyboxVBO);
    glBindVertexArray(skyboxVAO);
    glBindBuffer(GL_ARRAY_BUFFER, skyboxVBO);
    glBufferData(GL_ARRAY_BUFFER, sizeof(skyboxVertices), &skyboxVertices, GL_STATIC_DRAW);
    glEnableVertexAttribArray(0);
    glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(GLfloat), (GLvoid*)0);
    glBindVertexArray(0);
}

/** Loop + Tic */
void gLoop() {
//    shaderTest0();
    //Main loop flag
    mpt = 1000 / ticksPerSecond;
    running = 1;

    //Event handler
    SDL_Event e;

    //Enable text input
    SDL_StartTextInput();

    //While application is running
    while( running ) {
        if(SDL_GetTicks() >= nextTick) {
            ticksThisSecond++;
            nextTick += mpt;

            if (SDL_GetTicks() - lastSec > 1000) {
                if (ticksThisSecond < ticksPerSecond - 1 || ticksThisSecond > ticksPerSecond + 1) {
                    if (SDL_GetTicks() > 2000) {
                        printf("Ticks this second: %u\n", ticksThisSecond);
                        printf("SDL_GetTicks(): %u\n", SDL_GetTicks());
                        printf("nextTick: %u\n", nextTick);
                    }
                }

                ticksThisSecond = 0;
                lastSec = (SDL_GetTicks());
            }

            if(stage == csGame) {
                inputEventsGame(e);
                gameTic();
            } else {
                inputEvents(e);
                tic();
            }

            //Update screen
            SDL_GL_SwapWindow( gWindow );
        }
    }

    //Disable text input
    SDL_StopTextInput();
}

// Stage is not game
void tic() {
    int msg = 0;
//    printf("tic\n");
    // Render
    renderUI();

    // Connection check.
    /// Data in (init).
    if(stage >= csWakeupCreate && stage < csGame) {
        int nread;
        while((nread = read(sfd, buf, BUFSIZE)) > 0) {
            lastRecv = SDL_GetTicks();
            bufIndex = 0;
            // Dont need while loop because everything either resends or disconnects that player.
//            while(bufIndex < nread) {
                msg++;
                if(msg > 10) {
                    exit(0);
                }
                printf("nread[%d]: %x\tlength: %d\n", bufIndex, buf[bufIndex], nread);
                // stage 3 or 3
                if(buf[bufIndex] == soWakeup) {
                    bufIndex += 1;

                    scrollListSetText(sl, "Connection: Success", servSelected, 2, &colorGre, gRenderer, font);
                    if(stage == csWakeupCreate) {
                        // Send Create Account.
                        struct jalbServer *ser = arrayListGetPointer(servList, servSelected);
                        sendCreateAccount(ser, sfd, buf);
                        scrollListSetText(sl, "Account Creation: Pending", servSelected, 3, &colorOrng, gRenderer, font);
                    } else if(stage == csWakeupLogin) {
                        // Send Login.
                        struct jalbServer *ser = arrayListGetPointer(servList, servSelected);
                        sendLogIn(ser, sfd, buf, chunkRadius);
                        scrollListSetText(sl, "Log In: Pending", servSelected, 3, &colorOrng, gRenderer, font);
                    }
                    stage = csWoke;
                } else if(buf[bufIndex] == soNewGood) {
                    bufIndex += 1;
                    if(stage == csWoke) {
                        scrollListSetText(sl, "Account Creation: Success", servSelected, 3, &colorGre, gRenderer, font);
                        stage = csMain;
                    }
                } else if(buf[bufIndex] == soNewBad) {
                    bufIndex += 1;
                    if(stage == csWoke) {
                        scrollListSetText(sl, "Account Creation: Failed", servSelected, 3, &colorRed, gRenderer, font);
                        stage = csMain;
                    }
                } else if(buf[bufIndex] == soLogGood) {
                    bufIndex += 1;
                    if(stage == csWoke) {
//                    if(nread == 1 + KEYSIZE) {
                        if(nread == 1 + KEYSIZE + sizeof(int) + sizeof(float)) {
                            scrollListSetText(sl, "Log In: Success", servSelected, 3, &colorGre, gRenderer, font);
                            int i;
                            for(i = 0; i < KEYSIZE; i++) {
                                key[i] = buf[i + bufIndex];
                            }

                            bufIndex += KEYSIZE;
                            entId = buffToInt(&buf[bufIndex]);
                            bufIndex += sizeof(int);
                            memcpy(&terrainScale, &buf[bufIndex], sizeof(float));
                            bufIndex += sizeof(float);

                            // Log in successful, send update. Receive terrain next.
                            buf[0] = coUpdate;
                            if (write(sfd, buf, 17) != 17) {
                                fprintf(stderr, "partial/failed write\n");
                                // exit(EXIT_FAILURE);
                            }
                            closingUI();
                            stage = csRecvInit;
                        }
                    }
                } else if(buf[bufIndex] == soLogBad) {
                    bufIndex += 1;
                    if(stage == csWoke) {
                        scrollListSetText(sl, "Log In: Failed", servSelected, 3, &colorRed, gRenderer, font);
                        stage = csMain;
                    }
                } else if(buf[bufIndex] == soInitTer) {
                    bufIndex += 1;
                    if(stage == csRecvInit) {
                        int numTers = buffToInt(&buf[bufIndex]);
                        printf("numTers: %d\n", numTers);
                        bufIndex += sizeof(int);
                        int i;
                        for(i = 0; i < numTers; i++) {
                            bufIndex += recvTerList(&buf[bufIndex], nread - bufIndex, terList);
                        }
//                        printf("soInitTer: %d\n", arrayListGetLength(terList));
//                        bufIndex += recvTerList(&buf[bufIndex], nread - bufIndex, terList);
//                        printf("loadedTer: %d\n", arrayListGetLength(terList));
                        sendUpdate(sfd, buf, key);
                    }
                } else if(buf[bufIndex] == soInitEnt) {
                    bufIndex += 1;
                    printf("stage: %d\n", stage);
                    if(stage == csRecvInit) {
                        printf("soInitEnt: %d\n", arrayListGetLength(entList));
                        bufIndex += recvEnts(&buf[bufIndex], nread - bufIndex, entList);
                        printf("loadedEnt: %d\n", arrayListGetLength(entList));
                        sendUpdate(sfd, buf, key);
                    }
                } else if(buf[bufIndex] == soUpdate) {
                    bufIndex += 1;
                    // Capstone.
                    if(stage == csRecvInit) {
                        // Normally i would move to loading models and ters. But for testing im moving onto game.
                        stage = csGame;
                        oldToNew();
                        printf("entID: %d\n", entId);
                        struct jalbEntity *myEnt = arrayListGetPointer(entList, entId);
                        printf("myEnt: %p\n", myEnt);
                        myCam->followVect = myEnt->pos;
                        sendUpdate(sfd, buf, key);
                        myEntChunkCoordsOld[0] = myEnt->chunkCoords[0];
                        myEntChunkCoordsOld[1] = myEnt->chunkCoords[1];
                    }
                }
//            }
        }

        if(SDL_GetTicks() - lastRecv > 5000) {
            stage = csMain;

            char a[256];
            strncpy(a, "Connection: Timed Out", 255);
            scrollListSetText(sl, a, servSelected, 2, &colorRed, gRenderer, font);
        }
    }
}

void closing(){
    if(stage == csGame) {
        SDL_GL_DeleteContext(gContext);

        int numChunks = arrayListGetLength(terList);
        while (numChunks > 0) {
            struct jalbTerrainList *ter = arrayListGetPointer(terList, 0);
            freeTerrain(ter);
            arrayListRemove(terList, 0);
            numChunks--;
        }
        freeArrayList(terList);
        // Free terList
        int numModels = arrayListGetLength(modList);
        while(numModels > 0) {
            struct jalbModel *mod = arrayListGetPointer(modList, 0);
            jalbFreeModel(mod);
            arrayListRemove(modList, 0);
            numModels--;
        }
        int numEnts = arrayListGetLength(entList);
        while(numEnts > 0) {
            struct jalbEntity *ent = arrayListGetPointer(entList, 0);
            free(ent);
            arrayListRemove(entList, 0);
            numEnts -= 1;
        }
    } else if(stage == csMain || stage == csWoke) {
        closingUI();
    } else {
        if(gRenderer != NULL) {
            SDL_DestroyRenderer(gRenderer);
        }
    }
    SDL_DestroyWindow(gWindow);
    saveKeyMap(dirKeyMap);
}

// Do the opposite of initUI.
void closingUI() {
    // Save the servList.
    // Free the scrollList and servList.
    // Free SDL rendering.
    // Free SDL_ttf.
    // Free options button array.
    // Free options scroll bar.

    // SDL renderer
    if(gRenderer != NULL) {
        SDL_DestroyRenderer(gRenderer);
        gRenderer = NULL;
    }

    // Font + ttf
    TTF_CloseFont(font);
    TTF_Quit();
    font = NULL;

    // Theme
    if(theme != NULL) {
        SDL_DestroyTexture(theme);
        theme = NULL;
    }

    // Servers + servScrollList.
    saveServs(dirServList, servList);

    if(sl != NULL) {
        int i;
        for(i = 0; i < sl->numButts; i++) {
            int r;
            for(r = 0; r < sl->rowsPerButt; r++) {
                SDL_Texture *tex = arrayListGetPointer(sl->textList, i * sl->rowsPerButt + r);
                SDL_DestroyTexture(tex);
            }

            struct jalbServer *s = arrayListGetPointer(servList, i);
            free(s);
        }
        freeArrayList(sl->textList);
        freeArrayList(sl->widthList);
        freeArrayList(servList);
        free(sl);
        servList = NULL;
        sl = NULL;
    }

    // main buttonArray
    freeButtonArray(mainBa);
    mainBa = NULL;

    freeButtonArray(editBa0);
    freeButtonArray(editBa1);
    freeButtonArray(editBa2);
    editBa0 = NULL;
    editBa1 = NULL;
    editBa2 = NULL;

    freeButtonArray(optionBaKeys);
    freeButtonArray(optionBa1);
    optionBaKeys = NULL;
    optionBa1 = NULL;

    free(optionSB);
    optionSB = NULL;


    if(loadingMap != NULL) {
        if(loadingMap->tex != NULL) {
            SDL_DestroyTexture(loadingMap->tex);
        }
        free(loadingMap);
        loadingMap = NULL;
    }
}

/** Game Function */
// stage == csGame;
void gameTic() {
//    printf("gameTic\n");

    int nread;
    while((nread = read(sfd, buf, BUFSIZE)) > 0) {
        ticlyIn(nread);
        lastRecv = SDL_GetTicks();
    }

    makeProjMatrix(myCam);

    struct jalbEntity *myEnt = arrayListGetPointer(entList, entId);

    /// Free Old Chunks
    if(myEnt->chunkCoords[0] != myEntChunkCoordsOld[0]
            || myEnt->chunkCoords[1] != myEntChunkCoordsOld[1]) {
//        printf("justLeft\n");
        // Left a chunk;
        int numTers = arrayListGetLength(terList);
        int i;
        for(i = 0; i < numTers; i++) {
            struct jalbTerrainList *ter = arrayListGetPointer(terList, i);
            int dx = ter->chunkCoords[0] - myEnt->chunkCoords[0];
            int dy = ter->chunkCoords[1] - myEnt->chunkCoords[1];
            int r = terRadius(dx, dy);
//            printf("terRad: %d\n", r);
            if(r >= chunkRadius) {
//                struct jalbTerrainList *next = arrayListGetPointer(terList, i + 1);
//                printf("points (%p, %p)\n", ter, next);
                freeTerrain(ter);
                arrayListRemove(terList, i);
//                int newlen = arrayListGetLength(terList);
//                printf("lens (%d, %d)\n", numTers, newlen);
                numTers--;
                i--;
            }
        }
        myEntChunkCoordsOld[0] = myEnt->chunkCoords[0];
        myEntChunkCoordsOld[1] = myEnt->chunkCoords[1];
    }

    /// Render
    // Clear the screen
    // The depth buffer is only necessary if i enable GL_DEPTH_TEST
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//    glClear( GL_COLOR_BUFFER_BIT );

    int i;
    int numTers = arrayListGetLength(terList);
//    printf("numTers: %d\n", numTers);
    for(i = 0; i < numTers; i++) {
        struct jalbTerrainList *ter = arrayListGetPointer(terList, i);
        if(i == 0) {
            ter->verts[0].texi = 1;
        }
        renderTerList(ter, myCam, myEnt->chunkCoords);
    }

    int numEnts = arrayListGetLength(entList);
    for(i = 0; i < numEnts; i++) {
        struct jalbEntity *ent = arrayListGetPointer(entList, i);
        renderEntity(ent, myCam);
    }

    renderSkyBox();

//    drawGameUI();

    /// Data Out.

    ticlyOut();

    /// Clear / Update buffers.
    mouseDownLeftOld = mouseDownLeft;
    mouseDownRightOld = mouseDownRight;

    /// Auto DC
    if(SDL_GetTicks() - lastRecv > returnWaitTime * returnFailMax) {
        // Disconnect the client, free OGL re init
        printf("Timeout: Auto DC\n");
        stage = csMain;
        initUI();
    }
}

void inputEventsGame(SDL_Event e) {
    /** Input Events */
    //Handle events on queue
    while( SDL_PollEvent( &e ) != 0 ) {
        //User requests quit
        if( e.type == SDL_QUIT ) {
            running = 0;
        } else if(e.type == SDL_KEYDOWN) {
            SDL_Keycode c = (SDL_Keycode) e.key.keysym.sym;
            int j;
            for(j = 0; j < numKeyMaps * 2; j++) {
//                printf("e.key.keysym.sym: %d (key: %d)\n", c, keyMaps[j]);
                if(c == keyMaps[j]) {
                    // If it is a key that relates to the game:
                    // check to see if it is already pressed, so i can save in gameInputs, to send the change.
                    int gameIndex = (j / 2) - kmWalkForward;
                    if(gameIndex >= 0) {
                        if(!allKeyInputs[j / 2]) {
                            // If not already pressed.
                            allKeyInputs[j/2] = 1;
                            gameInputs[gameIndex] = 1;
                        }
                    } else {
                        allKeyInputs[j/2] = 1;
                    }
                }
            }
        } else if(e.type == SDL_KEYUP) {
            SDL_Keycode c = (SDL_Keycode) e.key.keysym.sym;
            int j;
            for(j = 0; j < numKeyMaps * 2; j++) {
                if(c == keyMaps[j]) {
                    int gameIndex = (j / 2) - kmWalkForward;
                    if(gameIndex == 0) {
                        // Special walk key.
                        // Only dissable gameInputs, if all other walk reasons are false.
                        if(!mouseDownLeft || !mouseDownRight) {
                            gameInputs[gameIndex] = 2;
                        }
                    } else if(gameIndex > 0) {
                        if(allKeyInputs[j / 2]) {
                            // If already pressed.
                            gameInputs[gameIndex] = 2;
                        }
                    }
                    allKeyInputs[j/2] = 0;
                }
            }
        } else if(e.type == SDL_MOUSEBUTTONDOWN) {
            // Mouse press.
            int x = 0, y = 0;
            SDL_GetMouseState( &x, &y );
            if (e.button.button == SDL_BUTTON_LEFT) {
                mouseDownLeft = 1;
                mouseLoc[0] = x;
                mouseLoc[1] = y;
            } else if (e.button.button == SDL_BUTTON_RIGHT) {
                mouseDownRight = 1;
                mouseLoc[0] = x;
                mouseLoc[1] = y;
            }
        } else if(e.type == SDL_MOUSEBUTTONUP) {
            scrollBarPressed = 0;
            if (e.button.button == SDL_BUTTON_LEFT) {
                mouseDownLeft = 0;
                if(!allKeyInputs[kmWalkForward]) {
                    if(mouseDownLeftOld && mouseDownRightOld) {
                        // If there are no reasons to walk, and mouse WAS a reason to walk, them set to release.
                        gameInputs[kmWalkForward - kmWalkForward] = 2;
                    }
                }
            } else if (e.button.button == SDL_BUTTON_RIGHT) {
                mouseDownRight = 0;
                if(!allKeyInputs[kmWalkForward]) {
                    if(mouseDownLeftOld && mouseDownRightOld) {
                        // If there are no reasons to walk, and mouse WAS a reason to walk, them set to release.
                        gameInputs[kmWalkForward - kmWalkForward] = 2;
                    }
                }
            }
        } else if (e.type == SDL_MOUSEMOTION) {
            if(mouseDownLeft || mouseDownRight) {
                // Update mouse loc.
                SDL_GetMouseState( mouseLoc, &mouseLoc[1] );
                if(!allKeyInputs[kmShift]) {
                    // Handles mouse dragging.
                    // Limits x axis to 90 up and down.
                    if(myCam->rots[0] + e.motion.yrel / 100.0 >= M_PI/2) {
                        myCam->rots[0] = M_PI/2;
                    } else if(myCam->rots[0] + e.motion.yrel / 100.0 <= -M_PI/2) {
                        myCam->rots[0] = -M_PI/2;
                    } else {
                        myCam->rots[0] = myCam->rots[0] + e.motion.yrel / 100.0;
                    }
                    // Keeps y between 2pi and -2pi.
                    myCam->rots[1] += e.motion.xrel / 100.0;
                    if(myCam->rots[1] >= 2 * M_PI ||
                        myCam->rots[1] <= -2 * M_PI) {
                        myCam->rots[1] = fmod(myCam->rots[1], M_PI);
                    }
                }
            }
        } else if (e.type == SDL_MOUSEWHEEL) {
            myCam->distance -= e.wheel.y;
        }
    }
}

/** Menu Functions */

void inputEvents(SDL_Event e) {
    /** Input Events */
    //Handle events on queue
    while( SDL_PollEvent( &e ) != 0 ) {
        //User requests quit
        if( e.type == SDL_QUIT ) {
            running = 0;
        } else if(e.type == SDL_KEYDOWN) {
            handleKeyDown((SDL_Keycode) e.key.keysym.sym);
        } else if(e.type == SDL_KEYUP) {
//            keyRelease((char) e.key.keysym.sym);
        } else if(e.type == SDL_MOUSEBUTTONDOWN) {
            // Mouse press.
            int x = 0, y = 0;
            SDL_GetMouseState( &x, &y );
            if (e.button.button == SDL_BUTTON_LEFT) {
                mouseDownLeft = 1;
                handleMouse(x, y);
            }
        } else if(e.type == SDL_MOUSEBUTTONUP) {
            scrollBarPressed = 0;
            if (e.button.button == SDL_BUTTON_LEFT) {
                mouseDownLeft = 0;
            }
        } else if (e.type == SDL_MOUSEMOTION) {
            if(stage == csMain || stage == csWakeupCreate || stage == csWakeupLogin) {
                if(scrollBarPressed == 1) {
                    int buttonHeight = sl->edgeW * 2 + sl->rowsPerButt * sl->fontH;
                    int drawHeight = sl->numButts * (sl->edgeW + buttonHeight) + sl->edgeW;
                    int ratio = drawHeight / sl->height;

                    if(sl->yIndent + e.motion.yrel * ratio< 0) {
                        sl->yIndent = 0;
                    } else if (sl->yIndent + e.motion.yrel * ratio> sl->maxYindent) {
                        sl->yIndent = sl->maxYindent;
                    } else {
                        sl->yIndent += e.motion.yrel * ratio;
                    }
                }
            } else if (stage == csOption) {
                if(scrollBarPressed) {
                    scrollBarDrag(optionSB, e.motion.yrel);
                }
            } else if (stage == csGame) {
                // Handles mouse dragging.
                if(mouseDownLeft) {
                    myCam->rots[0] = myCam->rots[0] + e.motion.yrel / 100.0;
                    myCam->rots[1] = myCam->rots[1] + e.motion.xrel / 100.0;
                }
            }
        } else if (e.type == SDL_MOUSEWHEEL) {
            if(stage == 0 || stage == 3 || stage == 4 ) {
                int x = 0, y = 0;
                SDL_GetMouseState( &x, &y );
                if(x > sl->x && x < sl->x + sl->width) {
                    if(y > sl->y && y < sl->y + sl->height) {
                        if(sl->yIndent - e.wheel.y * 4 < 0) {
                            sl->yIndent = 0;
                        } else if (sl->yIndent - e.wheel.y * 4 > sl->maxYindent) {
                            sl->yIndent = sl->maxYindent;
                        } else {
                            sl->yIndent -= e.wheel.y * 4;
                        }
                    }
                }
            } else if(stage == 2) {
                if (optionSB->yIndent - e.wheel.y * 3 < 0) {
                    optionSB->yIndent = 0;
                } else if (optionSB->yIndent - e.wheel.y * 3 > optionMaxYscroll) {
                    optionSB->yIndent = optionMaxYscroll;
                } else {
                    optionSB->yIndent -= e.wheel.y * 3;
                }
            }
        }
    }
}

void handleKeyDown(SDL_Keycode c) {
    // What i want to do with this.
    // Minimize the below code, specifically when it comes toetext editing.
    // First test to see if a text box is selected. If so then iterate the text keyMaps, and handle the text box.
    // After wards depending on the stage deal with the other key inputs appropriatly.
    char *str = stageToStr();
    if(str) {
        // Ascii: SpecialChars, Numbers, SpecialChars, UpperCase, SpecialChars, LowerCase, SpecialChars.
        if(c >= 32 && c < 127) {
            editCharArray(str, 255, c, cursorIndex);
            // buttArSetText(editBa0, gRenderer, s->name, 0, 0, &color, font);
            if(cursorIndex < 255) {
                cursorIndex++;
            }
            stageUpdateText(str);
        }
        if(c == keyMaps[kmScrollLeft * keysPerInput]) {
            // Scroll Left
            if(cursorIndex > 0) {
                cursorIndex--;
            }
        } else if(c == keyMaps[kmScrollRight * keysPerInput]) {
            // Scroll right
            int strLen = 0;
            if(str) {
                while(str[strLen] != '\0') {
                    strLen++;
                }
            }
            if(cursorIndex < strLen && strLen < 255) {
                cursorIndex++;
            }
        } else if(c == keyMaps[kmEnd * keysPerInput]) {
            // End
            int strLen = 0;
            while(str[strLen] != '\0') {
                strLen++;
            }
            cursorIndex = strLen;
        } else if(c == keyMaps[kmHome * keysPerInput]) {
            // Home
            cursorIndex = 0;
        } else if(c == keyMaps[7 * keysPerInput]) {
            // Backspace
            backSpaceCharArray(str, cursorIndex);
            stageUpdateText(str);
            if(cursorIndex > 0) {
                cursorIndex--;
            }
        } else if(c == keyMaps[kmDelete * keysPerInput]) {
            // Delete Key
            backSpaceCharArray(str, cursorIndex + 1);
            stageUpdateText(str);
        }
    }
    if(stage == csMain) {
        if(c == keyMaps[kmReturn * keysPerInput]) {
            // Return
            stage = csMain;
            servSelected = -1;
        }
    } else if(stage == csOption) {
        // Option Screen.
        printf("txtBoxSelected: %d\n", txtBoxSelected);
        if(txtBoxSelected == -1) {
            if(c == keyMaps[kmReturn * keysPerInput]) {
                // Return
                stage = csMain;
            } else if(c == keyMaps[kmScrollUp * keysPerInput]) {
                // Up
                if(optionSB->yIndent - 10 > 0) {
                    optionSB->yIndent -= 10;
                } else {
                    optionSB->yIndent = 0;
                }
            } else if(c == keyMaps[kmScrollDown * keysPerInput]) {
                // Down
                if(optionSB->yIndent + 10 < optionMaxYscroll) {
                    optionSB->yIndent += 10;
                } else {
                    optionSB->yIndent = optionMaxYscroll;
                }
            }
        } else {
            keyMaps[txtBoxSelected] = c;
            const char *keyName = SDL_GetKeyName(c);
            int row = txtBoxSelected / 2;
            int col = txtBoxSelected - row * 2 + 1;
            buttArSetText(optionBaKeys, gRenderer, (char*) keyName, col, row, &color, font);
        }
    } else if(stage == csWakeupCreate || stage == csWakeupLogin) {
        // Should these be else-ifs?
        if(c == keyMaps[0 * keysPerInput]) {
            // Return
            stage = csMain;
//            servSelected = -1;
        }
    }
}

void handleMouse(int x, int y) {
    if(mouseDownLeft) {
        if(stage == 0) {
            int list = scrollListClickButt(sl, x, y);
            if(sl->maxYindent) {
                scrollBarPressed = scrollListClickBar(sl, x, y);
            }
            int butt = buttArClick(mainBa, x, y);
            if(butt == -1) {
                servSelected = list;
                if(list != -1) {
                    buttArSetText(mainBa, gRenderer, "Edit", 0, 1, &color, font);
                } else {
                    buttArSetText(mainBa, gRenderer, "Add", 0, 1, &color, font);
                }
            } else if(butt == 0) {
                // Log in.
                initGame();
                // Wakeup
                if(servSelected != -1) {
                    struct jalbServer *ser = arrayListGetPointer(servList, servSelected);
                    sfd = sendWakeUp(ser, sfd, buf);
                    printf("sfd: %d\n", sfd);

                    scrollListSetText(sl, "Connection: Pending", servSelected, 2, &colorOrng, gRenderer, font);
                    stage = csWakeupLogin;
                    lastRecv = SDL_GetTicks();
                }
            } else if(butt == 1) {
                // Options
                stage = csOption;
                txtBoxSelected = -1;
                optionSB->yIndent = 0;
            } else if(butt == 2) {
                // Add / Edit
                if(servSelected == -1) {
                    // Add
                    struct jalbServer *s = malloc(sizeof(struct jalbServer));
                    s->name[0] = 'n';
                    s->name[1] = 'a';
                    s->name[2] = 'm';
                    s->name[3] = 'e';
                    s->name[4] = '\0';

                    s->address[0] = 'a';
                    s->address[1] = 'd';
                    s->address[2] = 'd';
                    s->address[3] = 'r';
                    s->address[4] = '\0';

                    s->username[0] = 'u';
                    s->username[1] = 's';
                    s->username[2] = 'e';
                    s->username[3] = 'r';
                    s->username[4] = '\0';

                    s->password[0] = 'p';
                    s->password[1] = 'a';
                    s->password[2] = 's';
                    s->password[3] = 's';
                    s->password[4] = '\0';
//                    scrollListAddServer(sl, s);
//                    addArrayListEndPointer(sl->aList, s);
//                    sl->numButts++;
                    scrollListAddButt(sl);
                    scrollListSetText(sl, s->name, sl->numButts - 1, 0, &color, gRenderer, font);
                    scrollListSetText(sl, s->address, sl->numButts - 1, 1, &color, gRenderer, font);

                    arrayListAddEndPointer(servList, s);

                    printf("servAdded: %p\n", s);

                } else {
                    // Edit
                    // Load the relevant server information into edit button arrays
//                    struct jalbServer *s = arrayListGetPointer(sl->aList, servSelected);
                    printf("editPress\n");
                    struct jalbServer *s = arrayListGetPointer(servList, servSelected);

                    printf("editServ: %p\n", s);

                    buttArSetText(editBa0, gRenderer, s->name, 0, 0, &color, font);
                    buttArSetText(editBa0, gRenderer, s->address, 0, 1, &color, font);
                    buttArSetText(editBa1, gRenderer, s->username, 0, 0, &color, font);
                    buttArSetText(editBa1, gRenderer, s->password, 1, 0, &color, font);
                    stage = csEdit;

                    txtBoxSelected = -1;
                }
            } else if(butt == 3) {
                // Quit
                running = 0;
            }
        } else if (stage == 1) {
            // Edit screen
            int butt0 = buttArClick(editBa0, x, y);
            int butt1 = buttArClick(editBa1, x, y);
            int butt2 = buttArClick(editBa2, x, y);

            printf("butts (%d, %d)\n", butt0, butt1);

            if(butt0 == 0) {
                // Edit server name.
                txtBoxSelected = 0;
                struct jalbServer *s = arrayListGetPointer(servList, servSelected);
//                struct jalbServer *s = arrayListGetPointer(sl->aList, servSelected);
                int strLen = 0;
                while(s->name[strLen] != '\0') {
                    strLen++;
                }
                cursorIndex = strLen;
            } else if(butt0 == 1) {
                // Edit server address.
                txtBoxSelected = 1;
                struct jalbServer *s = arrayListGetPointer(servList, servSelected);
//                struct jalbServer *s = arrayListGetPointer(sl->aList, servSelected);
                int strLen = 0;
                while(s->address[strLen] != '\0') {
                    strLen++;
                }
                cursorIndex = strLen;
            }

            if(butt1 == 0) {
                // Edit username
                txtBoxSelected = 2;
                struct jalbServer *s = arrayListGetPointer(servList, servSelected);
//                    struct jalbServer *s = arrayListGetPointer(sl->aList, servSelected);
                int strLen = 0;
                while(s->username[strLen] != '\0') {
                    strLen++;
                }
                cursorIndex = strLen;
                printf("txtBoxSelected: %d\n", txtBoxSelected);
                printf("strLen: %d\n", strLen);
            } else if(butt1 == 1) {
                // Edit password
                txtBoxSelected = 3;
                struct jalbServer *s = arrayListGetPointer(servList, servSelected);
//                    struct jalbServer *s = arrayListGetPointer(sl->aList, servSelected);
                int strLen = 0;
                while(s->password[strLen] != '\0') {
                    strLen++;
                }
                cursorIndex = strLen;
            } else if(butt1 == 2) {
                // Create Account
                if(servSelected != -1) {
                    struct jalbServer *ser = arrayListGetPointer(servList, servSelected);
                    sfd = sendWakeUp(ser, sfd, buf);

                    scrollListSetText(sl, "Connection: Pending", servSelected, 2, &colorOrng, gRenderer, font);
                    stage = csWakeupCreate;
                    lastRecv = SDL_GetTicks();
                }
            } else if(butt1 == 3) {
                // Delete
                struct jalbServer *s = arrayListGetPointer(servList, servSelected);
                arrayListRemove(servList, servSelected);
                free(s);

                scrollListRemove(sl, servSelected);

                stage = csMain;
                servSelected = -1;
                buttArSetText(mainBa, gRenderer, "Add", 0, 1, &color, font);
            }

            if(butt2 == 0) {
                // Done
                stage = csMain;

                struct jalbServer *s = arrayListGetPointer(servList, servSelected);

                scrollListSetText(sl, s->name, servSelected, 0, &color, gRenderer, font);
                scrollListSetText(sl, s->address, servSelected, 1, &color, gRenderer, font);
            }
        } else if(stage == 2) {
            scrollBarPressed = scrollBarClick(optionSB, x, y);
            printf("scrollBarPressed: %d\n", scrollBarPressed);
            int butt0 = buttArClick(optionBaKeys, x, y + optionSB->yIndent);
            int butt1 = buttArClick(optionBa1, x, y + optionSB->yIndent);
            printf("option Butts (%d, %d)\n", butt0, butt1);
            if(butt1 == 0) {
                // Done
                stage = csMain;
            } else if(butt0 == -1) {
                txtBoxSelected = -1;
                printf("txtBoxSelected: %d\n", txtBoxSelected);
            } else {
                int row = butt0 / 3;
                butt0 -= row * 3;
                printf("row, col (%d, %d)\n", row, butt0);
                if(butt0 > 0) {
                    txtBoxSelected = row * 2 + butt0 - 1;
                    printf("txtBoxSelected: %d\n", txtBoxSelected);
                }
            }
        } else if(stage == 3 || stage == 4) {
            if(sl->maxYindent) {
                scrollBarPressed = scrollListClickBar(sl, x, y);
            }
            int butt = buttArClick(mainBa, x, y);
            if(butt == 3) {
                // quit.
                running = 0;
            }
        }
    }
}

char *stageToStr() {
    printf("servSelected: %d\n", servSelected);
    printf("txtBoxSelected: %d\n", txtBoxSelected);
    if(stage == 1) {
        if(servSelected != -1) {
            struct jalbServer *s =arrayListGetPointer(servList, servSelected);
//        struct jalbServer *s = arrayListGetPointer(sl->aList, servSelected);
            if(txtBoxSelected == 0) {
                return s->name;
            } else if(txtBoxSelected == 1) {
                return s->address;
            } else if(txtBoxSelected == 2) {
                return s->username;
            } else if(txtBoxSelected == 3) {
                return s->password;
            }
        }
    }
    return NULL;
}

struct buttonArray *stageToButtAr() {

    if(stage == 1) {
        if(servSelected == 0 || servSelected == 1) {
            return editBa0;
            // I should be able to
        }
    }
    return NULL;
}

void stageUpdateText(char *str) {
    // Gets ran when a stage and textBox is selected. And then a the button needs to get updated.
    // So i need to know which buttonArray to edit, and which button to effect.
    // I want this to get ran when i press a key and when i switch to the edit screen. I would like thoes text boxes to be automaticly set.
    // Is this to complex.
    // One function that has no parameter, and updates nly the texture of the button selected.
    // And maybe another function for updating the scene switch.
    if(stage == 1) {
        if(txtBoxSelected < 2) {
            buttArSetText(editBa0, gRenderer, str, 0, txtBoxSelected, &color, font);
        } else {
            buttArSetText(editBa1, gRenderer, str, txtBoxSelected - 2, 0, &color, font);
        }
    }
}

/*  Special Key Indexes
 *  0: Left
 *  1: Up
 *  2: Right
 *  3: Down
 *  4: BackSpace
 *  5: Delete
 *  6: Home
 *  7: End
 */
void specialKeys(int key) {
    if(stage == csEdit) {
        if(servSelected != -1) {
            struct jalbServer *s = arrayListGetPointer(servList, servSelected);
//            struct jalbServer *s = arrayListGetPointer(sl->aList, servSelected);
            char *str = NULL;
            if(txtBoxSelected == 0) {
                str = s->name;
            } else if(txtBoxSelected == 1) {
                str = s->address;
            } else if(txtBoxSelected == 2) {
                str = s->username;
            } else if(txtBoxSelected == 3) {
                str = s->password;
            }
            if(key == kmScrollLeft) {
                if(cursorIndex > 0) {
                    cursorIndex--;
                }
            } else if(key == kmScrollRight) {
                int strLen = 0;
                while(str[strLen] != '\0') {
                    strLen++;
                }
                if(cursorIndex < strLen && strLen < 255) {
                    cursorIndex++;
                }
            } else if(key == kmBackSpace) {
                backSpaceCharArray(str, cursorIndex);
                if(txtBoxSelected < 2) {
                    buttArSetText(editBa0, gRenderer, str, 0, txtBoxSelected, &color, font);
                } else {
                    buttArSetText(editBa1, gRenderer, str, txtBoxSelected - 2, 0, &color, font);
                }
                if(cursorIndex > 0) {
                    cursorIndex--;
                }
            } else if(key == kmDelete) {
                backSpaceCharArray(str, cursorIndex + 1);
                if(txtBoxSelected < 2) {
                    buttArSetText(editBa0, gRenderer, str, 0, txtBoxSelected, &color, font);
                } else {
                    buttArSetText(editBa1, gRenderer, str, txtBoxSelected - 2, 0, &color, font);
                }
//                if(cursorIndex > 0){
//                    cursorIndex--;
//                }
            } else if(key == kmHome) {
                cursorIndex = 0;
            } else if(key == kmEnd) {
                int strLen = 0;
                while(str[strLen] != '\0') {
                    strLen++;
                }
                cursorIndex = strLen;
            }
        }
    }
}

/** Ticly Networking */
// Uses buf.
void ticlyIn(int nread) {
    /// Data In.
    bufIndex = 0;
    printf("oglLoop: nread: %d\n", nread);
    while(bufIndex < nread) {
        printf("buf[%d]: %x\n", bufIndex, buf[bufIndex]);
        if(buf[bufIndex] == soUpdate) {
//                printf("\nRecv Update\n\n");
            // I think i should buffer this till later.
            bufIndex += 1;
            sendUpdate(sfd, buf, key);
            lastRecv = SDL_GetTicks();
            return;
        } else if(buf[bufIndex] == soEntUpdate) {
            bufIndex += 1;
            recvEntsTicly(buf);
        } else if(buf[bufIndex] == soTerUpdate) {
            bufIndex += 1;
            recvTerTicly(buf);
        } else if(buf[bufIndex] == soInitTer) {
            // Make a check to see if i already have this chunk, if so overwrite.
//            printf("terRecv: %d\n", bufIndex);
            bufIndex += 1;
            int numTers = buffToInt(&buf[bufIndex]);
//            printf("numTers: %d\n", numTers);
            bufIndex += sizeof(int);
            int i;
            for(i = 0; i < numTers; i++) {
                bufIndex += recvTerList(&buf[bufIndex], nread - bufIndex, terList);
            }
        } else {
            printf("Glitch: Receive Id Unknown. (bufIndex: %d) (id: %d)\n", bufIndex, buf[bufIndex]);
            break;
        }
    }
}

void ticlyOut() {
    /** InpId
     *  0: Key
     *  1: Mouse
     */

    // Turns double mouse button + no shift into walk input.
    // Handle mouse alone, if not walk,
    if(mouseDownLeft && mouseDownRight && !allKeyInputs[kmShift]) {
        // If it is not already set.
        if(!allKeyInputs[kmWalkForward]) {
            if(gameInputs[kmWalkForward - kmWalkForward] == 2) {
                // If the key was released this tic, dont send any update. (so it stays pressed)
                gameInputs[kmWalkForward - kmWalkForward] = 0;
            } else {
                // If the key was not released this tic, send the press. (so it becomes pressed)
                gameInputs[kmWalkForward - kmWalkForward] = 1;
            }
        }
    } else {
        // Dissable walk if w is not pressed.
        // How can i do this.
        // If either mouse is released, or if shift is pressed. Check to see if walking is true for this reason. If so dissable walking.
        // As of here, i have no idea if w is pressed or not.
        // So i need to save if w is pressed or not.
        // 3 lists. Keys Pressed.-> inputs pressed -> input sends.
        if(allKeyInputs[kmWalkForward]) {

        }

    }

    /// Send
    // Key
    int bufIndex = 1 + KEYSIZE; // Id, Key, { inpId, data }, { ... }, ...
    int oldIndex = bufIndex;
    int numBuffered = 0;

    // Both mouse down = walk.
    if(!allKeyInputs[kmShift] && mouseDownLeft && mouseDownRight) {
        gameInputs[kmWalkForward - kmWalkForward] = 1;
    }

    bufIndex += 1 + sizeof(int); // Id, numSent
    int i;
    for(i = 0; i < numGameInputs; i++) {
        if(gameInputs[i] > 0) {
            intToBuffer(i, &buf[bufIndex]);
            bufIndex += sizeof(int);
            buf[bufIndex] = gameInputs[i];
            bufIndex += 1;
//            intToBuffer(gameInputs[i], &buf[bufIndex]);
//            bufIndex += sizeof(int);
            // Clear buffer.
            gameInputs[i] = 0;
            // Iterate counter.
            numBuffered += 1;
        }
    }
    if(numBuffered == 0) {
        // Nothing buffered
        bufIndex = oldIndex;
    } else {
        buf[oldIndex] = cosKey; // KeyId
        intToBuffer(numBuffered, &buf[oldIndex + 1]);
    }


    /// Key and mouse inputs are handled seperatly.
    // Mouse
    if((mouseDownLeft || mouseDownRight) && allKeyInputs[kmShift]) {
//        printf("writeMouse (%d -> ", bufIndex);
        buf[bufIndex] = cosMouse; // MouseId
        bufIndex += 1;

        GLfloat tempVec[3];

        genCamTransVect3rd(myCam, tempVec);
        memcpy(&buf[bufIndex], tempVec, 3 * sizeof(float));
        bufIndex += 3 * sizeof(float);

        makeClickVect(mouseLoc[0], mouseLoc[1], tempVec);
        memcpy(&buf[bufIndex], tempVec, 3 * sizeof(float));
        bufIndex += 3 * sizeof(float);

        // Id { 0 = push, 1 = pull }
        if(mouseDownLeft) {
            buf[bufIndex] = 1;
        } else if (mouseDownRight) {
            buf[bufIndex] = 0;
        }
        bufIndex += 1;

//        printf("%d)\n", bufIndex);
    }
    // Admin mode.
    if(mouseDownRight && !allKeyInputs[kmShift]) {
        // Shift not pressed.
        printf("cosSetRots: %d\n", bufIndex);
        buf[bufIndex] = cosSetRots;
        bufIndex += 1;

//        struct jalbEntity *myEnt = arrayListGetPointer(entList, entId);

        memcpy(&buf[bufIndex], myCam->rots, sizeof(float));
        bufIndex += sizeof(float);
        float negY = -myCam->rots[1];
        memcpy(&buf[bufIndex], &negY, sizeof(float));
        bufIndex += sizeof(float);
        memcpy(&buf[bufIndex], &myCam->rots[2], sizeof(float));
        bufIndex += sizeof(float);
    }

//    printf("\nChanged: %d\n\n", changed);
    if(bufIndex > 1 + KEYSIZE) {
        buf[0] = coGameTic;
        memcpy(&buf[1], key, KEYSIZE);
        if (write(sfd, buf, bufIndex) != bufIndex) {
            fprintf(stderr, "partial/failed write\n");
//                            exit(EXIT_FAILURE);
        } else {
            printf("wrote %d bytes\n", bufIndex);
        }
    }
}

void drawGameUI() {
    render2d(portraitLoc[0], portraitLoc[1], portraitDraw[0], portraitDraw[1], portrait0);

    render2d(hotBarLoc[0], hotBarLoc[1], hotBarDraw[0], hotBarDraw[1], hotbar0);

    render2d(mapLoc[0], mapLoc[1], mapDraw[0], mapDraw[1], map0);

    gameRenderString("Schmitty Werb", portraitLoc[0] + nameOffset[0] + 2, portraitLoc[1] + nameOffset[1] + 16);

    drawWindow(10, 10, windowMinWidth, windowMinHeight);
}

void makeProjMatrix(struct jalbCamera *cam) {
// Makes Perspective matrix
    GLfloat tempMatrix[16] = {0};

    // Zero projMatrix.
    memset(projMatrix, 0, 16 * sizeof(float));

    // Apply Perspective matrix
    projMatrix[0 * 4 + 0] = (float) SCREEN_HEIGHT/SCREEN_WIDTH;
//    projMatrix[0 * 4 + 0] = 1;
    projMatrix[1 * 4 + 1] = 1;
    projMatrix[2 * 4 + 2] = 2.0 / (far - near);
    projMatrix[2 * 4 + 3] = -(near+far) / (float) (far - near);
    projMatrix[3 * 4 + 2] = 1;

    // Makes + Apply cam x angle matrix.
    xRotMatrix4(tempMatrix, -cam->rots[0]);
    matMatMult(projMatrix, tempMatrix);

    // Makes + Applies cam rotation matrix
    yRotMatrix4(tempMatrix, cam->rots[1]);
    matMatMult(projMatrix, tempMatrix);

    // Makes + Apply camTrans matrix
    float wx = 0;
    float wy = 0;
//    terToWorldNew(gMem->chunkCoords[0], gMem->chunkCoords[1], &wx, &wy, terrainScale);

    genCamTransMatrix3rd(cam, tempMatrix);

    tempMatrix[0] += wx;
    tempMatrix[2] += wy;

    matMatMult(projMatrix, tempMatrix);
}

/** Data In */

void recvEntsTicly() {
    int numEnts = buffToInt(&buf[bufIndex]);
    bufIndex += sizeof(int);
    int i;
    for(i = 0; i < numEnts; i++) {
        int id = buffToInt(&buf[bufIndex]);
        bufIndex += sizeof(int);
        int numSubs = buffToInt(&buf[bufIndex]);
        bufIndex += sizeof(int);
        struct jalbEntity *ent = arrayListGetPointer(entList, id);
        int j;
        for(j = 0; j < numSubs; j++) {
            if(buf[bufIndex] == soEntLoc) {
//                printf("walkForw: %d\n", bufIndex);
                bufIndex += 1;
                ent->chunkCoords[0] = buffToInt(&buf[bufIndex]);
                bufIndex += sizeof(int);
                ent->chunkCoords[1] = buffToInt(&buf[bufIndex]);
                bufIndex += sizeof(int);
                memcpy(ent->pos, &buf[bufIndex], sizeof(float) * 3);
                bufIndex += sizeof(float) * 3;
            } else if(buf[bufIndex] == soEntRot) {
//                printf("rotNao: %d\n", bufIndex);
                bufIndex += 1;
                memcpy(ent->rots, &buf[bufIndex], sizeof(float) * 3);
                bufIndex += sizeof(float) * 3;
            }
        }
        if(id == entId) {
//            printf("setAng\n");
            if(!mouseDownLeft && !mouseDownRight) {
                myCam->rots[1] = -ent->rots[1];
            }
        }
    }
}

void recvTerTicly(unsigned char *buf) {
//    printf("recvTerTicly (bufIndex: %d)\n", bufIndex);
    int numChunks = buffToInt(&buf[bufIndex]);
    bufIndex += sizeof(int);
    printf("numChunks: %d (bufIndex: %d)\n", numChunks, bufIndex);
    int i;
    for(i = 0; i < numChunks; i++) {
        int coordX = buffToInt(&buf[bufIndex]);
        bufIndex += sizeof(int);
        int coordY = buffToInt(&buf[bufIndex]);
        bufIndex += sizeof(int);

        // Find the chunk with the inputed coords.
        int numTers = arrayListGetLength(terList);
        struct jalbTerrainList *ter = NULL;
        int j;
        for(j = 0; j < numTers; j++) {
            ter = arrayListGetPointer(terList, j);
            if(ter->chunkCoords[0] == coordX && ter->chunkCoords[1] == coordY) {
                break;
            }
        }
        if(j >= numTers) {
            /// FIX IT
            // Ter doesnt exist.
            return;
        }

//        printf("soTerVertMove: %d (bufIndex: %d)\n", soTerVertMove, bufIndex);
        if(buf[bufIndex] == soTerVertMove) {
            bufIndex += 1;
            int numTerMove = buffToInt(&buf[bufIndex]);
            bufIndex += sizeof(int);
//            printf("numTerMove: %d\n", numTerMove);
            for(j = 0; j < numTerMove; j++) {
                int vertId = buffToInt(&buf[bufIndex]);
                bufIndex += sizeof(int);
//                printf("vertId: %d (chunk i: %d)\n", vertId, i);
                memcpy(&ter->verts[vertId], &buf[bufIndex], sizeof(struct vertTex));
                bufIndex += sizeof(struct vertTex);
//                ter->verts[vertId].texi = 1;
            }
        }
    }
}

void renderUI() {
    if(stage == csMain || stage == csWakeupCreate || stage == csWakeupLogin || stage == csWoke) {
        // White background.
        SDL_Rect rect;
        rect.x = 0;
        rect.y = 0;
        rect.w = SCREEN_WIDTH;
        rect.h = SCREEN_HEIGHT;

        SDL_SetRenderDrawColor(gRenderer, 255, 255, 255, 1);
        SDL_RenderFillRect(gRenderer, &rect);

        // Theme
        rect.x = (SCREEN_WIDTH - themeW) / 2;
        rect.y = 10;
        rect.w = themeW;
        rect.h = themeH;
        SDL_RenderCopy(gRenderer, theme, NULL, &rect);

        // Server List
        scrollListRender(sl, gRenderer, font, &color, servSelected);

        buttArRender(mainBa, gRenderer);

        // Please Distribute.
        char *str = "Original Game, Please Distribute.";
        int strlen = 0;
        while(str[strlen] != '\0') {
            strlen++;
        }

        SDL_Surface *txt = TTF_RenderText_Blended(font, str, color);
        SDL_Texture *tx = SDL_CreateTextureFromSurface(gRenderer, txt);

        SDL_FreeSurface(txt);

        rect.x = 0;
        rect.y = SCREEN_HEIGHT - textH;
        rect.w = strlen * textW;
        rect.h = textH;
        SDL_RenderCopy(gRenderer, tx, NULL, &rect);
    } else if(stage == csEdit) {
//        printf("editRender Start\n");
        // Edit screen.
        // White background.
        SDL_Rect rect;
        rect.x = 0;
        rect.y = 0;
        rect.w = SCREEN_WIDTH;
        rect.h = SCREEN_HEIGHT;

        SDL_SetRenderDrawColor(gRenderer, 255, 255, 255, 1);
        SDL_RenderFillRect(gRenderer, &rect);

        // Theme
        rect.x = (SCREEN_WIDTH - themeW) / 2;
        rect.y = 10;
        rect.w = themeW;
        rect.h = themeH;
        SDL_RenderCopy(gRenderer, theme, NULL, &rect);

//        printf("editRender\n");

        if(txtBoxSelected < 2) {
            buttArRenderCursor(editBa0, gRenderer, txtBoxSelected, cursorIndex, cursorW, cursorH);
            buttArRender(editBa1, gRenderer);
        } else {
            buttArRender(editBa0, gRenderer);
            buttArRenderCursor(editBa1, gRenderer, txtBoxSelected - 2, cursorIndex, cursorW, cursorH);
        }
        buttArRender(editBa2, gRenderer);
    } else if(stage == csOption) {
        // White background.
        SDL_Rect rect;
        rect.x = 0;
        rect.y = 0;
        rect.w = SCREEN_WIDTH;
        rect.h = SCREEN_HEIGHT;

        SDL_SetRenderDrawColor(gRenderer, 255, 255, 255, 1);
        SDL_RenderFillRect(gRenderer, &rect);

        // Theme
        rect.x = (SCREEN_WIDTH - themeW) / 2;
        rect.y = 10;
        rect.w = themeW;
        rect.h = themeH;

        SDL_Rect srcrect;
        srcrect.x = 0;
        srcrect.y = 0;
        srcrect.w = themeW;
        srcrect.h = themeH;

        if(optionSB->yIndent < 10 + themeH) {
            // Render the theme.
        }
        rect.y -= optionSB->yIndent;
        if(rect.y < 0) {
            rect.y = 0;
            rect.h = themeH - (optionSB->yIndent - 10);

//            printf("scroll: %d\n", optionSB->yIndent);
//            printf("rect.h: %d\n", rect.h);

            srcrect.y = optionSB->yIndent - 10;
            srcrect.h = rect.h;

        }
        SDL_RenderCopy(gRenderer, theme, &srcrect, &rect);

        optionBaKeys->y -= optionSB->yIndent;
        buttArRender(optionBaKeys, gRenderer);
        optionBaKeys->y += optionSB->yIndent;

        optionBa1->y -= optionSB->yIndent;
        buttArRender(optionBa1, gRenderer);
        optionBa1->y += optionSB->yIndent;

        scrollBarRender(optionSB, gRenderer);
    }
    if(stage == csWakeupCreate || stage == csWakeupLogin) {
        int centX = sl->x + (sl->width - loadingMap->width) / 2;
        int centY = sl->y + (sl->height - loadingMap->height) / 2;

        animMapRender(loadingMap, centX, centY, gRenderer);
    }
}

void editCharArray(char* str, int maxLength, char c, int cursorIndex) {
    int strLen = 0;
    while(str[strLen] != '\0') {
        strLen++;
    }
    if(cursorIndex >= maxLength) {
        // dont do anything.
        printf("char array too long\n");
        return;
    }
    if(cursorIndex <= strLen) {
        int i;
        for(i = strLen; i > cursorIndex; i--) {
            str[i] = str[i - 1];
        }
        str[cursorIndex] = c;
        str[strLen + 1] = '\0';
    }
}

// This will be used for both backspace, and delete.
// When deleting i will do cursorIndex + 1 before hand.
void backSpaceCharArray(char* str, int cursorIndex) {
    int strLen = 0;
    while(str[strLen] != '\0') {
        strLen++;
    }
    if(cursorIndex <= 0) {
        // At the start of the string.
        return;
    }
    printf("cursorIndex: %d\n", cursorIndex);
    printf("strLen: %d\n", strLen);
    if(cursorIndex <= strLen) {
        // This loop automatically moves the null pointer up a slot.
        int i;
        for(i = cursorIndex - 1; i < strLen; i++) {
            str[i] = str[i+1];
        }
//        str[strLen - 1] = '\0';
    }
}

/** Loading */
SDL_Texture* loadTexture( char *path, int *retWidth, int *retHeight) {
    //The final texture
    SDL_Texture* newTexture = NULL;
    //Load image at specified path
    SDL_Surface* loadedSurface = IMG_Load( path );
    if( loadedSurface == NULL ) {
        printf( "Unable to load image %s! SDL_image Error: %s\n", path, IMG_GetError() );
    } else {
        *retWidth = loadedSurface->w;
        *retHeight = loadedSurface->h;
        //Create texture from surface pixels
        newTexture = SDL_CreateTextureFromSurface( gRenderer, loadedSurface );
        if( newTexture == NULL ) {
            printf( "Unable to create texture from %s! SDL Error: %s\n", path, SDL_GetError() );
        }
        //Get rid of old loaded surface
        SDL_FreeSurface( loadedSurface );
    }
    return newTexture;
}

void saveServs(char *dir, ArrayList *servList) {
    FILE *file;
    file = fopen(dir, "w");

    if(file == NULL ) {
        printf("error opening file\n");
        return;
    }

    int total = arrayListGetLength(servList);

    printf("total: %d\n", total);

    fwrite(&total, sizeof(int), 1, file);

    int i;
    for(i = 0; i < total; i++) {
        struct jalbServer *ser = arrayListGetPointer(servList, i);
        saveServ(ser, file);
    }
    fclose(file);
}

int loadServs(char *dir, ArrayList *servList) {
    FILE *file;
    file = fopen(dir, "r");
    if(file == NULL ) {
        printf("error opening file\n");
        return 0;
    }
    int total;

    int nread = fread(&total, sizeof(int), 1, file);
    if(nread != 1) {
        printf("serv load error\n");
    }
    int current = arrayListGetLength(servList);

    printf("current: %d\n", current);

    printf("total: %d\n", total);

    int i;
    for(i = 0; i < total; i++) {
        struct jalbServer *ser;
        if(i >= current) {
            ser = malloc(sizeof(struct jalbServer));
            arrayListAddEndPointer(servList, ser);
        } else {
            ser = arrayListGetPointer(servList, i);
        }
        loadServ(ser, file);
    }
    fclose(file);
    return 1;
}

void saveServ(struct jalbServer *ser, FILE *file) {
    fwrite(ser->name, sizeof(char), 256, file);
    fwrite(ser->address, sizeof(char), 256, file);
    fwrite(ser->username, sizeof(char), 256, file);
    fwrite(ser->password, sizeof(char), 256, file);
}

void loadServ(struct jalbServer *ser, FILE *file) {
//    printf("loadServ: %p\n", ser);
    int nread = fread(ser->name, sizeof(char), 256, file);
    if(nread != 256) {
        printf("serv load error name\n");
    }
    nread = fread(ser->address, sizeof(char), 256, file);
    if(nread != 256) {
        printf("serv load error addr\n");
    }
    nread = fread(ser->username, sizeof(char), 256, file);
    if(nread != 256) {
        printf("serv load error user\n");
    }
    nread = fread(ser->password, sizeof(char), 256, file);
    if(nread != 256) {
        printf("serv load error pass\n");
    }
}

void saveKeyMap(char *dir) {
    FILE *file;
    file = fopen(dir, "w");

    if(file == NULL ) {
        printf("error opening file\n");
        return;
    }
    fwrite(keyMaps, sizeof(SDL_Keycode), numKeyMaps * 2, file);
}

// Return 0 if unsuccessful, 1 if successful.
int loadKeyMap(char *dir) {
    FILE *file = fopen(dir, "r");
    if(file == NULL ) {
        printf("error opening keyMap file\n");
        return 0;
    }
    int nread = fread(keyMaps, sizeof(SDL_Keycode), numKeyMaps * 2, file);
    if(nread != numKeyMaps * 2) {
        printf("keyMap load error\n");
    }
    return 1;
}

/** Ogl Inits */

void initGlew() {
    //Create context
    gContext = SDL_GL_CreateContext( gWindow );
    if( gContext == NULL ) {
        printf( "OpenGL context could not be created! SDL Error: %s\n", SDL_GetError() );
//        success = 0;
    }
    SDL_GL_MakeCurrent(gWindow, gContext);

    //Initialize GLEW
    glewExperimental = GL_TRUE;
    GLenum glewError = glewInit();
    if( glewError != GLEW_OK ) {
        printf( "Error initializing GLEW! %s\n", glewGetErrorString( glewError ) );
    }

    //Use Vsync
    if( SDL_GL_SetSwapInterval( 1 ) < 0 ) {
        printf( "Warning: Unable to set VSync! SDL Error: %s\n", SDL_GetError() );
    }
}

void initGL() {
//    ter = initTerrain(chunkRadius, 0, 0, terrainScale/chunkRadius);

    initShaders();

    initUniformLocations();

//    // Enable depth test
    glEnable(GL_DEPTH_TEST);
//    // Accept fragment if it closer to the camera than the former one
    glDepthFunc(GL_LESS);



//    glEnable(GL_CULL_FACE);
//
//    glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);
//    glHint(GL_CLIP_VOLUME_CLIPPING_HINT_EXT,GL_FASTEST);
//    glEnable(GL_DEPTH_CLAMP);

    // Initialize buffer objects
    glGenBuffers( 1, &vertexBuffer );
    glGenBuffers( 1, &indsBuf );
    glGenBuffers( 1, &indexBuffer );
    glGenBuffers( 1, &uvBuffer0 );
    glGenBuffers( 1, &uvBuffer1 );
    glGenBuffers( 1, &uvBuffer2 );
    glGenBuffers( 1, &vbo_triangle_colors );

    // Generate, bind, and specify the data store to hold fragments.  The
    // NULL pointer in BufferData says that the intial buffer contents are
    // undefined.  They will be filled in by the fragment shader code.
    glGenBuffers(1, &ssboVerts);

    glGenBuffers(1, &ssboIndex);

//    loadSkyBoxArrays();
//    cubemapTexture = loadCubemap();

    initTextures();

    // Load models
    arrayListAddEndPointer(modList, jalbMakeModel("res/model/demon.obj", demonTex));
    arrayListAddEndPointer(modList, jalbMakeModel("res/model/tree0.obj", treeTex));
    arrayListAddEndPointer(modList, jalbMakeModel("res/model/wall0.obj", wallTex));

    // Init Camera
    myCam = initCam();
    struct jalbEntity *myEnt = arrayListGetPointer(entList, entId);
    myCam->rots[1] = -myEnt->rots[1];
    //myCam->position[1] = 3;

    // Skybox.
    loadSkyBoxArrays();
    cubemapTexture = loadCubemap();
}

void initShaders() {
    skyBoxProgramID = loadShaders( "res/shader/skyBox.vs", "res/shader/skyBox.fs" );
    statModelProgramID = loadShaders( "res/shader/model.vs", "res/shader/model.fs" );
    animModelProgramID = loadShaders( "res/shader/animModel.vs", "res/shader/animModel.fs" );
    flatImgProgramId = loadShaders( "res/shader/flatImg.vs", "res/shader/flatImg.fs" );
    singleColProgId = loadShaders( "res/shader/singleCol.vs", "res/shader/singleCol.fs" );
    terProgramID = loadShaders( "res/shader/SSBO0.vs", "res/shader/SSBO0.fs" );
    skyBoxProgramID = loadShaders( "res/shader/skyBox.vs", "res/shader/skyBox.fs" );
}

void initUniformLocations() {
    skyBoxMatID = glGetUniformLocation(skyBoxProgramID, "MVP");
    skyBoxTexID = glGetUniformLocation(skyBoxProgramID, "skybox");

    statModelMatID = glGetUniformLocation(statModelProgramID, "MVP");
    statModelTexID = glGetUniformLocation(statModelProgramID, "myTextureSampler");

    animModelMatID = glGetUniformLocation(animModelProgramID, "MVP");
    animModelBoneID = glGetUniformLocation(animModelProgramID, "Bone");
    animModelTexID = glGetUniformLocation(animModelProgramID, "myTextureSampler");
    animModelIndexID = glGetUniformLocation(animModelProgramID, "Index");
    animModelWeightID = glGetUniformLocation(animModelProgramID, "weight");

    singleColUniformId = glGetUniformLocation(singleColProgId, "singleCol");
//    printf("ter1TexID: %d\n", ter1TexID);

    SSBOMatID = glGetUniformLocation(terProgramID, "MVP");
    SSBOUvID = glGetUniformLocation(terProgramID, "UVexamp");
    SSBOTexID = glGetUniformLocation(terProgramID, "tex0");

    flatImgTexID = glGetUniformLocation(flatImgProgramId, "tex0");


//    printf("SSBOMatID: %d\n", SSBOMatID);
//    printf("SSBOUvID: %d\n", SSBOUvID);
}

void initTextures() {
    terAtlas = SOIL_load_OGL_texture
               (
                   "res/texture/atlas1.png",
                   SOIL_LOAD_AUTO,
                   SOIL_CREATE_NEW_ID,
                   SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT
               );
    printf("terAtlas: %d\n", terAtlas);

    demonTex = SOIL_load_OGL_texture
               (
                   "res/texture/demon.png",
                   SOIL_LOAD_AUTO,
                   SOIL_CREATE_NEW_ID,
                   SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT
               );
    printf("demonTex: %d\n", demonTex);

    treeTex = SOIL_load_OGL_texture
              (
                  "res/texture/tree.png",
                  SOIL_LOAD_AUTO,
                  SOIL_CREATE_NEW_ID,
                  SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT
              );
    printf("treeTex: %d\n", treeTex);

    wallTex = SOIL_load_OGL_texture
              (
                  "res/texture/wall.png",
                  SOIL_LOAD_AUTO,
                  SOIL_CREATE_NEW_ID,
                  SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT
              );
    printf("wallTex: %d\n", wallTex);

    portrait0 = SOIL_load_OGL_texture
                (
                    "res/texture/portrait1.png",
                    SOIL_LOAD_AUTO,
                    SOIL_CREATE_NEW_ID,
                    SOIL_FLAG_MIPMAPS | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT
                );
    hotbar0 = SOIL_load_OGL_texture
              (
                  "res/texture/hotbar1.png",
                  SOIL_LOAD_AUTO,
                  SOIL_CREATE_NEW_ID,
                  SOIL_FLAG_MIPMAPS | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT
              );
    printf("hotbar0: %d\n", hotbar0);

    map0 = SOIL_load_OGL_texture
           (
               "res/texture/lumMap0.png",
               SOIL_LOAD_AUTO,
               SOIL_CREATE_NEW_ID,
               SOIL_FLAG_MIPMAPS | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT
           );
    printf("map0: %d\n", map0);
    /* check for an error during the load process */
//    if( terAtlas == 0 ) {
//        printf( "SOIL loading error: '%s'\n", SOIL_last_result() );
//    }
}

GLuint loadShaders(char * vertex_file_path, char * fragment_file_path) {
    // Create the shaders
    GLuint VertexShaderID = glCreateShader(GL_VERTEX_SHADER);
    GLuint FragmentShaderID = glCreateShader(GL_FRAGMENT_SHADER);

    // Read the Vertex Shader code from the file
    char* VertexShaderCode;
    VertexShaderCode = textFileRead(vertex_file_path);

    // Read the Fragment Shader code from the file
    char* FragmentShaderCode;
    FragmentShaderCode = textFileRead(fragment_file_path);

    GLint Result = GL_FALSE;
    int InfoLogLength;

    // Compile Vertex Shader
//    printf("Compiling shader : %s\n", vertex_file_path);
    // char const * VertexSourcePointer = VertexShaderCode.c_str();
    char const * VertexSourcePointer = VertexShaderCode;
    glShaderSource(VertexShaderID, 1, &VertexSourcePointer , NULL);
    glCompileShader(VertexShaderID);

    // Check Vertex Shader
    glGetShaderiv(VertexShaderID, GL_COMPILE_STATUS, &Result);
    glGetShaderiv(VertexShaderID, GL_INFO_LOG_LENGTH, &InfoLogLength);

    // Compile Fragment Shader
//    printf("Compiling shader : %s\n", fragment_file_path);
    char const * FragmentSourcePointer = FragmentShaderCode;
    glShaderSource(FragmentShaderID, 1, &FragmentSourcePointer , NULL);
    glCompileShader(FragmentShaderID);

    // Check Fragment Shader
    glGetShaderiv(FragmentShaderID, GL_COMPILE_STATUS, &Result);
    glGetShaderiv(FragmentShaderID, GL_INFO_LOG_LENGTH, &InfoLogLength);

    // Link the program
    fprintf(stdout, "Linking program\n");
    GLuint ProgramID = glCreateProgram();
    glAttachShader(ProgramID, VertexShaderID);
    glAttachShader(ProgramID, FragmentShaderID);
    glLinkProgram(ProgramID);

    // Check the program
    glGetProgramiv(ProgramID, GL_LINK_STATUS, &Result);
    glGetProgramiv(ProgramID, GL_INFO_LOG_LENGTH, &InfoLogLength);

    glDeleteShader(VertexShaderID);
    glDeleteShader(FragmentShaderID);

    free(VertexShaderCode);
    free(FragmentShaderCode);

    return ProgramID;
}

GLuint texHandle;

// Free the SDL Render and init the Ogl renderer
void oldToNew() {
    SDL_DestroyRenderer(gRenderer);
    gRenderer = NULL;

    /// Free Type Init.
    int error = FT_Init_FreeType( &ftLib );
    if( error ) {
        printf("Free Type Init Error.");
        exit(0);
    }

    error = FT_New_Face( ftLib,
                         "res/UbuntuMono-R.ttf",
                         0,
                         &ftFace );

    if ( error == FT_Err_Unknown_File_Format ) {
        printf("FontError\n");
        exit(0);
//        ... the font file could be opened and read, but it appears
//        ... that its font format is unsupported
    } else if ( error ) {
        printf("FontError\n");
        exit(0);
//        ... another error code means that the font file could not
//        ... be opened or read, or that it is broken...
    }

    error = FT_Set_Pixel_Sizes(
                ftFace,   /* handle to face object */
                0,      /* pixel_width           */
                20 );   /* pixel_height          */
    if(error) {
        printf("setPixelSizeError\n");
        exit(0);
    }

    initGlew();
    initGL();

    // other

    glGenTextures(1, &texHandle);

//    uniform_tex = glGetUniformLocation(uiProgramId, "tex0");

//        glGenTextures(1, &tex);


//        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
//        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
//
//        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
//        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    // Allow textures of any size, not just multiples of 4.
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    // end
}

/** Rendering */

void renderTerList(struct jalbTerrainList *ter, struct jalbCamera *cam, int *chunkCoords) {
    float wx = 0;
    float wy = 0;

    int relCoords[2];
    relCoords[0] = ter->chunkCoords[0] - chunkCoords[0];
    relCoords[1] = ter->chunkCoords[1] - chunkCoords[1];

    terToWorldNew(relCoords[0], relCoords[1], &wx, &wy, terrainScale);

//    wx *= 1.005;
//    wy *= 1.005;

    float transMatrix[16] = {0};
    int i;
    for(i = 0; i < 4; i++) {
        transMatrix[i * 4 + i] = 1;
    }
    transMatrix[0 * 4 + 3] += wx;
    transMatrix[2 * 4 + 3] += wy;

    // Create MVP buffer
    GLfloat MVP[16] = {0};
    for(i = 0; i < 4; i++) {
        MVP[i * 4 + i] = 1;
    }

    matMatMult(MVP, projMatrix);
    matMatMult(MVP, transMatrix);

    glBindBufferBase(GL_SHADER_STORAGE_BUFFER, 2, ssboVerts);
    glBufferData(GL_SHADER_STORAGE_BUFFER, sizeof(struct vertTex) * ter->numVerts, ter->verts, GL_DYNAMIC_DRAW);

    glBindBufferBase(GL_SHADER_STORAGE_BUFFER, 1, ssboIndex);
    glBufferData(GL_SHADER_STORAGE_BUFFER, sizeof(int) * 3 * ter->numFaces, ter->faces, GL_DYNAMIC_DRAW);

    glUseProgram( terProgramID );

    // Uniforms.
    glUniformMatrix4fv(SSBOMatID, 1, GL_TRUE, MVP);
    //    glUniform2fv(ter1UvID, 3, UVexamps);
    float UVexamps[6] = {.02, .98, .48, .98, .25, .52,};
    glUniform2fv(SSBOUvID, 3, UVexamps);

    // Bind our texture in Texture Unit 0
    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D, terAtlas);
    glUniform1i(SSBOTexID, 0);
    // Other

    glMemoryBarrier(GL_ALL_BARRIER_BITS);

//    glDrawArraysInstancedBaseInstance(GL_TRIANGLES, 0, ter->numFaces * 3, ter->numFaces, 0);

    // Wireframe on.
//    glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
    glDrawArraysInstancedBaseInstance(GL_TRIANGLES, 0, ter->numFaces * 3, ter->numFaces, 0);
    // Wireframe off.
//    glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );

    //Unbind program
    glUseProgram( 0 );
}

void renderEntity(struct jalbEntity *ent, struct jalbCamera *cam) {
    struct jalbModel *mod = arrayListGetPointer(modList, ent->modelIndex);
    GLuint texture = mod->tex;

    //Bind program
    glUseProgram( statModelProgramID );

    int i;

    // Create MVP buffer
    GLfloat MVP[16] = {0};
    for(i = 0; i < 4; i++) {
        MVP[i * 4 + i] = 1;
    }

    /** Projection, trans, rot.
     *  Scale can be last or 2nd to last
     */

    // Apply Projecting Matrix.
    matMatMult(MVP, projMatrix);


    // Make translation matrix
    GLfloat tempMatrix[16] = {0};
    for(i = 0; i < 4; i++) {
        tempMatrix[i * 4 + i] = 1;
    }

    tempMatrix[0 * 4 + 3] = ent->pos[0];
    tempMatrix[1 * 4 + 3] = ent->pos[1];
    tempMatrix[2 * 4 + 3] = ent->pos[2];

    // Apply transformation matrix.
    matMatMult(MVP, tempMatrix);

    // Make + Apply rotation matrix.
    yRotMatrix4(tempMatrix, ent->rots[1]);
    matMatMult(MVP, tempMatrix);
    xRotMatrix4(tempMatrix, ent->rots[0]);
    matMatMult(MVP, tempMatrix);
//    zRotMatrix4(tempMatrix, ent->rots[2]);
//    matMatMult(MVP, tempMatrix);

    // Make Scale matrix.
    for(i = 0; i < 16; i++) {
        tempMatrix[i] = 0;
    }
    for(i = 0; i < 4; i++) {
        tempMatrix[i * 4 + i] = 1;
    }

    tempMatrix[0 * 4 + 0] *= ent->scale;
    tempMatrix[1 * 4 + 1] *= ent->scale;
    tempMatrix[2 * 4 + 2] *= ent->scale;

    // Apply Scale matrix.
    matMatMult(MVP, tempMatrix);

    // Send our transformation to the currently bound shader,
    // in the "MVP" uniform
    glUniformMatrix4fv(statModelMatID, 1, GL_TRUE, MVP);

    // Bind our texture in Texture Unit 0
    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D, texture);

    glUniform1i(statModelTexID, 0);

    // Temp model pointer to make the code more comprehesible.

    //Set vertex data
    glEnableVertexAttribArray( 0 );
    glBindBuffer( GL_ARRAY_BUFFER, vertexBuffer );
    glBufferData( GL_ARRAY_BUFFER, mod->numVerts * 3 * sizeof(GLfloat), mod->verts, GL_STATIC_DRAW );
    glVertexAttribPointer( 0, 3, GL_FLOAT, GL_FALSE, 0, (void*)0);

    // Sets the UV data.
    if(mod->dimensions > 1) {
        glEnableVertexAttribArray( 1 );
        glBindBuffer( GL_ARRAY_BUFFER, uvBuffer0 );
        glBufferData( GL_ARRAY_BUFFER, mod->numVerts * 2 * sizeof(GLfloat), mod->UVs, GL_STATIC_DRAW );
        glVertexAttribPointer( 1, 2, GL_FLOAT, GL_FALSE, 0, (void*)0);
    }

    // Sets the index data.
    glBindBuffer( GL_ELEMENT_ARRAY_BUFFER, indexBuffer );
    glBufferData( GL_ELEMENT_ARRAY_BUFFER, mod->numFaces * 3 * sizeof(GLuint), mod->faces, GL_STATIC_DRAW );

    // Draw the triangles !
    glDrawElements(
        GL_TRIANGLES,      // mode
        mod->numFaces * 3,                // count
        GL_UNSIGNED_INT,   // type
        (void*)0           // element array buffer offset
    );

    //Disable vertex position
    glDisableVertexAttribArray( 0 );
    glDisableVertexAttribArray( 1 );

    //Unbind program
    glUseProgram( 0 );
}

void render2d(int x, int y, int width, int height, GLuint tex) {
    glDepthFunc(GL_ALWAYS);
    glUseProgram( flatImgProgramId );

    glEnable (GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D, tex);

    /// Fix it:
    // It renders even when commented out
    glUniform1i(flatImgTexID, 0);

    // Clockwise
    float verts[8];
    verts[0] = (float)x / SCREEN_WIDTH * 2 - 1;
    verts[1] = (float)-y / SCREEN_HEIGHT * 2 + 1;

//    sayVect("verts", verts, 2);

    verts[2] = verts[0] + (float)width / SCREEN_WIDTH * 2;
    verts[3] = verts[1];

    verts[4] = verts[2];
    verts[5] = verts[1] - (float)height / SCREEN_HEIGHT * 2;

    verts[6] = verts[0];
    verts[7] = verts[5];

    //Set vertex data
    glEnableVertexAttribArray( 0 );
    glBindBuffer( GL_ARRAY_BUFFER, vertexBuffer );
    glBufferData( GL_ARRAY_BUFFER, 4 * 2 * sizeof(GLfloat), verts, GL_STATIC_DRAW );
    glVertexAttribPointer( 0, 2, GL_FLOAT, GL_FALSE, 0, (void*)0);

    float UVs[8] = {0, 0,
                    1, 0,
                    1, 1,
                    0, 1
                   };

    // Sets the UV data.
    glEnableVertexAttribArray( 1 );
    glBindBuffer( GL_ARRAY_BUFFER, uvBuffer0 );
    glBufferData( GL_ARRAY_BUFFER, 4 * 2 * sizeof(GLfloat), UVs, GL_STATIC_DRAW );
    glVertexAttribPointer( 1, 2, GL_FLOAT, GL_FALSE, 0, (void*)0);

    glDrawArrays(
        GL_TRIANGLE_FAN,    // mode
        0,                  // first
        4                   // count
    );

    //Disable vertex position
    glDisableVertexAttribArray( 0 );
    glDisableVertexAttribArray( 1 );

    //Unbind program
    glUseProgram( 0 );
    glDepthFunc(GL_LESS);
    glDisable (GL_BLEND);
}

void gameRenderString(char *str, int x, int y) {
    glEnable (GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    while(*str != '\0') {
        if(*str == ' ') {
            x+=textW;
            str += sizeof(char);
            continue;
        }

//        printf("%x ", *str);
        int glyphIndex = FT_Get_Char_Index(ftFace, *str);
//        printf("glyphIndex: %d\n", glyphIndex);

        int error = FT_Load_Glyph(
                        ftFace,          /* handle to face object */
                        glyphIndex,   /* glyph index           */
                        0 );  /* load flags, see below */
        if(error) {
            printf("loadGlyphError\n");
            exit(0);
        }
        error = FT_Render_Glyph( ftFace->glyph,   /* glyph slot  */
                                 FT_RENDER_MODE_NORMAL ); /* render mode */
        if(error) {
            printf("ftError Render_Glyph\n");
            exit(0);
        }

        // create the texture

        glBindTexture(GL_TEXTURE_2D, texHandle);
//        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_RGB, GL_FLOAT, &image[0]);

        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);

        FT_GlyphSlot g = ftFace->glyph;

//        printf("dim (%d, %d)\n", g->bitmap.width, g->bitmap.rows);

        glTexImage2D(GL_TEXTURE_2D, 0, GL_ALPHA, g->bitmap.width, g->bitmap.rows, 0, GL_ALPHA, GL_UNSIGNED_BYTE, g->bitmap.buffer);

        render2d(x + g->bitmap_left, y-g->bitmap_top, g->bitmap.width, g->bitmap.rows, texHandle);
        x+=g->bitmap.width + g->bitmap_left;
        str += sizeof(char);
    }

    glDisable (GL_BLEND);
//    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
//    printf("\n");
}

void renderRect(int x, int y, int width, int height, float r, float g, float b, float a) {
    glDepthFunc(GL_ALWAYS);

    glEnable (GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    glUseProgram( singleColProgId );

    glUniform4f(singleColUniformId, r, g, b, a);

    // Clockwise
    float verts[8];
    verts[0] = (float)x / SCREEN_WIDTH * 2 - 1;
    verts[1] = (float)-y / SCREEN_HEIGHT * 2 + 1;

//    sayVect("verts", verts, 2);

    verts[2] = verts[0] + (float)width / SCREEN_WIDTH * 2;
    verts[3] = verts[1];

    verts[4] = verts[2];
    verts[5] = verts[1] - (float)height / SCREEN_HEIGHT * 2;

    verts[6] = verts[0];
    verts[7] = verts[5];

    //Set vertex data
    glEnableVertexAttribArray( 0 );
    glBindBuffer( GL_ARRAY_BUFFER, vertexBuffer );
    glBufferData( GL_ARRAY_BUFFER, 4 * 2 * sizeof(GLfloat), verts, GL_STATIC_DRAW );
    glVertexAttribPointer( 0, 2, GL_FLOAT, GL_FALSE, 0, (void*)0);

    glDrawArrays(
        GL_TRIANGLE_FAN,    // mode
        0,                  // first
        4                   // count
    );

    //Disable vertex position
    glDisableVertexAttribArray( 0 );
    glDisableVertexAttribArray( 1 );

    //Unbind program
    glUseProgram( 0 );
    glDepthFunc(GL_LESS);

    glDisable (GL_BLEND);
}

void renderSkyBox() {
//    glDisable(GL_DEPTH_TEST);
    glDepthMask(GL_FALSE);// Remember to turn depth writing off
    glUseProgram( skyBoxProgramID );

    float *matrix = calloc(16, sizeof(float));
    int i;
    for( i = 0; i < 4; i++) {
        matrix[i + i * 4] = 1.0;
    }

    // Makes Perspective matrix
    GLfloat *tempMatrix;

    tempMatrix = calloc(16, sizeof(float));

    tempMatrix[0 * 4 + 0] = 1;
    tempMatrix[1 * 4 + 1] = 1;
    tempMatrix[2 * 4 + 2] = 2.0 / (2.0 - .1);
    tempMatrix[2 * 4 + 3] = -(.1+2.0) / (float) (2.0 - .1);

    tempMatrix[3 * 4 + 2] = 1;

    // Apply Perspective matrix
    matMatMult(matrix, tempMatrix);

    // Makes + Apply cam x angle matrix.
    xRotMatrix4(tempMatrix, -myCam->rots[0]);
    matMatMult(matrix, tempMatrix);

    // Makes + Applies cam rotation matrix
    yRotMatrix4(tempMatrix, myCam->rots[1]);
    matMatMult(matrix, tempMatrix);


    glUniformMatrix4fv(skyBoxMatID, 1, GL_TRUE, matrix);

    // skybox cube
    glBindVertexArray(skyboxVAO);
    glActiveTexture(GL_TEXTURE0);

    glUniform1i(skyBoxTexID, 0);

    glBindTexture(GL_TEXTURE_CUBE_MAP, cubemapTexture);

    glDrawArrays(GL_TRIANGLES, 0, 36);

    glBindVertexArray(0);
    glDepthMask(GL_TRUE);

    free(matrix);
    free(tempMatrix);
//    glEnable(GL_DEPTH_TEST);
}

void saveBmp(char *dir, int width, int height, unsigned char *buffer) {
    printf("width: %d\n", width);
    int y;
    for(y = 0; y < height; y++) {
        int x;
        for(x = 0; x < width; x++) {
            printf("%2x ", buffer[y * width + x]);
        }
        printf("\n");
    }
    FILE *file;
    file = fopen(dir, "w");

    if(file == NULL ) {
        printf("error opening file\n");
        return;
    }

    fwrite(buffer, sizeof(unsigned char), width * height, file);

    fclose(file);
}

struct jalbWindow {
    int x, y;
    int widnth, height;
    ArrayList tabList;
};

struct jalbTab {
    int type;
};

void drawWindow(int x, int y, int width, int height) {
    // Background
    renderRect(x, y, width, height, .3, .3, .3, .8);
    //
    int ix = 1;
    int iy = 1;
    renderRect(x+ix, y+iy, width-ix*2, textH, .15, .15, .15, .9);

    iy += textH;
    iy += 2;
    ix += 1;
    renderRect(x+ix, y+iy, width-ix*2, height-iy-2, .4, .4, .4, .8);

    int numTabs = 2;
    int types[2];
}

/** Test */

void makeClickVect(int x, int y, float *ret) {
    /// Make ray.
    float cx = (2.0 * x / SCREEN_WIDTH) - 1;
    // Scale x to aspect ratio.
    cx *= (float)SCREEN_WIDTH/SCREEN_HEIGHT;
    float cy = 1 - (2.0 * y / SCREEN_HEIGHT);

    ret[0] = cx;
    ret[1] = cy;
    ret[2] = -1.0;
    ret[3] = 0.0;
    vectNormalize(ret, 3);

    float buffMat[16] = {0};

    xRotMatrix4(buffMat, -myCam->rots[0]);
    matVectMult(buffMat, ret, ret);

    int i;
    for(i = 0; i < 16; i++) {
        buffMat[i] = 0;
    }
    yRotMatrix4(buffMat, myCam->rots[1]);
    matVectMult(buffMat, ret, ret);

    ret[2] = -ret[2];
}


