#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "jalbArrayList.h"

// There is a way to iterate the arrayList that is more efficient than using index i and the entire length.
// Instead keep track of the arrayListPart and iterate that part then move onto the next.

ArrayList *initArrayList(int size, int elementSize) {
    ArrayList *al = (ArrayList*) malloc(sizeof(ArrayList));
    al->filled = 0;
    al->numElements = size;
    al->elementSize = elementSize;
    al->data = malloc((size_t) (elementSize * size));
    al->next = NULL;

    return al;
}

void freeArrayList(ArrayList *al) {
    ArrayList *temp;
    while(al) {
        temp = al->next;
        free(al->data);
        free(al);
        al = temp;
    }
}

void freeArrayListPointer(ArrayList *al) {
    ArrayList *temp;
    int i;
    while(al) {
        temp = al->next;
        for(i = 0; i < al->filled; i++) {
            free(arrayListGetPointer(al, i));
        }
        free(al);
        al = temp;
    }
}

// This adds what data points to, i would prefer if it just added data.
// This thing has a memory leak. (I dont know if i even tried to fix this so far)
void arrayListAddEnd(ArrayList *al, void *data) {
    while(al->next) {
        al = al->next;
    }
    if(al->filled == al->numElements) {
        al->next = initArrayList(al->numElements, al->elementSize);
        al = al->next;
    }
    int i;
    for(i = 0; i  < al->elementSize; i++) {
        ((char*)al->data)[al->filled * al->elementSize + i] = ((char*)data)[i];
    }
    al->filled++;
}

void arrayListAddEndPointer(ArrayList *al, void *data) {
    while(al->next) {
        al = al->next;
    }
    if(al->filled == al->numElements) {
        al->next = initArrayList(al->numElements, al->elementSize);
        al = al->next;
    }
    ((void**)al->data)[al->filled] = data;
    al->filled++;
}

void arrayListSet(ArrayList *al, int index, void *data){
    while(al && index >= al->filled) {
        index -= al->filled;
        al = al->next;
    }
    if(al && index < al->filled) {
        int i;
        for(i = 0; i  < al->elementSize; i++) {
            ((char*)al->data)[index * al->elementSize + i] = ((char*)data)[i];
        }
    }
}

void arrayListDisplay(ArrayList *al) {
    printf("Display ArrayList\n");
    while(al) {
        printf("[%d]: ", al->filled);
        int i =0;
//        int i = al->filled - 1;
        int j;
        for(; i < al->filled; i++) {
//        for(; i >= 0; i--) {
            printf("[ ");
//            for( j = 0; j < al->elementSize; j++) {
            for( j = al->elementSize - 1; j >= 0; j--) {
                if(((unsigned char*)al->data)[i  * al->elementSize + j] < 16) {
                    printf("0%x ", ((char*)al->data)[i * al->elementSize + j] & 0xff);
                } else {
                    printf("%x ", ((char*)al->data)[i * al->elementSize + j] & 0xff);
                }
            }
            printf("] ");
        }
        printf("\n");
        al = al->next;
    }
}

void arrayListDisplayInt(ArrayList *al) {
    printf("Display ArrayList\n");
    while(al) {
        printf("[%d]: ", al->filled);
        int i =0;
        for(; i < al->filled; i++) {
            printf("[ ");
            printf("%d ", ((int*)al->data)[i]);
            printf("] ");
        }
        printf("\n");
        al = al->next;
    }
}

// If index is to high or lor then return 0;
int arrayListGetInt(ArrayList *al, int index) {
    while(al && index >= al->filled) {
        index -= al->filled;
        al = al->next;
    }
    if(al && index < al->filled) {
        return ((int*)al->data)[index];
    }
    return 0;
}

// If index is to high or low then return 0;
void *arrayListGetPointer(ArrayList *al, int index) {
    while(al && index >= al->filled) {
        index -= al->filled;
        al = al->next;
    }
    if(al && index < al->filled) {
        return ((void**)al->data)[index];
    }
    return 0;
}

void *arrayListGetEndPointer(ArrayList *al) {
    while(al->next) {
        al = al->next;
    }
    return ((void**)al->data)[al->filled - 1];
}

void *arrayListSwitchPointer(ArrayList *al, int index, void *pointer) {
    while(al && index >= al->filled) {
        index -= al->filled;
        al = al->next;
    }
    if(al && index < al->filled) {
        void *temp = ((void**)al->data)[index];
        ((void**)al->data)[index] = pointer;
        return temp;
    }
    return NULL;
}

int isArrayListSumGreater(ArrayList *al, int compare) {
    int i;
    while(al) {
        for(i = 0; i < al->filled; i++) {
            compare -= ((int*)al->data)[i];
            if(compare < 0) {
                return 1;
            }
        }
        al = al->next;
    }
    return 0;
}

void removeArrayListEnd(ArrayList *al) {
    ArrayList *last = NULL;
    while(al->next) {
        last = al;
        al = al->next;
    }
    if(al->filled > 0) {
        al->filled--;
        if(al->filled == 0) {
            if(last) {
                free(last->next);
                last->next = 0;
            }
        }
    }
}

void arrayListRemove(ArrayList *al, int i) {
    while(al && i >= al->filled) {
        i -= al->filled;
        al = al->next;
    }
    if(al) {
        int removed = 0;
        while(i < al->filled) {
            removed = 1;
            int j;
            for(j = 0; j < al->elementSize; j++) {
                ((char*)al->data)[i * al->elementSize + j] = ((char*)al->data)[(i + 1) * al->elementSize + j];
            }
            i++;
        }
        if(removed) {
            al->filled--;
        }
    }
}

int sizeOfArrayList(ArrayList *al) {
    int sum= 0;
    while(al) {
        sum += al->filled;
        al = al->next;
    }
    return sum;
}

int arrayListGetLength(ArrayList *al) {
    int length = 0;
    while(al) {
        length+=al->filled;
        al = al->next;
    }
    return length;
}

void saveArrayListInt(ArrayList *al, FILE *file) {
    // Write the length of the list, the write the ints.
    int length = arrayListGetLength(al);
    fwrite(&length, sizeof(int), 1, file);
    int i;
    for(i = 0; i < length; i++) {
        int ele = arrayListGetInt(al, i);
        fwrite(&ele, sizeof(int), 1, file);
    }
}

void loadArrayListInt(ArrayList *al, FILE *file) {
    int length;
    fread(&length, sizeof(int), 1, file);
    int i;
    for(i = 0; i < length; i++) {
        int ele;
        fread(&ele, sizeof(int), 1, file);
        arrayListAddEnd(al, &ele);
    }
}

// Checks all the previous data to make sure that
// First check entire list, and then find the 1st open slot.
// Aslong as i will be adding to end i dont need to check entire list.
int arrayListAddUnique(ArrayList *al, void *data) {
    while(al) {
        int i = al->filled -1;
        while(i >= 0) {
            // Check to see if the input data is teh same as
            // Casts data to a char* before receiving data from an index.
            if(memcmp(&((char*)al->data)[i * al->elementSize], data, al->elementSize) == 0) {
                // They are the same. So, done.
                return 0;
            }
            i--;
        }
        if(al->filled == al->numElements) {
            // This part is full.
            if(!al->next) {
                // Next is null.
                al->next = initArrayList(al->numElements, al->elementSize);
                al = al->next;
                // I should just add it here and end, instead of hitting up the loop again.
            } else {
                // next is there.
                al = al->next;
            }
        } else {
            // There is space at the end of this list, so add it here
            int i;
            for(i = 0; i  < al->elementSize; i++) {
                ((char*)al->data)[al->filled * al->elementSize + i] = ((char*)data)[i];
            }
            al->filled += 1;
            return 1;
        }
    }
    return 0;
}

// Return 1 if the data Is in the list. 0 if not.
int arrayListContain(ArrayList *al, void *data) {
    while(al) {
        int i = al->filled -1;
        while(i >= 0) {
            // Check to see if the input data is the same as the list data.
            // Casts data to a char* before receiving data from an index.
            if(memcmp(&((char*)al->data)[i * al->elementSize], data, al->elementSize) == 0) {
                // They are the same. So, done.
                return 1;
            }
            i--;
        }
        al = al->next;
    }
    return 0;
}

void *arrayListDataPointer(ArrayList *al, int index) {
    while(index >= al->filled) {
        if(al->next == NULL) {
            return NULL;
        }
        index -= al->filled;
        al = al->next;
    }
    return &((char*)al->data)[al->elementSize * index];
}
