#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <SDL.h>
#include <SDL_image.h>
#include "SDL_ttf.h"

#include "jalbButtonColumn.h"

/** Button Column */

struct buttonColumn *initButtonColumn(int x, int y, int width, int height, int fontW, int fontH, int numButts, int buttH) {
    struct buttonColumn *bc = malloc(sizeof(struct buttonColumn));

    bc->x = x;
    bc->y = y;

    if(buttH * numButts > height){
        buttH = height / numButts;
    }

    bc->width = width;
    bc->height = height;

    bc->fontW = fontW;
    bc->fontH = fontH;

    bc->buttH = buttH;

    bc->numButts = numButts;

    bc->textures = malloc(bc->numButts * sizeof(SDL_Texture*));
    bc->textureWs = malloc(bc->numButts * sizeof(int));

    return bc;
}

void buttColAddButton(struct buttonColumn *bc, char *str, int strLen, int buttIndex, TTF_Font *font, SDL_Color *color, SDL_Renderer *renderer) {
//    printf("buttColAddButton Start\n");
    printf("str: %s\n", str);
    SDL_Surface *txt = TTF_RenderText_Blended(font, str, *color);
    SDL_Texture *tx = SDL_CreateTextureFromSurface(renderer, txt);

    SDL_FreeSurface(txt);
//    printf("buttColAddButton Mid\n");

    printf("buttIndex: %d\n", buttIndex);

    bc->textures[buttIndex] = tx;
    bc->textureWs[buttIndex] = strLen;
//    printf("buttColAddButton End\n");
}

void buttColRender(struct buttonColumn *bc, SDL_Color *topColor, SDL_Renderer *renderer) {
    SDL_Rect rect;
    rect.x = bc->x;
    rect.y = bc->y + bc->height - bc->buttH;

    rect.w = bc->width;
    rect.h = bc->buttH;

//    printf("rect (%d, %d, %d, %d)\n", rect.x, rect.y, rect.w, rect.h);

    int i;
    for(i = 0; i < bc->numButts; i++){
        SDL_SetRenderDrawColor(renderer, topColor->r, topColor->g, topColor->b, 1);
        SDL_RenderDrawRect(renderer, &rect);

        rect.w = bc->textureWs[i] * bc->fontW;
        rect.h = bc->fontH;

        rect.x += (bc->width - rect.w) / 2;
        rect.y += (bc->buttH - rect.h) / 2;
        SDL_RenderCopy(renderer, bc->textures[i], NULL, &rect);
    }
}

int buttColMouseHandle(struct buttonColumn *bc, int mx, int my) {
    mx -= bc->x;
    my -= bc->y;

    if(mx >= 0 && my >= 0){
        if(mx < bc->width){
//            printf("mouse (%d, %d)\n", mx, my);
            int i;
            for(i = 0; i < bc->numButts; i++){
                if(my < bc->height && my > bc->height - bc->buttH * (i + 1)) {
                    return i;
                }
            }
        }
    }
    return -1;
}

void buttColFree(struct buttonColumn *buttCol) {
    if(!buttCol) {
        return;
    }
    int i;
    for(i = 0; i < buttCol->numButts; i++) {
        if(buttCol->textures[i]) {
            printf("freeTex: %d\n", i);
            SDL_DestroyTexture(buttCol->textures[i]);
        }
    }
    free(buttCol->textures);
    free(buttCol->textureWs);

    free(buttCol);
}
