#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <SDL.h>
#include <SDL_image.h>
#include "SDL_ttf.h"

#include <string.h>

#include "jalbArrayList.h"

#include "jalbScrollList.h"

struct scrollList *initScrollList(int x, int y, int width, int height, int fontW, int fontH, int edgeW, int rowsPerButt, int rightText) {
    struct scrollList *sl = malloc(sizeof(struct scrollList));

    sl->x = x;
    sl->y = y;

    sl->width = width;
    sl->height = height;

    sl->fontW = fontW;
    sl->fontH = fontH;

    sl->edgeW = edgeW;

    sl->numCols = (width - edgeW * 2) / fontW;

    sl->numButts = 0;

    sl->rowsPerButt = rowsPerButt;

//    sl->aList = initArrayList(10, sizeof(void*));
    sl->textList = initArrayList(10, sizeof(void*));
    sl->widthList = initArrayList(10, sizeof(int));

    sl->yIndent = 0;
    sl->maxYindent = 0;

    sl->rightText = rightText;

    return sl;
}

void scrollListAddButt(struct scrollList *sl) {
//    printf("scrollListAddButt()\n");

    int texts = sl->rowsPerButt;
    if(sl->rightText) {
        texts *= 2;
    }

    int i;
    for(i = 0; i < texts; i++) {
        arrayListAddEndPointer(sl->textList, NULL);
        int zero = 0;
        arrayListAddEnd(sl->widthList, &zero);
    }
    sl->numButts++;

    int drawHeight = sl->numButts * (sl->edgeW * 3 + sl->rowsPerButt * sl->fontH) + sl->edgeW;

    if(drawHeight > sl->height) {
        // There is a scroll bar.
//        sl->yIndex = 0;
        sl->maxYindent = drawHeight - sl->height;
    }
}

void scrollListSetText(struct scrollList *sl, char *str, int buttIndex, int rowIndex, SDL_Color *color, SDL_Renderer *renderer, TTF_Font *font) {
    int strlen = 0;
    while(str[strlen] != '\0') {
        strlen++;
    }

    int index = buttIndex * sl->rowsPerButt;
    if(sl->rightText) {
        index *= 2;
    }
    index += rowIndex;

    arrayListSet(sl->widthList, index, &strlen);
//    int length = arrayListGetInt(sl->widthList, index);

    SDL_Surface *txt = TTF_RenderText_Blended(font, str, *color);
    SDL_Texture *tx = SDL_CreateTextureFromSurface(renderer, txt);

    SDL_FreeSurface(txt);

    SDL_Texture *old = arrayListSwitchPointer(sl->textList, index, tx);
    if(old != NULL) {
        SDL_DestroyTexture(old);
    }

}

void scrollListRender(struct scrollList *sl, SDL_Renderer *renderer, TTF_Font *font, SDL_Color *color, int highlighted) {
    SDL_Color backColor = {127, 127, 127, 1};

    SDL_SetRenderDrawColor(renderer, backColor.r, backColor.g, backColor.b, backColor.a);

    // Sets the dimensions of each scrll button.
    SDL_Rect rect;
    rect.x = sl->x + sl->edgeW;
    rect.y = sl->y + sl->edgeW;

    rect.w = sl->width - sl->edgeW * 2;

    int buttonHeight = sl->edgeW * 2 + sl->rowsPerButt * sl->fontH;

    rect.h = buttonHeight;

    int drawHeight = sl->numButts * (sl->edgeW + buttonHeight) + sl->edgeW;

    int firstButt = 0;
    int lastButt = sl->numButts;
    int relY = 0;

    int remainingHeight = 0;

    if(drawHeight > sl->height) {
        // There is a scroll bar.
        rect.w -= 10;

        // Draw scroll bar.
        int boxHeight = sl->height * sl->height / drawHeight;
        int boxYindent = sl->yIndent * sl->height / drawHeight;

        SDL_Rect sbrect;
        sbrect.x = sl->x + sl->width - 9;
        sbrect.y = sl->y + boxYindent;
        sbrect.w = 8;
        sbrect.h = boxHeight;

        SDL_RenderFillRect(renderer, &sbrect);

        firstButt = sl->yIndent / (buttonHeight + sl->edgeW);

        relY = sl->yIndent - firstButt * (buttonHeight + sl->edgeW);

        // How many butts can i fit inside the space.
        lastButt = (sl->height + relY) / (buttonHeight + sl->edgeW) + firstButt;

        remainingHeight = (sl->height + relY) - (buttonHeight + sl->edgeW) * (lastButt - firstButt);
    }

    rect.y -= relY;

    int i;
    for(i = firstButt; i < lastButt; i++) {
        // Hight lighting
        if(i == highlighted) {
            SDL_SetRenderDrawColor(renderer, 255, 255, 0, 1);
            int tempY = rect.y;
            rect.x -= sl->edgeW;
            rect.y -= sl->edgeW;
            rect.w += sl->edgeW * 2;
            rect.h += sl->edgeW * 2;

            // Handle top overlap
            if(rect.y < sl->y) {
                rect.y = sl->y;
                rect.h -= relY - sl->edgeW;
            }

            // Handle Bot overlap
            if(i == lastButt - 1 && remainingHeight && remainingHeight < sl->edgeW) {
                rect.h -= sl->edgeW - remainingHeight;
            }

            SDL_RenderFillRect(renderer, &rect);
            rect.y = tempY;
            rect.x += sl->edgeW;
//            rect.y += sl->edgeW;
            rect.w -= sl->edgeW * 2;
//            rect.h -= sl->edgeW * 2;
            rect.h = buttonHeight;
            SDL_SetRenderDrawColor(renderer, backColor.r, backColor.g, backColor.b, backColor.a);
        }
        int tempY = rect.y;
        if(rect.y < sl->y) {
            rect.y = sl->y;
            rect.h -= relY - sl->edgeW;
        }
        // Draw the button
        SDL_RenderFillRect(renderer, &rect);
        rect.y = tempY;
        rect.h = buttonHeight;


        int index = i * sl->rowsPerButt;
        if(sl->rightText) {
            index *= 2;
        }
//        printf("index: %d\n", index);

        rect.y += sl->edgeW;

        int j;
        for(j = 0; j < sl->rowsPerButt; j++) {
            int length = arrayListGetInt(sl->widthList, index + j);
            SDL_Texture *tx = arrayListGetPointer(sl->textList, index + j);

            SDL_Rect srcrect;
            srcrect.x = 0;
            srcrect.y = 0;
            srcrect.w = length * sl->fontW;
            srcrect.h = sl->fontH;

            SDL_Rect dstrect;
            dstrect.x = rect.x + sl->edgeW;
            dstrect.y = rect.y;
            dstrect.w = srcrect.w;
            dstrect.h = srcrect.h;

            // Top overlap.
            tempY = dstrect.y;
            if(i == firstButt && relY > sl->edgeW * 2) {
                srcrect.y += relY - sl->edgeW * 2;
                srcrect.h -= relY - sl->edgeW * 2;

                dstrect.y += relY - sl->edgeW * 2;
                dstrect.h -= relY - sl->edgeW * 2;

                relY -= sl->fontH;
            }

            SDL_RenderCopy(renderer, tx, &srcrect, &dstrect);

            if(sl->rightText) {
                length = arrayListGetInt(sl->widthList, index + sl->rowsPerButt + j);
                tx = arrayListGetPointer(sl->textList, index + sl->rowsPerButt + j);
                dstrect.w = length * sl->fontW;
                srcrect.w = dstrect.w;

                dstrect.x = rect.x + rect.w - sl->edgeW - dstrect.w;

                SDL_RenderCopy(renderer, tx, &srcrect, &dstrect);
            }

            srcrect.y = 0;
            srcrect.h = sl->fontH;

            dstrect.y = tempY;
            dstrect.h = sl->fontH;

            rect.y += sl->fontH;
        }
        rect.y += sl->edgeW * 2;
    }
    // Handle room for partial button at he bottom.
    if(i < sl->numButts) {
        // Some of the highlight and background.
        if(i == highlighted) {
            SDL_SetRenderDrawColor(renderer, 255, 255, 0, 1);

            rect.x -= sl->edgeW;
            rect.y -= sl->edgeW;
            rect.w += sl->edgeW * 2;
            rect.h = remainingHeight;

            SDL_RenderFillRect(renderer, &rect);


            rect.x += sl->edgeW;
            rect.y += sl->edgeW;
            rect.w -= sl->edgeW * 2;
//            rect.h -= sl->edgeW;

            SDL_SetRenderDrawColor(renderer, backColor.r, backColor.g, backColor.b, backColor.a);
        }

        // Draw background
        if(remainingHeight > sl->edgeW) {
            rect.h = remainingHeight - sl->edgeW;
            SDL_RenderFillRect(renderer, &rect);
        }

        remainingHeight -= sl->edgeW * 2;

        int j;
        for(j = 0; j < sl->rowsPerButt; j++) {
            // Draw text0
            if(remainingHeight > 0) {
                int index = i * sl->rowsPerButt;
                if(sl->rightText) {
                    index *= 2;
                }
//            printf("index: %d\n", index);

                int length = arrayListGetInt(sl->widthList, index + j);
                SDL_Texture *tx = arrayListGetPointer(sl->textList, index + j);

                SDL_Rect srcrect;
                srcrect.x = 0;
                srcrect.y = 0;
                srcrect.w = length * sl->fontW;
                srcrect.h = sl->fontH;

                SDL_Rect dstrect;
                dstrect.x = rect.x + sl->edgeW;
                dstrect.y = rect.y + sl->edgeW;
                dstrect.w = srcrect.w;
                dstrect.h = srcrect.h;

                // Text is only partially drawn.
                if(remainingHeight < sl->fontH) {
                    srcrect.h = remainingHeight;
                    dstrect.h = srcrect.h;
                }

                SDL_RenderCopy(renderer, tx, &srcrect, &dstrect);

                remainingHeight -= sl->fontH;
                rect.y += sl->fontH;
            } else {
                break;
            }
        }
    }
}

int scrollListClickButt(struct scrollList *sl, int mx, int my) {
    mx -= sl->x;
    my -= sl->y;

    if(my > sl->height) {
        return -1;
    }

    my += sl->yIndent;

//    printf("mouseRel (%d, %d)\n", mx, my);

    int rightEdge = sl->width - sl->edgeW;

//    int scrollHeight = 0;
//    int scrollY = 0;

    if(sl->maxYindent) {
        rightEdge -= 10;
    }
    if(mx > sl->edgeW && mx < rightEdge) {
        // How to do the y check, i would like to not use a loop, but instead use devision.
        // The part that makes it hard is the gaps inbetwee.
        // I can use devision to find which
        int ySect = my / ((sl->rowsPerButt * sl->fontH) + sl->edgeW * 3);
        my -= ySect * ((sl->rowsPerButt * sl->fontH) + sl->edgeW * 3);
//        printf("next (%d, %d)\n", ySect, my);
        if(my > sl->edgeW) {
            if(ySect < sl->numButts) {
                return ySect;
            }
        }
    }
    return -1;
}

// Only gets ran if there is a scroll bar, returns 1 if the scroll bar is clicked on.
int scrollListClickBar(struct scrollList *sl, int mx, int my) {
    mx -= sl->x;
    my -= sl->y;

    int buttonHeight = sl->edgeW * 2 + sl->rowsPerButt * sl->fontH;
    int drawHeight = sl->numButts * (sl->edgeW + buttonHeight) + sl->edgeW;

    int scrollHeight = sl->height * sl->height / drawHeight;
    int scrollY = sl->yIndent * sl->height / drawHeight;
    if(mx >= sl->width - 9 && mx < sl->width - 1) {
        // scroll bar.
        if(my >= scrollY && my < scrollY + scrollHeight) {
            return 1;
        }
    }
    return 0;
}

void scrollListRemove(struct scrollList *sl, int index) {
    sl->numButts--;

    int texts = sl->rowsPerButt;
    if(sl->rightText) {
        texts *= 2;
    }

    index *= texts;

    int i;
    for(i = 0; i < texts; i++) {
        arrayListRemove(sl->textList, index);
        arrayListRemove(sl->widthList, index);
    }

    int drawHeight = sl->numButts * (sl->edgeW * 3 + sl->rowsPerButt * sl->fontH) + sl->edgeW;

    if(drawHeight > sl->height) {
        // There is a scroll bar.
        sl->maxYindent = drawHeight - sl->height;

        if(sl->yIndent > sl->maxYindent){
            sl->yIndent = sl->maxYindent;
        }
    } else {
        sl->maxYindent = 0;
        sl->yIndent = 0;
    }
}

struct scrollBar *initScrollBar(int screenHeight, int drawHeight, int x, int y) {
    struct scrollBar *sb = malloc(sizeof(struct scrollBar));
    sb->x = x;
    sb->y = y;
    sb->yIndent = 0;
    sb->screenH = screenHeight;
    sb->drawH = drawHeight;
    sb->barH = screenHeight * screenHeight / drawHeight;

    //    printf("barH: %d\n", sb->barH);

    return sb;
}

void scrollBarRender(struct scrollBar *sb, SDL_Renderer *renderer) {
    SDL_SetRenderDrawColor(renderer, 100, 100, 100, 1);

    SDL_Rect rect;
    rect.x = sb->x + 1;
    rect.y = sb->y + sb->yIndent * sb->screenH / sb->drawH;
    rect.w = 8;
    rect.h = sb->barH;

    SDL_RenderFillRect(renderer, &rect);
}

int scrollBarClick(struct scrollBar *sb, int mx, int my) {
    mx -= sb->x;
    my -= sb->y;

    if(mx >= 1 && mx <= 9) {
        if(my > sb->yIndent && my < sb->yIndent + sb->barH) {
            return 1;
        }
    }
    return 0;
}

void scrollBarWheel(int wheelMotionY) {

}

void scrollBarDrag(struct scrollBar *sb, int relY) {
    int ratio = sb->drawH / sb->screenH;

    int maxYindent = sb->drawH - sb->screenH;

    if(sb->yIndent + relY * ratio < 0) {
        sb->yIndent = 0;
    } else if (sb->yIndent + relY * ratio > maxYindent) {
        sb->yIndent = maxYindent;
    } else {
        sb->yIndent += relY * ratio;
    }
}

