#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "jama.h"

/** Matrix Math */

// Sets mat1 to mat1 * mat2;
void matMatMult(float *mat1, float *mat2) {
    float temp[4];
    int i = 0;
    int j = 0;
    for(; i < 4; i++) {
        for(j = 0; j < 4; j++) {
            temp[j] = mat1[i * 4 + 0] * mat2[0 + j] + mat1[i * 4 + 1] * mat2[4 + j] + mat1[i * 4 + 2] * mat2[8 + j] + mat1[i * 4 + 3] * mat2[12 + j];
        }
        for(j = 0; j < 4; j++) {
            mat1[i * 4 + j] = temp[j];
        }
    }
}

/** Rotation Matrices */
// Doesnt need to be zeroed.
void xRotMatrix4(float *mat, float thea) {
    mat[0] = 1;
    mat[1] = 0;
    mat[2] = 0;
    mat[3] = 0;

    mat[1 * 4 + 0] = 0;
    mat[1 * 4 + 1] = cos(thea);
    mat[1 * 4 + 2] = sin(thea);
    mat[1 * 4 + 3] = 0;

    mat[2 * 4 + 0] = 0;
    mat[2 * 4 + 1] = -sin(thea);
    mat[2 * 4 + 2] = cos(thea);
    mat[2 * 4 + 3] = 0;

    mat[3 * 4 + 0] = 0;
    mat[3 * 4 + 1] = 0;
    mat[3 * 4 + 2] = 0;
    mat[3 * 4 + 3] = 1;
}

void yRotMatrix4(float *mat, float thea) {
    mat[0 * 4 + 0] = cos(thea);
    mat[0 * 4 + 1] = 0;
    mat[0 * 4 + 2] = -sin(thea);
    mat[0 * 4 + 3] = 0;

    mat[1 * 4 + 0] = 0;
    mat[1 * 4 + 1] = 1;
    mat[1 * 4 + 2] = 0;
    mat[1 * 4 + 3] = 0;

    mat[2 * 4 + 0] = sin(thea);
    mat[2 * 4 + 1] = 0;
    mat[2 * 4 + 2] = cos(thea);
    mat[2 * 4 + 3] = 0;

    mat[3 * 4 + 0] = 0;
    mat[3 * 4 + 1] = 0;
    mat[3 * 4 + 2] = 0;
    mat[3 * 4 + 3] = 1;
    //return mat;
}

void zRotMatrix4(float *mat, float thea) {
    mat[0 * 4 + 0] = cos(thea);
    mat[0 * 4 + 1] = sin(thea);
    mat[0 * 4 + 2] = 0;
    mat[0 * 4 + 3] = 0;

    mat[1 * 4 + 0] = -sin(thea);
    mat[1 * 4 + 1] = cos(thea);
    mat[1 * 4 + 2] = 0;
    mat[1 * 4 + 3] = 0;

    mat[2 * 4 + 0] = 0;
    mat[2 * 4 + 1] = 0;
    mat[2 * 4 + 2] = 1;
    mat[2 * 4 + 3] = 0;

    mat[3 * 4 + 0] = 0;
    mat[3 * 4 + 1] = 0;
    mat[3 * 4 + 2] = 0;
    mat[3 * 4 + 3] = 1;
    //return mat;
}

void matVectMult(float *mat, float *vect, float *newVect) {
    float *temp = malloc(sizeof(float) * 4);
    int i = 0;
    for(; i < 4; i++) {
        temp[i] = mat[i * 4 + 0] * vect[0] + mat[i * 4 + 1] * vect[1] + mat[i * 4 + 2] * vect[2] + mat[i * 4 + 3] * vect[3];
    }
    for(i = 0; i < 4; i++) {
        newVect[i] = temp[i];
    }
    free(temp);
}

void matVect3Mult(float *mat, float *vect, float *newVect) {
    float *temp = malloc(sizeof(float) * 4);
    int i = 0;
    for(; i < 3; i++) {
        temp[i] = mat[i * 4 + 0] * vect[0] + mat[i * 4 + 1] * vect[1] + mat[i * 4 + 2] * vect[2];
    }
    for(i = 0; i < 3; i++) {
        newVect[i] = temp[i];
    }
    free(temp);
}

/** Vector math */

void vectAdd(float *v1, float *v2, float *vSet, int length) {
    length--;
    while(length>=0) {
        vSet[length] = v1[length] + v2[length];
        length--;
    }
}

// Sets vSet. (doesnt need to be zeroed)
void vectSub(float *v1, float *v2, float *vSet, int length) {
    length--;
    while(length>=0) {
        vSet[length] = v1[length] - v2[length];
        length--;
    }
}


void vectMultScalar(float *vect, float scalar, int length) {
    length--;
    while(length>=0) {
        vect[length] *= scalar;
        length--;
    }
}

float dotProd(float *vect1, float *vect2, int length) {
    float a = 0;
    int i = 0;
    for(; i < length; i++ ) {
        a += vect1[i] * vect2[i];
    }
    return a;
}

float vectNorm(float *vec, int length) {
    float mag =0;

    int i =0;
    for(; i < length; i++) {
        mag += vec[i] * vec[i];
    }
    mag = pow( mag, .5);
    return mag;
}

void vectNormalize(float *vec, int length) {
    float sum = vectNorm(vec, length);
    if(sum == 0) {
        return;
    }
    int i;
    for(i = 0; i < length; i++) {
        vec[i] /= sum;
    }
}

void crossProd3(float *v1, float *v2, float *vSet) {
    vSet[0] = (v1[1] * v2[2] - v2[1] * v1[2]);
    vSet[1] = (v1[2] * v2[0] - v2[2] * v1[0]);
    vSet[2] = (v1[0] * v2[1] - v2[0] * v1[1]);
}

void vectScaleTo(float *vect, float scalar, int length) {
    scalar /= vectNorm(vect, length);
    int i;
    for(i = 0; i < length; i++) {
        vect[i] *= scalar;
    }
}

void axisAlignNewer(float *f0, float *f1, float *f2, float *dx, float *dz) {
    float *v = calloc(3, sizeof(float));
    vectSub(f1, f0, v, 3);
    float *u = calloc(3, sizeof(float));
    vectSub(f2, f0, u, 3);

    // v is on the z axis.
    if(v[0] == 0) {
        *dz = (v[1] / v[2]);
        *dx = ((u[1] - (*dz) * u[2]) / u[0]);
    }
    // If v is on the x axis.
    else if(v[2] == 0) {
        *dx = (v[1] / v[0]);
        *dz = ((u[1] - (*dx) * u[0]) / u[2]);
    }
    // If u is on the z axis.
    else if (u[0] == 0) {
        *dz = (u[1] / u[2]);
        *dx = ((v[1] - (*dz) * v[2]) / v[0]);
    }
    // If u is on the x axis.
    else if(u[2] == 0) {
        *dx = (u[1] / u[0]);
        *dz = ((v[1] - (*dx) * v[0]) / v[2]);
    }
    // If neither are X or Z axis aligned
    else {
        // If i divide the entire v by this scalar then they will have the same X length.
        // If i multiply the entire u by this scalar then they will have the same X length.
        // Scale, subtract to zero the x. devide to get dy/dz
        float sx = v[0] / u[0];

        // I wanna scale and sub Y and Z. since X is def 0 there is no need to do it.
        *dz = (u[1] - v[1] / sx) / (u[2] - v[2] / sx);
        *dx = ((u[1] - (*dz) / u[2]) / u[0]);
    }
    free(v);
    free(u);
}

void quadEq(float a, float b, float c, float *ret) {
    // -b +- rad(b^2 - 4ac) / 2a
    float rad = b *b - 4 * a * c;
    ret[0] = (-b + pow(rad, .5)) / (2 * a);
    ret[1] = (-b - pow(rad, .5)) / (2 * a);
}

/** Data Comparison */
int lowestPosFloat3(float f0, float f1, float f2) {
    if (f0 >= 0) {
        if (f1 >= 0) {
            if (f2 >= 0) {
                // Lowest of 3.
                if (f0 < f1) {
                    if (f0 < f2) {
                        return 0;
                    } else {
                        return 2;
                    }
                } else {
                    if (f1 < f2) {
                        return 1;
                    } else {
                        return 2;
                    }
                }

            } else {
                // Lowest of 2.
                if (f0 < f1) {
                    return 0;
                } else {
                    return 1;
                }
            }
        } else {
            if (f2 >= 0) {
                // Lowest of 2.
                if (f0 < f2) {
                    return 0;
                } else {
                    return 2;
                }
            } else {
                return 0;
            }
        }
    } else if (f1 >= 0) {
        if (f2 >= 0) {
            if (f1 < f2) {
                return 1;
            } else {
                return 2;
            }
        } else {
            return 1;
        }
    } else {
        // if (f2 >= 0)
        return 2;
    }
}

/** Saying */

void sayMat4(char *str, float *mat) {
    printf("%s {\n", str);
    int i = 0;
    int j;
    for(; i < 4; i++) {
        for(j = 0; j < 4; j++) {
            printf("%f", mat[i * 4 + j]);
            if(j < 3) {
                printf(", ");
            }
        }
        if(i < 3) {
            printf("},\n");
        }
    }
    printf("}\n");
}

void sayVect(char *str, float *vect, int length) {
    printf("%s (", str);
    int i = 0;
    for(; i < length; i++) {
        printf("%f, ", vect[i]);
    }
    printf("}\n");
}

// A x B = |a| |b| sinThea * n
float *crossProduct3(float *v1, float *v2) {
    float *vect = malloc(sizeof(float) * 4);
    vect[0] = (v1[1] * v2[2] - v2[1] * v1[2]);
    vect[1] = (v1[2] * v2[0] - v2[2] * v1[0]);
    vect[2] = (v1[0] * v2[1] - v2[0] * v1[1]);
    return vect;
}

int vectVectSame(float *v0, float *v1, int length) {
    int i;
    for(i = 0; i < length; i++) {
        if(v0[i] != v1[i]) {
            return 0;
        }
    }
    return 1;
}




