#include <SDL.h>
#include <SDL_image.h>
#include "SDL_ttf.h"

// Sleep
#include <unistd.h>
#include <fcntl.h>

#include <netinet/in.h>
#include <arpa/inet.h>

#include <limits.h>
#include <math.h>

#include "jama.h"

#include "jalbArrayList.h"
#include "jalbTextBox.h"
#include "jalbButtonArray.h"
#include "jalbServer.h"

#include "jalbTerrain.h"
#include "jalbEntity.h"
#include "jalbGravity.h"

#include "rayFace.h"

#include "float.h"

#define PORTNUM 2300
#define BUFSIZE 12000
#define KEYSIZE 16

/** Functions */
int init();
void initUI();
void initServ();
void initGame();

void closingGame();
void closingGame();

void gLoop();
void inputEvents(SDL_Event e);
void handleMouse(int x, int y);

void tic();
void gameTic();

void renderUI();

int buffToInt(unsigned char *buff);
void intToBuffer(int i, unsigned char *buff);

void ticlyIn(unsigned char *buff, int length, struct sockaddr_in *remaddr, socklen_t addrlen);
void ticlyTimeOut(struct jalbProfile *pro);
void ticlyOutGame(struct jalbProfile *pro);

void sendEntsTicly(struct jalbProfile *pro);

void sendNewChunks(struct jalbProfile *prof);
void profDisconnect(struct jalbProfile *prof);

/** Data Out */
void sendMovedVerts(struct jalbProfile *pro);
void initSendTers(struct jalbProfile *pro);
int buffChunk(struct jalbTerrain *ter);
int buffChunkMin(struct jalbTerrain *ter);
void sendEnts(struct jalbProfile *pro);
int buffEnt(struct jalbEntity *ent);
void sendPoUpdate(struct jalbProfile *pro);
void sendKey(struct jalbProfile *pro);

int terRadius(int x, int y);
void getTerRel(int terR, int i, int *x, int *y);
struct jalbTerrain *getTerFromCoords(ArrayList *terList, int x, int y);

void profCreation(unsigned char *buff, int length, struct sockaddr_in *remaddr, socklen_t addrlen);
void profLogin(unsigned char *buff, int length, struct sockaddr_in *remaddr, socklen_t addrlen);

int isUsernameInUse(unsigned char *name);

void makeKey(unsigned char *buff);

int strCompare(unsigned char *str0, unsigned char *str1, int maxlen);

struct jalbProfile *keyToProf(unsigned char *buf);

SDL_Texture* loadTexture( char *path, int *retWidth, int *retHeight);

void saveProfs(char *dir, ArrayList *profList);
int loadProfs(char *dir, ArrayList *profList);
void saveProf(struct jalbProfile *pro, FILE *file);
void loadProf(struct jalbProfile *pro, FILE *file);

void saveChunks(char *dir, ArrayList *chunkList);
void saveChunk(struct jalbTerrain *chunk, FILE *file);

// txtBox add
void textBoxAddInt(struct textBox *textB, char *str, int var);
void textBoxAddStr(struct textBox *textB, char *str);
void textBoxAddFloat(struct textBox *textB, char *str, float var);

void addFloatTb(char *str, double f);

void rayCollideTerArray(float *loc, float *vec, struct jalbGravity *gravMem, int *retFace, int *retChunk);
void terEdit(float *loc, float *vec, struct jalbGravity *memObj, int push);
void terEditNew(float *loc, float *vec, struct jalbGravity *gravMem, int push, int editType);
void brush0(float *loc, float *editVect, void *terArray);
void brush1(float *loc, float *editVect, int *chunkCoords);

void brushTexi(float *loc, int *chunkCoords, int texi);

void brushNormalInOut(float *loc, int faceIndex, int *chunkCoords, float radius);
int brushNormChunk(struct jalbTerrain *ter, ArrayList *todo, ArrayList *doneEdges, int *chunkCoords, int *todoVert, float *loc, int *temp, float radius, float *normSum);
void brushNormVert(struct jalbTerrain *ter, ArrayList *todo, ArrayList *doneEdges, int *chunkCoords, int *todoVert, float *loc, int *temp, float radius);

void brushNormal(float *loc, int *chunkCoords, float radius, int push);
void brushSmooth(float *loc, int faceIndex, int *chunkCoords, float radius);

void vertEdit(struct jalbTerrain *editTer, int vertIndex, float *editVect);

void gravTest0(struct jalbEntity *ent);
void gravTest1(struct jalbEntity *ent);

void offChunkCheck(struct jalbGravity *gravMem);
float sigmoid2(float x);

int addTerVertUnique(ArrayList *todo, ArrayList *doneEdge, int *inVert);

/** Variables */

//Screen dimension constants
const int SCREEN_WIDTH = 1000;
const int SCREEN_HEIGHT = 600;

//The window we'll be rendering to
SDL_Window* gWindow = NULL;

SDL_Renderer *gRenderer;

/** Vars for gLoop Below */
int ticksPerSecond = 20;
int mpt;
unsigned int lastSec = 0;
unsigned int nextTick = 0;
unsigned int ticksThisSecond = 0;
int running = 0;
/** Vars for gLoop Above */

int servOn = 0;

/** TTF Vars*/
TTF_Font *font;
int textH = 20;
int textW;

SDL_Color color = {0, 0, 0, 1};
SDL_Color backColor = {255, 255, 255, 1};

SDL_Texture *theme;
int themeW, themeH;
struct textBox *tb;
struct buttonArray *bc;

int edgeW = 4;

// Input vars
int mouseDownLeft = 0;

int bufIndex = 0;
unsigned char buf[BUFSIZE];

struct sockaddr_in remaddr; /* remote address */
socklen_t addrlen;
int recvlen; /* # bytes received */

int mysocket = -1; /* socket used to listen for incoming connections */

// Profile
ArrayList *profList = NULL;
char *dirProf = "res/profiles";

// Terrain
ArrayList *terList = NULL;
char *dirWorld = "res/world";

int terEdge = 5;
float terScale = 10;

int chunkArrayCoords[7 * 2] = {0, 0, -1, 1, 0, 1, 1, 0, 1, -1, 0, -1, -1, 0};

// Entities
ArrayList *entList = NULL;
char *dirEntity = "res/entities";

int returnWaitTime = 1000;
int returnFailMax = 3;

int spawnCoords[2] = {0, 0};

// Spawn x chunks around each entity.
int spawnR = 11; // Client max render is spawnR-1.

/// Gravity vars
int detectRadius = .5;
float indentRadius = .51;
float littleG = -.6;

struct bag {
    int type;
    int slots;
    float compWeight; // Compressed item.
    int compSlots;
    ArrayList buffer;
};

/// Items
struct itemPart {
    int type;
    int material;
    int quality;
};

struct itemTool {
    int headType;
    int headMat;
    int headQual;

    int shaftType;
    int shaftMat;
    int shaftQual;
};

struct itemArmor {
    int type;
    int material;
    int quality;
};

struct itemGlob {
    int type;
    int stack;
};

/// Functions

int main(int argc, char *args[]) {
    if(!init()) {
        printf("Init Failed\n");
    } else {
        gLoop();
        closingGame();
    }

    return 0;
}

/** Init */

int init() {
    //Initialize SDL
    if( SDL_Init( SDL_INIT_VIDEO ) < 0 ) {
        printf( "SDL could not initialize! SDL Error: %s\n", SDL_GetError() );
        return 0;
    }
    // Is this only necessary for 3d rendering?
    //Use OpenGL 3.1 core
    SDL_GL_SetAttribute( SDL_GL_CONTEXT_MAJOR_VERSION, 3 );
    SDL_GL_SetAttribute( SDL_GL_CONTEXT_MINOR_VERSION, 1 );
    SDL_GL_SetAttribute( SDL_GL_CONTEXT_PROFILE_MASK, SDL_GL_CONTEXT_PROFILE_CORE );

    gWindow = SDL_CreateWindow( "Footh Incremental: Server 10", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, SCREEN_WIDTH, SCREEN_HEIGHT, SDL_WINDOW_OPENGL |SDL_WINDOW_SHOWN );

    if( gWindow == NULL ) {
        printf( "Window could not be created! SDL Error: %s\n", SDL_GetError() );
        return 0;
    }
    gRenderer = SDL_CreateRenderer(gWindow, 0, SDL_RENDERER_ACCELERATED);

    initUI();

    return 1;
}

void initUI() {
    /// Init TTF.
    if(TTF_Init()==-1) {
        printf("TTF_Init: %s\n", TTF_GetError());
        exit(2);
    }

    font = TTF_OpenFont("res/UbuntuMono-R.ttf", textH);

    TTF_SizeText(font,"a",&textW, NULL);

    printf("textW: %d\n", textW);
    if(!font) {
        printf("TTF_OpenFont: %s\n", TTF_GetError());
        // handle error
    }

    // Init theme image.
    theme = loadTexture("res/Footh.png", &themeW, &themeH);

    int y = themeH + edgeW * 2;

    // Init button column
    int buttColH = SCREEN_HEIGHT - themeH - edgeW * 4;
//    bc = initButtonColumn(edgeW, y, themeW, buttColH, textW, textH, 1, 200);
    bc = initButtonArray(0, y, themeW, textW, textH, buttColH, edgeW, 1, 1);
    buttArSetText(bc, gRenderer, "Start", 0, 0, &color, font);
//    buttColAddButton(bc, "Start", 5, 0, font, &color, gRenderer);

    int x = themeW + edgeW * 2;

    int textBoxW = SCREEN_WIDTH - themeW - edgeW * 3;
    int textBoxH = SCREEN_HEIGHT - edgeW * 2;
    // Init text box.
    tb = initTextBox(x, edgeW, textBoxW, textBoxH, textW, textH);

    char *a = malloc(256 * sizeof(char));
    strcpy(a, "initUI: Completed");
    txtBoxAddString(tb, a, 17, font, &color, gRenderer);
    free(a);
}

void initServ() {
    // Init for networking.
    /** Make the server socket, set to non blocking. */
    addrlen = sizeof(remaddr); /* length of addresses */

//    struct sockaddr_in dest; /* socket info about the machine connecting to us */
    struct sockaddr_in serv; /* socket info about our server */
//    socklen_t socksize = sizeof(struct sockaddr_in);

    memset(&serv, 0, sizeof(serv));           /* zero the struct before filling the fields */
    serv.sin_family = AF_INET;                /* set the type of connection to TCP/IP */
    serv.sin_addr.s_addr = htonl(INADDR_ANY); /* set our address to any interface */
    serv.sin_port = htons(PORTNUM);           /* set the server port number */

    /* DGRAM makes it UDP. */
    mysocket = socket(AF_INET, SOCK_DGRAM, 0);

    printf("mysocket: %d\n", mysocket);

    /* bind serv information to mysocket */
    bind(mysocket, (struct sockaddr *)&serv, sizeof(struct sockaddr));

    printf("mysocket: %d\n", mysocket);

    /* Set to non-blocking. */
    int flags = fcntl(mysocket, F_GETFL);
    flags |= O_NONBLOCK;
    fcntl(mysocket, F_SETFL, flags);
}

void initGame() {
    // Init for 3d game.
    servOn = 1;

    // Terrain
    terList = initArrayList(10, sizeof(void*));
    int worldLoaded = loadWorld(dirWorld, terList);

    printf("Load World: %d\n", worldLoaded);

    if(!worldLoaded) {
        // Load chunks in a spawnRadius around (coX, coY).
        embedSpawnTer(terList, spawnR, 0, 0, terScale, terEdge);
    }

//    struct jalbTerrain *ter = arrayListGetPointer(terList, 0);
//    printf("ter->numVerts: %d\n", ter->numVerts);
//    exit(0);

    // Profiles
    profList = initArrayList(10, sizeof(void*));
    loadProfs(dirProf, profList);

    // Entities.
    entList = initArrayList(10, sizeof(void*));
    int entsLoaded = loadEntityList(dirEntity, entList);

    printf("entsLoaded: %d\n", entsLoaded);
    textBoxAddInt(tb, "entsLoaded (", entsLoaded);

    if(entsLoaded == -1) {
        // File not there.
        struct jalbEntity *ent = malloc(sizeof(struct jalbEntity));

        ent->ruleSet = 1;

        ent->gravMem = malloc(sizeof(struct jalbGravity));

        ent->globalId = 0;
        ent->modelIndex = 0;
        ent->scale = 2;

        ent->gravMem->position[0] = 5;
        ent->gravMem->position[1] = 1;
        ent->gravMem->position[2] = 5;

        ent->gravMem->rots[0] = 0;
        ent->gravMem->rots[1] = 0;
        ent->gravMem->rots[2] = 0;

        ent->gravMem->chunkCoords[0] = 0;
        ent->gravMem->chunkCoords[1] = 0;

        ent->gravMem->chunkCoordsOld[0] = 0;
        ent->gravMem->chunkCoordsOld[1] = 0;

        ent->gravMem->moveVect[0] = 0;
        ent->gravMem->moveVect[1] = 0;
        ent->gravMem->moveVect[2] = 0;

        ent->gravMem->tempVect[0] = 0;
        ent->gravMem->tempVect[1] = 0;
        ent->gravMem->tempVect[2] = 0;

        ent->gravMem->maxMoveSpeed = 2;
        ent->gravMem->curMoveSpeed = 0;
        ent->gravMem->addMoveSpeed = .4;
        ent->gravMem->midMoveSpeed = 0;

        ent->gravMem->jumping = 0;

//        fillTerArray(ent->gravMem, terList);
//        polyGrounded(ent->gravMem, detectRadius, terList);
//        gravTest0(ent);
//        gravTest1(ent);

        setjalbGravity(ent->gravMem, terList);

        arrayListAddEndPointer(entList, ent);
    }

//    int numChunks = arrayListGetLength(terList);
//    // 45, 46, 56
//    int i;
//    for(i = 0; i < numChunks; i++) {
//        struct jalbTerrain *ter = arrayListGetPointer(terList, i);
//        if(ter->chunkCoords[0] == 0 && ter->chunkCoords[1] == 0) {
//            ter->verts[45].loc[1] = -1;
//            ter->verts[45].texi = 1;
//
//            ter->verts[46].loc[1] = -1;
//            ter->verts[46].texi = 1;
//
//            ter->verts[56].loc[1] = -1;
//            ter->verts[56].texi = 1;
//        }
//    }

}

// Rand when closing the program.
void closingServ() {
    // UI
    txtBoxFree(tb);
    tb = NULL;
    buttArFree(bc);
    bc = NULL;

    // SDL renderer.
    SDL_DestroyRenderer(gRenderer);
}

void closingGame() {
    printf("profList: %p\n", profList);
    if(profList) {
        saveProfs(dirProf, profList);
    }

    if(terList) {
        saveChunks(dirWorld, terList);
    }

    SDL_DestroyRenderer(gRenderer);
    SDL_DestroyWindow(gWindow);
}

/** Loop + Tic */
void gLoop() {
    textBoxAddStr(tb, "gLoop: Starting");

    //Main loop flag
    mpt = 1000 / ticksPerSecond;
    running = 1;

    //Event handler
    SDL_Event e;

    //Enable text input
    SDL_StartTextInput();

    //While application is running
    while( running ) {
        if(SDL_GetTicks() >= nextTick) {
            ticksThisSecond++;
            nextTick += mpt;

            if (SDL_GetTicks() - lastSec > 1000) {
                if (ticksThisSecond < ticksPerSecond - 1 || ticksThisSecond > ticksPerSecond + 1) {
                    if (SDL_GetTicks() > 2000) {
                        printf("Ticks this second: %u\n", ticksThisSecond);
                        printf("SDL_GetTicks(): %u\n", SDL_GetTicks());
                        printf("nextTick: %u\n", nextTick);
                    }
                }

                ticksThisSecond = 0;
                lastSec = (SDL_GetTicks());
            }

            inputEvents(e);

            tic();

//            if(mode == 3 || mode == 4) {
//                GLtic();
//            } else {
//                UItic();
//            }
            //Update screen
            SDL_GL_SwapWindow( gWindow );
        }
    }

    //Disable text input
    SDL_StopTextInput();
}

void tic() {
//    printf("\ntic()\n");
    if(servOn) {
        gameTic();
    }
    renderUI();
}

void gameTic() {
//    printf("gameTic()\n");
    /// Ticly In.
    /* start listening, allowing a queue of up to 1 pending connection */
    if ((recvlen = recvfrom(mysocket, buf, BUFSIZE, 0, (struct sockaddr *)&remaddr, &addrlen)) > 0) {
        buf[recvlen] = 0;

        printf("received message: (%d bytes) (%02X first) (%02X postKey)\n", recvlen, buf[0], buf[1 + KEYSIZE]);

        ticlyIn(buf, recvlen, &remaddr, addrlen);
    }

    /// Game Logic.
    // Iterate the profiles, take this inputs and turn them into entity moveVects.
    int numProfs = arrayListGetLength(profList);
    int numEnts = arrayListGetLength(entList);
    int i;
    for(i = 0; i < numProfs; i++) {
        struct jalbProfile *pro = arrayListGetPointer(profList, i);
        if(pro->stage == 0) {
            continue;
        }
        struct jalbEntity *ent = arrayListGetPointer(entList, pro->entity);

        struct jalbGravity *gMem = ent->gravMem;
        // Make the velocity vector here.
        gMem->tempVect[0] = 0;
        gMem->tempVect[1] = 0;
        gMem->tempVect[2] = 0;

        gMem->jumping = 0;
        int j;
        // printf("pressed: (");
        for(j = 0; j < numServKeyMaps; j++) {
            if(pro->keyMapBuf[j]) {
                // printf("%d, ", j);
                if(j == skmWalkForward) {
                    gMem->tempVect[2] += 1;
                } else if(j == skmWalkBackwards) {
                    gMem->tempVect[2] -= 1;
                } else if(j == skmStrafeLeft) {
                    gMem->tempVect[0] -= 1;
                } else if(j == skmStrafeRight) {
                    gMem->tempVect[0] += 1;
                } else if(j == skmRotateleft) {
                    gMem->rots[1] += M_PI * 9 / 180;
                    gMem->rotChanged = 1;
                } else if(j == skmRotateright) {
                    gMem->rots[1] -= M_PI * 9 / 180;
                    gMem->rotChanged = 1;
                } else if(j == skmJump) {
                    gMem->jumping = 1;
                } else if(j >= skmHotBar0 && j <= skmHotBar5) {
                    ent->editType = j - skmHotBar0;
                }
            }
        }
        // printf(")\n");
        if(ent->ruleSet == 0) {
            // Player.
            if(gMem->tempVect[0] != 0 || gMem->tempVect[1] != 0 || gMem->tempVect[2] != 0) {
                // moveInputs is not 0 so deal with it.

                // Rotate the vect to face the character yAngle.
                // float *rots = ent->gravMem->rots;
                // float tx = sin(-rots[1]) * gMem->tempVect[2] + sin(-rots[1] - (1.0/2 * M_PI)) * gMem->tempVect[0];
                // gMem->tempVect[2] = cos(-rots[1]) * gMem->tempVect[2] + cos(-rots[1] - (1.0/2 * M_PI)) * gMem->tempVect[0];
                // gMem->tempVect[0] = tx;
                float tempMat[16] = {0};
                yRotMatrix4(tempMat, ent->gravMem->rots[1]);
                matVect3Mult(tempMat, ent->gravMem->tempVect, ent->gravMem->tempVect);

                vectNormalize(gMem->tempVect, 3);
            }
        } else if(ent->ruleSet == 1) {
            // Admin.
            if(gMem->tempVect[0] != 0 || gMem->tempVect[1] != 0 || gMem->tempVect[2] != 0) {
                float tempMat[16] = {0};
                xRotMatrix4(tempMat, ent->gravMem->rots[0]);
                matVect3Mult(tempMat, ent->gravMem->tempVect, ent->gravMem->tempVect);

                yRotMatrix4(tempMat, ent->gravMem->rots[1]);
                matVect3Mult(tempMat, ent->gravMem->tempVect, ent->gravMem->tempVect);
            } else {
                ent->gravMem->tempVect[0] = 0;
                ent->gravMem->tempVect[1] = 0;
                ent->gravMem->tempVect[2] = 0;
            }
        }
    }
    // Iterate the entites doing their thing.
    for(i = 0; i < numEnts; i++) {
        struct jalbEntity *ent = arrayListGetPointer(entList, i);

        if(ent->ruleSet == 0) {
            // Player
            ent->gravMem->locChanged = 0;

            makeMoveVect(ent->gravMem);
            ent->gravMem->midMoveSpeed = ent->gravMem->curMoveSpeed;
            if(ent->gravMem->midMoveSpeed != 0) {
                // Set old
                float posOld[3];
                posOld[0] = ent->gravMem->position[0];
                posOld[1] = ent->gravMem->position[1];
                posOld[2] = ent->gravMem->position[2];
                // Move Funct.
                gravMoveStart(ent->gravMem, terList);

                // exit(0);

                if(!vectVectSame(posOld, ent->gravMem->position, 3)) {
                    ent->gravMem->locChanged = 1;
                }
            }
        } else if(ent->ruleSet == 1) {
            // Admin
            if(ent->gravMem->tempVect[0] != 0 || ent->gravMem->tempVect[1] != 0 || ent->gravMem->tempVect[2] != 0) {
                ent->gravMem->position[0] += ent->gravMem->tempVect[0];
                ent->gravMem->position[1] += ent->gravMem->tempVect[1];
                ent->gravMem->position[2] += ent->gravMem->tempVect[2];
                ent->gravMem->locChanged = 1;

                offChunkCheck(ent->gravMem);
            }
        }
    }

    /// Ticly Out. + Prof Logic.
    for(i = 0; i < numProfs; i++) {
        struct jalbProfile *pro = arrayListGetPointer(profList, i);
        if(pro->stage == psNotConn) {
            continue;
        } else if(pro->stage == psGame) {
            ticlyOutGame(pro);
        } else {
            ticlyTimeOut(pro);
        }
    }

    /// Wipe Tic specific data.
    // Ents
    for(i = 0; i < numEnts; i++) {
        struct jalbEntity *ent = arrayListGetPointer(entList, i);
        ent->gravMem->locChanged = 0;
        ent->gravMem->rotChanged = 0;
    }

    // Chunks movedVert list. Should this list include added verts. They can be sent as one? Some way for the client to decypher the difference.
    // Delete all list parts after the initial, and then set the initial to 0.
    int numChunks = arrayListGetLength(terList);
    for(i = 0; i < numChunks; i++) {
        struct jalbTerrain *chunk = arrayListGetPointer(terList, i);
        if(chunk->movedVerts->next) {
            freeArrayList(chunk->movedVerts->next);
            chunk->movedVerts->next = NULL;
        }
        chunk->movedVerts->filled = 0;
    }
}

/** Input */

void inputEvents(SDL_Event e) {
    /** Input Events */
    //Handle events on queue
    while( SDL_PollEvent( &e ) != 0 ) {
        //User requests quit
        if( e.type == SDL_QUIT ) {
            running = 0;
        } else if(e.type == SDL_KEYDOWN) {
            if(e.key.keysym.sym == SDLK_SPACE) {
//                char *a = malloc(22 * sizeof(char));
//                strcpy(a, "1234567 890123 abcdef");
//                txtBoxAddString(tb, a, 21, font, &color, gRenderer);
//                free(a);
//                txtBoxAddString(tb, "newLine", 7, font, &color, gRenderer);
            }
        } else {
            // Mouse press.
            if(e.type == SDL_MOUSEBUTTONDOWN) {
                int x = 0, y = 0;
                SDL_GetMouseState( &x, &y );
                if (e.button.button == SDL_BUTTON_LEFT) {
                    mouseDownLeft = 1;
                    handleMouse(x, y);
                }
            }
        }
    }
}

void handleMouse(int x, int y) {
    if(mouseDownLeft) {
//        int buttIndex = buttColMouseHandle(bc, x, y);
        int buttIndex = buttArClick(bc, x, y);
//        printf("buttIndex: %d\n", buttIndex);
        if(buttIndex == 0) {
            // Start
            if(!servOn) {
                txtBoxAddString(tb, "Server: Starting", 16, font, &color, gRenderer);
                initServ();
                initGame();
                buttArSetText(bc, gRenderer, "Stop", 0, 0, &color, font);
            } else {
                // Server Stop
                textBoxAddStr(tb, "Server: Stopping + Saving");

                closingGame();

                buttArSetText(bc, gRenderer, "Start", 0, 0, &color, font);
            }
        }
    }
}

/** Render */

void renderUI() {
    // White background.
    SDL_Rect rect;
    rect.x = 0;
    rect.y = 0;
    rect.w = SCREEN_WIDTH;
    rect.h = SCREEN_HEIGHT;

    SDL_SetRenderDrawColor(gRenderer, 255, 255, 255, 1);
    SDL_RenderFillRect(gRenderer, &rect);

    // Theme
    rect.x = edgeW;
    rect.y = edgeW;
    rect.w = themeW;
    rect.h = themeH;
    SDL_RenderCopy(gRenderer, theme, NULL, &rect);

    // Button Column
//    buttColRender(bc, &color, gRenderer);
    buttArRender(bc, gRenderer);

    // Text Box
    txtBoxRender(tb, gRenderer);
}

/** Network Macro */

void intToBuffer(int i, unsigned char *buff) {
    *(int*)buff = htonl(i);
}

int buffToInt(unsigned char *buff) {
    return ntohl(*(int*)buff);
}

/** Ticly Networking */

// Reads first byte, No loop.
void ticlyIn(unsigned char *buff, int length, struct sockaddr_in *remaddr, socklen_t addrlen) {
    if(buff[0] == (char) coWakeup) {
        // Display ip.
        char addr[256];
        inet_ntop(AF_INET, &(remaddr->sin_addr), addr, addrlen);
        int strlen = 0;
        char a[256];
        strncpy(a, "Wake Up: (", 256);
        while(a[strlen] != '\0') {
            strlen++;
        }
        strncat(a, addr, 256 - strlen);
        while(a[strlen] != '\0') {
            strlen++;
        }
        strncat(a, ")", 256 - strlen);
        while(a[strlen] != '\0') {
            strlen++;
        }
        txtBoxAddString(tb, a, strlen, font, &color, gRenderer);

        // Return wakeup call.
        buf[0] = soWakeup;
        int nsent = sendto(mysocket, buf, 1, 0, (struct sockaddr*) remaddr, addrlen);
        if(nsent < 1) {
            printf("send Failed\n");
        }
    } else if(buff[0] == (char) coNewAccount) {
        // Client making an account.
        if(length == 513) {
            char *a = malloc(256 * sizeof(char));
            strcpy(a, "New Profile Creation: (");
            int strlen = 0;
            while(a[strlen] != '\0') {
                strlen++;
            }
            strncat(a, (char*) &buff[1], 256 - strlen);
            while(a[strlen] != '\0') {
                strlen++;
            }
            strncat(a, ") (", 256 - strlen);
            while(a[strlen] != '\0') {
                strlen++;
            }
            strncat(a, (char*) &buff[257], 256 - strlen);
            while(a[strlen] != '\0') {
                strlen++;
            }
            strncat(a, ")", 256 - strlen);
            strlen += 1;

            txtBoxAddString(tb, a, strlen, font, &color, gRenderer);
            free(a);

            /// Now check the data.
            // See if it is not a known profile.
            profCreation(buff, length, remaddr, addrlen);
        }
    } else if(buff[0] == (char) coLogin) {
        // Client logging in to an account.
        if(length == 517) {
            char *a = malloc(256 * sizeof(char));
            strcpy(a, "Profile Log In: (");
            int strlen = 0;
            while(a[strlen] != '\0') {
                strlen++;
            }
            strncat(a, (char*) &buff[1], 256 - strlen);
            while(a[strlen] != '\0') {
                strlen++;
            }
            strncat(a, ") (", 256 - strlen);
            while(a[strlen] != '\0') {
                strlen++;
            }
            strncat(a, (char*) &buff[257], 256 - strlen);
            while(a[strlen] != '\0') {
                strlen++;
            }
            strncat(a, ")", 256 - strlen);
            strlen += 1;

            txtBoxAddString(tb, a, strlen, font, &color, gRenderer);
            free(a);

            /// Now check the data.
            // See if it is a known profile.
            profLogin(&buff[1], 516, remaddr, addrlen);
        }
    } else if(buff[0] == (char) coUpdate) {
        printf("coUpdate: %d (getTic: %d)\n", coUpdate, SDL_GetTicks());
        // Update received, could be a couple of reasons.
        if(length == 1 + KEYSIZE) {
            // Iterate profiles to find the correct key.
            struct jalbProfile *pro;
            pro = keyToProf(&buff[1]);
            if(pro != NULL) {
                textBoxAddStr(tb, "coReturn");
                pro->failCount = 0;
                pro->lastRecv = SDL_GetTicks();

                textBoxAddInt(tb, "Update: Profile (stage: ", pro->stage);
                if(pro->stage == psKeyWait) {
                    // Receiving key confirm, move onto terrain inits.
                    pro->stage = psTerWait;
                    pro->curCounter = 0;
                    pro->oldCounter = 0;
                    textBoxAddStr(tb, "Log In: Completed + Confirmed");
                } else if(pro->stage == psTerWait) {
                    // Check to see if all the chunks have been sent.
                    int numChunks = arrayListGetLength(terList);
                    if(pro->curCounter >= numChunks) {
                        // Move onto ents.
                        textBoxAddStr(tb, "doneSendingChunks");
                        pro->stage = psEntWait;
                        pro->curCounter = 0;
                        pro->oldCounter = 0;
                        sendEnts(pro);
                    } else {
                        // Keep sending ter.
                        pro->oldCounter = pro->curCounter;
                        initSendTers(pro);
                    }
                } else if(pro->stage == psEntWait) {
                    // Check to see if all the ents have been sent.
                    int numEnts = arrayListGetLength(entList);
                    if(pro->curCounter >= numEnts) {
                        // Move onto game.
                        textBoxAddStr(tb, "doneSendingEnts");
                        pro->stage = psCapWait;
                        pro->curCounter = 0;
                        pro->oldCounter = 0;
                        printf("sendCapstone from data input\n");
                        sendPoUpdate(pro);
                    } else {
                        // Keep sending ents.
                        pro->oldCounter = pro->curCounter;
                        printf("wait, fail, resend\n");
                        sendEnts(pro);
                    }
                } else if(pro->stage == psCapWait) {
                    // Receiving confirmation of the capstone.
                    textBoxAddStr(tb, "doneSendingCap");
                    pro->stage = psGame;
                    pro->curCounter = 0;
                    pro->oldCounter = 0;
                } else if(pro->stage == psGame) {
                    textBoxAddStr(tb, "received poll");
                }
            }
        }
    } else if(buff[0] == coGameTic) {
        // Taking in client inputs.
        struct jalbProfile *pro = keyToProf(&buf[1]);
        if(pro != NULL) {
            pro->failCount = 0;
            pro->lastRecv = SDL_GetTicks();

//            textBoxAddInt(tb, "ticlyIn (", bufIndex);
            bufIndex = 1 + KEYSIZE;
            // While there is unread data.
            while(bufIndex < length) {
                if(buf[bufIndex] == cosKey) {
//                    textBoxAddStr(tb, "ticly Key");
                    // Key
                    bufIndex += 1;
                    int numKeys = buffToInt(&buf[bufIndex]);
                    bufIndex += sizeof(int);
                    int i;
                    for(i = 0; i < numKeys; i++) {
                        int index = buffToInt(&buf[bufIndex]);
                        bufIndex += sizeof(int);
                        char val = buf[bufIndex];
                        bufIndex += sizeof(char);
                        // int val = buffToInt(&buf[bufIndex]);
                        // bufIndex += sizeof(int);
                        if(val == 1) {
                            if(index < numServKeyMaps) {
                                pro->keyMapBuf[index] = 1;
                            }
                        } else if (val == 2) {
                            if(index < numServKeyMaps) {
                                pro->keyMapBuf[index] = 0;
                            }
                        }
                    }
                }
                if(buf[bufIndex] == cosMouse) {
                    printf("ticly Ray (%d ->\n", bufIndex);
//                    textBoxAddStr(tb, "ticly Ray (%d ->\n", bufIndex);
                    // Ray (mouse)
                    bufIndex += 1;
                    float loc[3];
                    memcpy(loc, &buf[bufIndex], 3 * sizeof(float));
                    bufIndex += 3 * sizeof(float);
                    sayVect("loc", loc, 3);

                    float ray[3];
                    memcpy(ray, &buf[bufIndex], 3 * sizeof(float));
                    bufIndex += 3 * sizeof(float);
                    sayVect("ray", ray, 3);

                    vectScaleTo(ray, .6, 3);

                    char id = buf[bufIndex];
                    bufIndex += 1;
//                    printf("id: %d\n", id);

                    struct jalbEntity *ent = arrayListGetPointer(entList, pro->entity);

                    if(id == 0) {
//                        terEdit(loc, ray, ent->gravMem, 1);
                        terEditNew(loc, ray, ent->gravMem, 1, ent->editType);
                    } else if(id == 1) {
//                        terEdit(loc, ray, ent->gravMem, 0);
                        terEditNew(loc, ray, ent->gravMem, 0, ent->editType);
                    }
                }
                if(buf[bufIndex] == cosSetRots) {
                    printf("bufIndex: %d\n", bufIndex);
                    bufIndex += 1;

                    struct jalbEntity *ent = arrayListGetPointer(entList, pro->entity);

                    sayVect("preR", ent->gravMem->rots, 3);
                    memcpy(ent->gravMem->rots, &buf[bufIndex], 3 * sizeof(float));
                    sayVect("postR", ent->gravMem->rots, 3);
                    bufIndex += 3 * sizeof(float);

                    ent->gravMem->rotChanged = 1;
                }
            }
        }
    } else if(buff[0] == (char) coDisconnect) {
        struct jalbProfile *pro;
        pro = keyToProf(&buff[1]);
        if(pro != NULL) {
            profDisconnect(pro);
        }
    }
}

// Time out check for profs in the init stages.
// Abort or Resend depending on ->failCount.
void ticlyTimeOut(struct jalbProfile *pro) {
    // Stage is NOT "NotConn" or "Game".

    // If the prof timesout during init.
    if(SDL_GetTicks() - pro->lastRecv > returnWaitTime) {
        pro->lastRecv = SDL_GetTicks();
        pro->failCount += 1;
        if(pro->failCount > returnFailMax) {
            // There have been enough fails, abort the connect.
            textBoxAddInt(tb, "Connection Aborted. Stage: (", pro->stage);
            pro->stage = 0;
        } else {
            // Resend the data.
            if(pro->stage == psKeyWait) {
                sendKey(pro);
            } else if(pro->stage == psTerWait) {
                pro->curCounter = pro->oldCounter;
                initSendTers(pro);
            } else if(pro->stage == psEntWait) {
                pro->curCounter = pro->oldCounter;
                sendEnts(pro);
            } else if(pro->stage == psCapWait) {
//                printf("sendCapstone from delay resend.\n");
                sendPoUpdate(pro);
                textBoxAddStr(tb, "sendCap prof");
            }
        }
    }
}

// Stage == psGame
void ticlyOutGame(struct jalbProfile *pro) {
//    printf("ticlyOutGame()\n");
    // Iterate through changedEnts.
    // Iterate ents, see the number of
    /// Changed Ents.
    // Write the header later, if it even neds to be send.
    bufIndex = 0;
    sendEntsTicly(pro);
    /// New Chunks
    sendNewChunks(pro);
    sendMovedVerts(pro);
    /// Updated Chunks.
    if(bufIndex > 1) {
        int nsent = sendto(mysocket, buf, bufIndex, 0, (struct sockaddr*) &pro->profAddr, pro->addrLen);
//        lastSent
//        textBoxAddInt(tb, "sendEnts nsent (", nsent);
        if(nsent < bufIndex) {
            printf("send Partial\n");
        }
    }

//    printf("(now: %d) (recv: %d) (sent: %d)\n", SDL_GetTicks(), pro->lastRecv, pro->lastSent);
    if(SDL_GetTicks() - pro->lastRecv > returnWaitTime || SDL_GetTicks() - pro->lastSent > returnWaitTime) {
        pro->failCount++;
        if(pro->failCount > returnFailMax) {
            // There have been enough fails, abort the connect.
            textBoxAddStr(tb, "Connection Aborted [poll]");

            profDisconnect(pro);
        } else {
            // Resend the data.
            sendPoUpdate(pro);
            textBoxAddStr(tb, "sendPoll prof");
            pro->lastSent = SDL_GetTicks();
        }
    }

    bufIndex = 0;
}

/** Data Out */

void sendEntsTicly(struct jalbProfile *pro) {
    int preIndex = bufIndex;
    bufIndex += 1; // Id
    bufIndex += sizeof(int); // numEnts
//    int entSize = sizeof(float) * 6 + sizeof(int);
    int writtenEnts = 0;

    struct jalbEntity *proEnt = arrayListGetPointer(entList, pro->entity);

    int i;
    int numEnts = arrayListGetLength(entList);
    for(i = 0; i < numEnts; i++) {
//        printf("entsTicly PreIndex: %d\n", preIndex);
        struct jalbEntity *ent = arrayListGetPointer(entList, i);
        // This ent is in range.
        int relX = ent->gravMem->chunkCoords[0] - proEnt->gravMem->chunkCoords[0];
        int relY = ent->gravMem->chunkCoords[1] - proEnt->gravMem->chunkCoords[1];
        if(terRadius(relX, relY) < pro->terRadius) {
//            printf("pre: %d\n", bufIndex);
            int preSub = bufIndex;
            bufIndex += sizeof(int); // Ent Id
            bufIndex += sizeof(int); // NumSubs
            int numSubs = 0;
            // Loc changed
            if(ent->gravMem->locChanged) {
//                printf("locChanged\n");
//                if(bufIndex + entSize > BUFSIZE) {
//                    /// Fix It: send the buf and then resume here.
//                    // Maybe make this all its own function with a fail/success returned.
//                    break;
//                }

                buf[bufIndex] = soEntLoc;
                bufIndex += 1;

                intToBuffer(ent->gravMem->chunkCoords[0], &buf[bufIndex]);
                bufIndex += sizeof(int);
                intToBuffer(ent->gravMem->chunkCoords[1], &buf[bufIndex]);
                bufIndex += sizeof(int);

                memcpy(&buf[bufIndex], ent->gravMem->position, sizeof(float) * 3);
                bufIndex += sizeof(float) * 3;

                numSubs += 1;
            }
            // Rot changed.
            if(ent->gravMem->rotChanged) {
//                printf("rotChanged\n");
                buf[bufIndex] = soEntRot;
                bufIndex += 1;
                memcpy(&buf[bufIndex], ent->gravMem->rots, sizeof(float) * 3);
                bufIndex += sizeof(float) * 3;
                numSubs += 1;
            }
            if(numSubs == 0) {
                bufIndex = preSub;
            } else {
                intToBuffer(pro->entity, &buf[preSub]);
                preSub += sizeof(int);
                intToBuffer(numSubs, &buf[preSub]);
//                printf("thisOne\n");
                writtenEnts += 1;
            }
//            printf("post: %d\n", bufIndex);
        }
    }
    if(writtenEnts == 0) {
        // If no ents were buffered then reset bufIndex back to 0.
        bufIndex = preIndex;
    } else {
        printf("writeEnts (%d -> %d)\n", preIndex, bufIndex);
        buf[preIndex] = soEntUpdate;
        preIndex += 1;
        intToBuffer(writtenEnts, &buf[preIndex]);
    }
}

// Plug in delta coords, Send the data to the buffer to be sent. This should be a single send.
void sendNewChunks(struct jalbProfile *pro) {
    struct jalbEntity *proEnt = arrayListGetPointer(entList, pro->entity);
    if(proEnt->gravMem->chunkCoords[0] == proEnt->gravMem->chunkCoordsOld[0]
        && proEnt->gravMem->chunkCoords[1] == proEnt->gravMem->chunkCoordsOld[1]) {
        // If the character is in the same chunk.
        return;
    }
//    printf("\nsendNewChunks()\n\n");
    int preIndex = bufIndex;
    bufIndex += 1; // Id
    bufIndex += sizeof(int); // Num Chunks Sent

    int numChunks = arrayListGetLength(terList);
    printf("numChunks: %d\n", numChunks);
    int numSent = 0;
    int i;
    for(i = 0; i < numChunks; i++) {
        struct jalbTerrain *ter = arrayListGetPointer(terList, i);
        int newR = terRadius(ter->chunkCoords[0] - proEnt->gravMem->chunkCoords[0], ter->chunkCoords[1] - proEnt->gravMem->chunkCoords[1]);
        int oldR = terRadius(ter->chunkCoords[0] - proEnt->gravMem->chunkCoordsOld[0], ter->chunkCoords[1] - proEnt->gravMem->chunkCoordsOld[1]);
        // Inside of my jalbGravity, i have old and newCoords, so i dont even need to plug indelta.
        if(newR == pro->terRadius-1 && oldR > pro->terRadius - 1) {
            // This is a good chunk.
            if(!buffChunk(ter)) {
                // Not enough space, send and continue.
                /// Fix It: If numSent == 0;
                // I can organize this better. Instead of using i = 0. use ->curCounter. And then send + recure right here.
                buf[preIndex] = soInitTer;
                intToBuffer(numSent, &buf[preIndex + 1]);
                printf("writeNewChunks Mid (%d -> %d)\n", preIndex, bufIndex);
                int nsproEnt = sendto(mysocket, buf, bufIndex, 0, (struct sockaddr*) &pro->profAddr, pro->addrLen);
                // textBoxAddInt(tb, "sendproEnts nsproEnt (", nsproEnt);
                if(nsproEnt < bufIndex) {
                    printf("send Partial\n");
                }
                bufIndex = 0;
                preIndex = bufIndex;
                bufIndex += 1; // Id
                bufIndex += sizeof(int); // Num Chunks Sent
                numSent = 0;
                if(!buffChunk(ter)) {
                    /// Fix It:
                    printf("BUG: chunk can not fit in buffer.");
                }
            }
            numSent += 1;
        }
    }

    if(numSent == 0) {
        bufIndex = preIndex;
    } else {
        printf("writeNewChunks (%d -> %d)\n", preIndex, bufIndex);
        buf[preIndex] = soInitTer;
        intToBuffer(numSent, &buf[preIndex + 1]);
    }
    proEnt->gravMem->chunkCoordsOld[0] = proEnt->gravMem->chunkCoords[0];
    proEnt->gravMem->chunkCoordsOld[1] = proEnt->gravMem->chunkCoords[1];
}

// Return buffIndex.
void sendMovedVerts(struct jalbProfile *pro) {
    int preIndex = bufIndex;
    bufIndex += 1; // Id
    bufIndex += sizeof(int); // numChunks
//    int entSize = sizeof(float) * 6 + sizeof(int);
    int writtenChunks = 0;

    struct jalbEntity *proEnt = arrayListGetPointer(entList, pro->entity);

    int i;
    int numChunks = arrayListGetLength(terList);
    for(i = 0; i < numChunks; i++) {
        struct jalbTerrain *ter = arrayListGetPointer(terList, i);
        // This ter is in range.
        int relX = ter->chunkCoords[0] - proEnt->gravMem->chunkCoords[0];
        int relY = ter->chunkCoords[1] - proEnt->gravMem->chunkCoords[1];
        if(terRadius(relX, relY) < pro->terRadius) {
            int numMovedVerts = arrayListGetLength(ter->movedVerts);
            // If changes are going to be sent.
            if(numMovedVerts > 0) {
//                printf("numMovedVerts: %d (bufIndex: %d)\n", numMovedVerts, bufIndex);
                /// Chunk Coords, Id(moved verts) numMovedVerts.
                // Do i want, cC, then movedVertsId, then numMovedVerts.
                intToBuffer(ter->chunkCoords[0], &buf[bufIndex]);
                bufIndex += sizeof(int);
                intToBuffer(ter->chunkCoords[1], &buf[bufIndex]);
                bufIndex += sizeof(int);

                buf[bufIndex] = soTerVertMove; // 0 means vert move. this is no ok, it needs to be
                bufIndex += 1;

                int preVertIndex = bufIndex;

//                printf("numMovedVerts: %d (bufIndex: %d)\n", numMovedVerts, bufIndex);
//                intToBuffer(numMovedVerts, &buf[bufIndex]);
                bufIndex += sizeof(int);

                int numVertsSent = 0;

                // Send verts from list.
                int j;
                for(j = 0; j < numMovedVerts; j++) {
                    int vertNum = arrayListGetInt(ter->movedVerts, j);

                    if(vertNum >= ter->numVerts) {
                        printf("Bug: sendMovedVerts() Out of Bound vert.\n");
                    } else {
                        intToBuffer(vertNum, &buf[bufIndex]);
                        bufIndex += sizeof(int);

                        memcpy(&buf[bufIndex], &ter->verts[vertNum], sizeof(struct vertTex));
                        bufIndex += sizeof(struct vertTex);
                        numVertsSent += 1;
                    }
                }

                intToBuffer(numVertsSent, &buf[preVertIndex]);

                writtenChunks += 1;
            }
        }
    }
    if(writtenChunks == 0) {
        bufIndex = preIndex;
    } else {
        buf[preIndex] = soTerUpdate;
        preIndex += 1;
        printf("writtenChunks: %d (preIndex: %d)\n", writtenChunks, preIndex);
        intToBuffer(writtenChunks, &buf[preIndex]);
    }
}

// Stage == 1
// Starts from buffIndex = 0;
void initSendTers(struct jalbProfile *pro) {
    printf("initSendTers()\n");
    bufIndex = 1; // Id
    bufIndex += sizeof(int); // Num Chunks Sent
//    textBoxAddStr(tb, "sendTers");
    struct jalbEntity *ent = arrayListGetPointer(entList, pro->entity);
    int numChunks = arrayListGetLength(terList);
    printf("numChunks: %d\n", numChunks);
    int numSent = 0;
    while(pro->curCounter < numChunks) {
        struct jalbTerrain *ter = arrayListGetPointer(terList, pro->curCounter);
        int dx = ter->chunkCoords[0] - ent->gravMem->chunkCoords[0];
        int dy = ter->chunkCoords[1] - ent->gravMem->chunkCoords[1];
        if(terRadius(dx, dy) < pro->terRadius) {
            if(!buffChunk(ter)) {
                // This chunk cant fit.
                break;
            }
            numSent += 1;
        }
        pro->curCounter++;
    }
    buf[0] = soInitTer;
    printf("numSent: %d\n", numSent);
    intToBuffer(numSent, &buf[1]);
    // Either the buffer is full, or the all the chunks are buffered.
    // So send this buff.
    int nsent = sendto(mysocket, buf, bufIndex, 0, (struct sockaddr*) &pro->profAddr, pro->addrLen);
    textBoxAddInt(tb, "sendChunk nsent (", nsent);
    if(nsent < bufIndex) {
        printf("send Partial\n");
    }
    bufIndex = 0;
    printf("curCounter: %d\n", pro->curCounter);

}

int buffChunk(struct jalbTerrain *ter) {
//    printf("buffChunk()\n");
    int chunkSize = 0;
    chunkSize += sizeof(int) * 4; // Lengths
    chunkSize += sizeof(int) * ter->numFaces * 3; // Faces
    chunkSize += sizeof(struct vertTex) * ter->numVerts; // Verts
    if(BUFSIZE >= bufIndex + chunkSize) {
        // There is room for this chunk.
        // Id, x, y, numVerts, numFaces, faces, verts, texis
//        printf("start: %d\n", bufIndex);
//        buf[bufIndex] = soInitTer;
//        bufIndex += 1;
        intToBuffer(ter->chunkCoords[0], &buf[bufIndex]);
        bufIndex += sizeof(int);
        intToBuffer(ter->chunkCoords[1], &buf[bufIndex]);
        bufIndex += sizeof(int);
        intToBuffer(ter->numVerts, &buf[bufIndex]);
        bufIndex += sizeof(int);
        intToBuffer(ter->numFaces, &buf[bufIndex]);
        bufIndex += sizeof(int);
//        printf("pre: %d\n", bufIndex);
//        printf("sizeof(struct vertTex), ter->numVerts (%lu, %d)\n", sizeof(struct vertTex), ter->numVerts);
        memcpy(&buf[bufIndex], ter->verts, sizeof(struct vertTex) * ter->numVerts);
        bufIndex += sizeof(struct vertTex) * ter->numVerts;
//        printf("verts: %d\n", bufIndex);
        memcpy(&buf[bufIndex], ter->faces, sizeof(int) * ter->numFaces * 3);
        bufIndex += sizeof(int) * ter->numFaces * 3;
//        printf("faces: %d\n", bufIndex);
//        memcpy(&buf[bufIndex], ter->texis, sizeof(int) * ter->numVerts);
//        bufIndex += sizeof(int) * ter->numVerts;

        return 1;
    }
    return 0;
}

int buffChunkMin(struct jalbTerrain *ter) {
//    printf("buffChunk()\n");
    int chunkSize = 1; // Id
    chunkSize += sizeof(int) * 4; // Lengths
    chunkSize += sizeof(int) * ter->numFaces * 3; // Faces
    chunkSize += sizeof(float) * ter->numVerts * 3; // Verts
    chunkSize += sizeof(int) * ter->numVerts; // Teris
    if(BUFSIZE >= bufIndex + chunkSize) {
        // There is room for this chunk.
        // Id, x, y, numVerts, numFaces, faces, verts, texis
        buf[bufIndex] = soInitTer;
        bufIndex += 1;
        intToBuffer(ter->chunkCoords[0], &buf[bufIndex]);
        bufIndex += sizeof(int);
        intToBuffer(ter->chunkCoords[1], &buf[bufIndex]);
        bufIndex += sizeof(int);
        intToBuffer(ter->numVerts, &buf[bufIndex]);
        bufIndex += sizeof(int);
        intToBuffer(ter->numFaces, &buf[bufIndex]);
        bufIndex += sizeof(int);
        memcpy(&buf[bufIndex], ter->verts, sizeof(float) * ter->numVerts * 3);
        bufIndex += sizeof(float) * ter->numVerts * 3;
        memcpy(&buf[bufIndex], ter->faces, sizeof(int) * ter->numFaces * 3);
        bufIndex += sizeof(int) * ter->numFaces * 3;
//        memcpy(&buf[bufIndex], ter->texis, sizeof(int) * ter->numVerts);
//        bufIndex += sizeof(int) * ter->numVerts;

        return 1;
    }
    return 0;
}

void sendEnts(struct jalbProfile *pro) {
    buf[0] = soInitEnt;
    bufIndex++;
    struct jalbEntity *myEnt = arrayListGetPointer(entList, pro->entity);
    int numEnts = arrayListGetLength(entList);
    while(pro->curCounter < numEnts) {
        struct jalbEntity *ent = arrayListGetPointer(entList, pro->curCounter);
        int dx = ent->gravMem->chunkCoords[0] - myEnt->gravMem->chunkCoords[0];
        int dy = ent->gravMem->chunkCoords[1] - myEnt->gravMem->chunkCoords[1];
        if(terRadius(dx, dy) < pro->terRadius) {
            if(!buffEnt(ent)) {
                // This chunk cant fit.
                break;
            }
        }

        pro->curCounter++;
    }
    // Either the buffer is full, or the all the relevant ents are buffered.
    // So send this buff.
    int nsent = sendto(mysocket, buf, bufIndex, 0, (struct sockaddr*) &pro->profAddr, pro->addrLen);
    textBoxAddInt(tb, "sendEnts nsent (", nsent);
    if(nsent < bufIndex) {
        printf("send Partial\n");
    }
    bufIndex = 0;
}

int buffEnt(struct jalbEntity *ent) {
    int entSize = 4 * sizeof(int) + 4 * sizeof(float);
    if(BUFSIZE >= bufIndex + entSize) {
        // There is room to fit the ent
        // Id, model, scale, pos, chunkCoords
        intToBuffer(ent->globalId, &buf[bufIndex]);
        bufIndex += sizeof(int);
        intToBuffer(ent->modelIndex, &buf[bufIndex]);
        bufIndex += sizeof(int);
        memcpy(&buf[bufIndex], &ent->scale, sizeof(float));
        bufIndex += sizeof(float);
        memcpy(&buf[bufIndex], &ent->gravMem->position, 3 * sizeof(float));
        bufIndex += 3 * sizeof(float);
        memcpy(&buf[bufIndex], &ent->gravMem->rots, 3 * sizeof(float));
        bufIndex += 3 * sizeof(float);
        intToBuffer(ent->gravMem->chunkCoords[0], &buf[bufIndex]);
        bufIndex += sizeof(int);
        intToBuffer(ent->gravMem->chunkCoords[1], &buf[bufIndex]);
        bufIndex += sizeof(int);
        return 1;
    }
    return 0;
}

// Doubles as a capstone, and a poll.
void sendPoUpdate(struct jalbProfile *pro) {
    buf[0] = soUpdate;
    bufIndex += 1;
    int nsent = sendto(mysocket, buf, bufIndex, 0, (struct sockaddr*) &pro->profAddr, pro->addrLen);
    if(nsent < bufIndex) {
        printf("send Partial\n");
    }
    bufIndex = 0;
}

// Sends key, profiles entityId, and terrain scale.
void sendKey(struct jalbProfile *pro) {
    buf[0] = soLogGood;
    memcpy(&buf[1], pro->key, KEYSIZE);
    bufIndex = 1 + KEYSIZE;
    intToBuffer(pro->entity, &buf[bufIndex]);
    bufIndex += sizeof(int);
    memcpy(&buf[bufIndex], &terScale, sizeof(float));
    bufIndex += sizeof(float);
    int nsent = sendto(mysocket, buf, bufIndex, 0, (struct sockaddr*) &pro->profAddr, pro->addrLen);
    if(nsent < bufIndex) {
        printf("sendFailed\n");
    }
    textBoxAddInt(tb, "keySent (", nsent);

    bufIndex = 0;
}

/** Other */

void profDisconnect(struct jalbProfile *prof) {
    prof->stage = 0;
    textBoxAddStr(tb, "Disconnected");

    struct jalbEntity *ent = arrayListGetPointer(entList, prof->entity);
    ent->gravMem->tempVect[0] = 0;
    ent->gravMem->tempVect[1] = 0;
    ent->gravMem->tempVect[2] = 0;

    memset(prof->keyMapBuf, 0, numServKeyMaps * sizeof(int));
}

// Need a better name for this.
int terRadius(int x, int y) {
//    printf("terRadius(%d, %d)\n", x, y);
    if(x > 0) {
        x = -x;
        y = -y;
    }
    if(y < 0) {
        return -x - y;
    } else if (y > -x) {
        return y;
    } else {
        return -x;
    }
}

// One off
void getTerRel(int terR, int i, int *x, int *y) {
    *y = 0;
    *x = -terR + 1;
    while(i >= terR) {
        i -= terR;
        if(*x < 0) {
            *y -= 1;
            terR++;
        } else {
            terR--;
        }
        *x += 1;
    }
    *y += i;
}

// Embed
void embedTerRel(int radius) {
    int i = 0;
    int ex = 0;
    int x = radius - 1;
    int y;
    for(; x > -radius; x--) {
        for (y = -x -ex; y <= ex; y++) {
            printf("action (%d, %d)\n", x, y);
            /// Action, have (x, y)
            i++;
        }
        if(x > 0) {
            ex++;
        }
    }
}

struct jalbTerrain *getTerFromCoords(ArrayList *terList, int x, int y) {
    int numTers = arrayListGetLength(terList);
    int i;
    for(i = 0 ; i < numTers; i++) {
        struct jalbTerrain *ter = arrayListGetPointer(terList, i);
        if(ter->chunkCoords[0] == x && ter->chunkCoords[1] == y) {
            return ter;
        }
    }
    return NULL;
}


void profCreation(unsigned char *buff, int length, struct sockaddr_in *remaddr, socklen_t addrlen) {
    int userLen = 0;
    while(buff[1 + userLen] != '\0') {
        userLen++;
    }

    int usernameIndex = isUsernameInUse(&buff[1]);
    if(usernameIndex != -1) {
        char a[256];
        strcpy(a, "Profile Creation: Username conflict.");
        int strlen = 0;
        while(a[strlen] != '\0') {
            strlen++;
        }
        txtBoxAddString(tb, a, strlen, font, &color, gRenderer);

        // Send creation failure.
        buf[0] = soNewBad;
        int nsent = sendto(mysocket, buf, 1, 0, (struct sockaddr*) remaddr, addrlen);
        if(nsent < 1) {
            printf("sendFailed\n");
        }
        return;
    }

    // Make a new profile and add it to the list.
    // Then send the correct signal back to the Client.
    struct jalbProfile *pro = malloc(sizeof(struct jalbProfile));
    pro->addrLen = addrlen;
    pro->profAddr = *remaddr;
    pro->stage = 0;
    strncpy((char*) pro->username, (char*) &buff[1], 256);
    strncpy((char*) pro->password, (char*) &buff[257], 256);

    arrayListAddEndPointer(profList, pro);

    char *a = malloc(256 * sizeof(char));
    strcpy(a, "Profile Creation: Success");
    int strlen = 0;
    while(a[strlen] != '\0') {
        strlen++;
    }

    txtBoxAddString(tb, a, strlen, font, &color, gRenderer);
    free(a);

    // Send creation success.
    buf[0] = soNewGood;
    int nsent = sendto(mysocket, buf, 1, 0, (struct sockaddr*)remaddr, addrlen);
    if(nsent < 1) {
        printf("sendFailed\n");
    }
}

void profLogin(unsigned char *buff, int length, struct sockaddr_in *remaddr, socklen_t addrlen) {
    printf("profLogin()\n");
    int userIndex = isUsernameInUse(buff);

    printf("userIndex: %d\n", userIndex);

    if(userIndex == -1) {
        textBoxAddStr(tb, "Log In: Failed (username)");

        buf[0] = soLogBad;
        int nsent = sendto(mysocket, buf, 1, 0, (struct sockaddr*) remaddr, addrlen);
        if(nsent < 1) {
            printf("sendFailed\n");
        }
        return;
    }

    // Compare passwords
    struct jalbProfile *pro = arrayListGetPointer(profList, userIndex);
    if(pro->stage != 0) {
        textBoxAddStr(tb, "Log In: Failed (overload)");

        buf[0] = soLogBad;
        int nsent = sendto(mysocket, buf, 1, 0, (struct sockaddr*) remaddr, addrlen);
        if(nsent < 1) {
            printf("sendFailed\n");
        }
        return;
    }
    printf("areSameCheck()\n");
    int areSame = strCompare(&buff[256], pro->password, 256);

    printf("areSame: %d\n", areSame);

    if(!areSame) {
        char a[256];
        strncpy(a, "Log In: Failed (password)", 256);
        txtBoxAddString(tb, a, 14, font, &color, gRenderer);

        buf[0] = soLogBad;
        int nsent = sendto(mysocket, buf, 1, 0, (struct sockaddr*) remaddr, addrlen);
        if(nsent < 1) {
            printf("sendFailed\n");
        }
        return;
    }
    // Log in is good.
    // Make a key and return it.
    // Set the return address information.
    // Set last received time.
    pro->terRadius = buffToInt(&buff[512]);
    pro->addrLen = addrlen;
    pro->profAddr = *remaddr;

    makeKey(pro->key);

    pro->stage = psKeyWait;
    pro->curCounter = 0;
    pro->failCount = 0;
    pro->lastRecv = SDL_GetTicks();

    memset(pro->keyMapBuf, 0, sizeof(int) * numServKeyMaps);

    sendKey(pro);
}

// Returns the index of the profile that has this name.
// Returns -1 if not in use.
int isUsernameInUse(unsigned char  *name) {
    printf("name: %s\n", name);
    int userLen = 0;
    while(name[userLen] != '\0') {
        userLen++;
    }
    printf("userLen: %d\n", userLen);
    if(userLen < 4) {
        return -1;
    }

    int numProfs = arrayListGetLength(profList);
    printf("numProfs: %d\n", numProfs);
    int i;
    for(i = 0; i < numProfs; i++) {
//        printf("numProfs: %d\n", numProfs);
        struct jalbProfile *pro = arrayListGetPointer(profList, i);
        printf("userNameInUse()\n");
        int areSame = strCompare(pro->username, name, 256);
        if(areSame) {
            return i;
        } else {
            continue;
        }
    }
    return -1;
}

// Makes a random key and then check it across all connected profiles.
// If the key is in use, recure.
void makeKey(unsigned char *buff) {
    int i;
    for(i = 0; i < KEYSIZE; i++) {
        buff[i] = (char) rand() % UCHAR_MAX;
    }

    int numProf = arrayListGetLength(profList);
    for(i = 0; i < numProf; i++) {
        struct jalbProfile *pro = arrayListGetPointer(profList, i);
        printf("pro->stage: %d\n", pro->stage);
        if(pro->stage > 0) {
            // This profile is connected, check keys.
            printf("makeKey\n");
            int areSame = strCompare(buff, pro->key, KEYSIZE);
            if(areSame) {
                printf("keys AreSame\n");
                makeKey(buff);
                return;
            }
        }
    }
}

// Sees if all the characters in the strings are the same, up to and including a null terminator.
int strCompare(unsigned char *str0, unsigned char *str1, int maxlen) {
    printf("strCompare(%s, %s)\n", str0, str1);
    int i;
    for(i = 0; i < maxlen; i++) {
        if(str0[i] != str1[i]) {
            return 0;
        }
        if(str0[i] == '\0') {
            return 1;
        }
    }
    // There is no difference, and a null terminator is not found.
    return 1;
}

struct jalbProfile *keyToProf(unsigned char *buf) {
    int i;
    int numProfs = arrayListGetLength(profList);
    for(i = 0; i < numProfs; i++) {
        struct jalbProfile *pro = arrayListGetPointer(profList, i);
        // If stage > 0 and memcpy == 0
        if(pro->stage) {
            if(!memcmp(buf, pro->key, KEYSIZE)) {
                return pro;
            }
        }
    }
    return NULL;
}

/** Save + Loading */
SDL_Texture* loadTexture( char *path, int *retWidth, int *retHeight) {
    //The final texture
    SDL_Texture* newTexture = NULL;
    //Load image at specified path
    SDL_Surface* loadedSurface = IMG_Load( path );
    if( loadedSurface == NULL ) {
        printf( "Unable to load image %s! SDL_image Error: %s\n", path, IMG_GetError() );
    } else {
        *retWidth = loadedSurface->w;
        *retHeight = loadedSurface->h;
        //Create texture from surface pixels
        newTexture = SDL_CreateTextureFromSurface( gRenderer, loadedSurface );
        if( newTexture == NULL ) {
            printf( "Unable to create texture from %s! SDL Error: %s\n", path, SDL_GetError() );
        }
        //Get rid of old loaded surface
        SDL_FreeSurface( loadedSurface );
    }
    return newTexture;
}

// Profiles.
void saveProfs(char *dir, ArrayList *profList) {
    printf("saveProfs()\n");
    FILE *file;
    file = fopen(dir, "w");

    if(file == NULL ) {
        printf("S Prof: error opening file\n");
        return;
    }

    int total = arrayListGetLength(profList);

    printf("total: %d\n", total);

    fwrite(&total, sizeof(int), 1, file);

    int i;
    for(i = 0; i < total; i++) {
        struct jalbProfile *pro = arrayListGetPointer(profList, i);
        saveProf(pro, file);
    }
    fclose(file);
}

int loadProfs(char *dir, ArrayList *profList) {
    FILE *file;
    file = fopen(dir, "r");
    if(file == NULL ) {
        printf("L Profs: error opening file\n");
        return 0;
    }
    int total;

    fread(&total, sizeof(int), 1, file);
    int current = arrayListGetLength(profList);

    printf("current: %d\n", current);

    printf("total: %d\n", total);

    int i;
    for(i = 0; i < total; i++) {
        struct jalbProfile *pro;
        if(i >= current) {
            pro = malloc(sizeof(struct jalbProfile));
            arrayListAddEndPointer(profList, pro);
        } else {
            pro = arrayListGetPointer(profList, i);
        }
        loadProf(pro, file);
    }
    fclose(file);
    return 1;
}

void saveProf(struct jalbProfile *pro, FILE *file) {
    fwrite(pro->username, sizeof(char), 256, file);
    fwrite(pro->password, sizeof(char), 256, file);
    fwrite(&pro->entity, sizeof(int), 1, file);
}

void loadProf(struct jalbProfile *pro, FILE *file) {
//    printf("loadServ: %p\n", ser);
    fread(pro->username, sizeof(char), 256, file);
    fread(pro->password, sizeof(char), 256, file);
    fread(&pro->entity, sizeof(int), 1, file);
    pro->stage = 0;
    pro->curCounter = 0;
    pro->oldCounter = 0;
    pro->failCount = 0;
}

// Chunks.
void saveChunks(char *dir, ArrayList *chunkList) {
    printf("saveChunks()\n");
    FILE *file;
    file = fopen(dir, "w");

    if(file == NULL ) {
        printf("S Chunk: error opening file\n");
        return;
    }

    int total = arrayListGetLength(chunkList);

    printf("total: %d\n", total);

    fwrite(&total, sizeof(int), 1, file);

    int i;
    for(i = 0; i < total; i++) {
        struct jalbTerrain *ter = arrayListGetPointer(chunkList, i);
        saveChunk(ter, file);
    }
    fclose(file);
}

void saveChunk(struct jalbTerrain *chunk, FILE *file) {
    fwrite(&chunk->numVerts, sizeof(int), 1, file);
    fwrite(&chunk->numFaces, sizeof(int), 1, file);
    fwrite(chunk->chunkCoords, sizeof(int), 2, file);
    fwrite(chunk->verts, sizeof(struct vertTex), chunk->numVerts, file);
    fwrite(chunk->faces, sizeof(int), chunk->numFaces * 3, file);
}

/** txtBox add */
void textBoxAddInt(struct textBox *textB, char *str, int var) {
    char a[256];
    int strlen = 0;
    while(str[strlen] != '\0') {
        strlen++;
    }
    strncpy(a, str, 256);
    sprintf(&a[strlen], "%d", var);
    while(a[strlen] != '\0') {
        strlen++;
    }
    a[strlen] = ')';
    strlen++;
    a[strlen] = '\0';
    txtBoxAddString(textB, a, strlen, font, &color, gRenderer);
}

// Need to test this.
void textBoxAdd2Int(struct textBox *textB, char *str0, int var0, char *str1, int var1) {
    char a[256];
    int strlen = 0;
    while(str0[strlen] != '\0') {
        strlen++;
    }
    strncpy(a, str0, 256);
    sprintf(&a[strlen], "%d", var0);
    while(a[strlen] != '\0') {
        strlen++;
    }
    strncpy(&a[strlen], str1, 256 - strlen);
    while(a[strlen] != '\0') {
        strlen++;
    }
    sprintf(&str1[strlen], "%d", var1);
    while(a[strlen] != '\0') {
        strlen++;
    }
    a[strlen] = ')';
    strlen++;
    a[strlen] = '\0';
    txtBoxAddString(textB, a, strlen, font, &color, gRenderer);
}

void textBoxAddFloat(struct textBox *textB, char *str, float var) {
    char a[256];
    int strlen = 0;
    while(str[strlen] != '\0') {
        strlen++;
    }
    strncpy(a, str, 256);
    sprintf(&a[strlen], "%f", var);
    while(a[strlen] != '\0') {
        strlen++;
    }
    a[strlen] = ')';
    strlen++;
    a[strlen] = '\0';
    txtBoxAddString(textB, a, strlen, font, &color, gRenderer);
}

void textBoxAddStr(struct textBox *textB, char *str) {
    char a[256];
    int strlen = 0;
    while(str[strlen] != '\0') {
        strlen++;
    }
    strncpy(a, str, 256);
    a[strlen] = '\0';
//    printf("a: %s\n", a);
    txtBoxAddString(textB, a, strlen, font, &color, gRenderer);
}

/** Other (tests) */

int ticCount = 0;

// Testing
void autoWalk() {
//    struct jalbEntity *ent = arrayListGetPointer(entList, 0);
    struct jalbProfile *pro = arrayListGetPointer(profList, 0);

    int start = 1;
    int turn = 8;
    int walk = 1;
    if(ticCount < start) {
        pro->stage = psGame;
        memset(pro->keyMapBuf, 0, sizeof(int) * numServKeyMaps);
    } else if(ticCount < start + turn) {
        pro->keyMapBuf[skmRotateleft] = 1;
    } else if(ticCount < start + turn + walk) {
        pro->keyMapBuf[skmRotateleft] = 0;
        pro->keyMapBuf[skmWalkForward] = 1;
    } else {
        pro->stage = psNotConn;
        pro->keyMapBuf[skmWalkForward] = 0;
        running = 0;
    }

    ticCount++;
}

void addFloatTb(char *str, double f) {
//    printf("\naddFloatTb: %f\n\n", f);
    textBoxAddFloat(tb, str, f);
}

void edgeTest() {
    float v[2];
    v[0] = -20.1;
    v[1] = 0;
    int edge = isOnEdge(v[0], v[1], 20);
    printf("edge: %d\n", edge);

    v[0] = 12.382837;
    v[1] = 14.1;

    edge = isOnEdge(v[0], v[1], 20);
    printf("edge: %d\n", edge);

    v[0] = 10.561710;
    v[1] = 23.895241;

    edge = isOnEdge(v[0], v[1], 20);
    printf("edge: %d\n", edge);

    edge = isOnEdgeNew(v[0], v[1], 20);
    printf("edge: %d\n", edge);

//    If it is corner, its even.
//    If it is edge, its odd.
    // index = edge / 2 + 1;
    // plus 1, inorder to get away from [0] which is the center vect.

    // Ok, i get 5, as i expected.
    // Now the question is, does this translate properly using terrain coords array.
    // Something about switching between x, y is weirding me out.
    // I dont keep the array as simple as i  want it to be.
    // The way the grounded does everything fine, then once i hit the air everything gets switched.

//    exit(0);
}

void terMinTest() {
    struct jalbTerrain *min = initTerrain(2, 0, 0, 1);
    int i;
    for(i = 0; i < min->numVerts; i++) {
        printf("i: %d\n", i);
        sayVect("min->verts[i*3]", min->verts[i].loc, 3);
    }
    exit(0);
}

/** Terrain Editing */

void rayCollideTerArray(float *loc, float *vec, struct jalbGravity *gravMem, int *retFace, int *retChunk) {
// Iterate through the faces of this chunk.
    // Run circRayFaceScalar. Find the face with the lowest scalar.
    float lowScale = FLT_MAX;

    int c;
    for(c = 0; c < 7; c++) {
        struct jalbTerrain *ter = gravMem->terArray[c];
        int f;
        for (f = 0; f < ter->numFaces; f++) {
            if(f == gravMem->faceIndex) {
                // Not going to collide with the face it is starting on.
                continue;
            }

            // New Face Verts.
            float *v0 = ter->verts[ter->faces[f *3 + 0]].loc;
            float *v1 = ter->verts[ter->faces[f *3 + 1]].loc;
            float *v2 = ter->verts[ter->faces[f *3 + 2]].loc;

            int retType = -1;
            float retBarys[2];

            float wx = 0;
            float wy = 0;
            terToWorldNew(chunkArrayCoords[c * 2], chunkArrayCoords[c * 2 + 1], &wx, &wy, terScale);

            float relPos[3];
            relPos[0] = loc[0] - wx;
            relPos[1] = loc[1];
            relPos[2] = loc[2] - wy;

            float scale = newerCircRayFaceScalar(relPos, vec, detectRadius, v0, v1, v2, retBarys, &retType);

            // Could be positive, negative, nan. Sometimes [1] is -1.
            if(scale > 0 && scale < lowScale) {
                lowScale = scale;
                *retFace = f;
                *retChunk = c;
            }
        }
    }
}

// I should make a funct that returns the low face, chunk, type. Maybe sets it for the memObj, No dont set.
void terEdit(float *loc, float *vec, struct jalbGravity *memObj, int push) {
    // Iterate through the faces of this chunk.
    // Run circRayFaceScalar. Find the face with the lowest scalar.

    int lowFace = -1;
    int lowChunk = -1;

    rayCollideTerArray(loc, vec, memObj, &lowFace, &lowChunk);

    if(lowFace != -1) {
        printf("lowFace: %d\n", lowFace);
        // There is a collision.
        struct jalbTerrain *ter = memObj->terArray[lowChunk];

        int verts[3];
        verts[0] = ter->faces[lowFace * 3];
        verts[1] = ter->faces[lowFace * 3 + 1];
        verts[2] = ter->faces[lowFace * 3 + 2];
        printf("verts[]: (%d, %d, %d)\n", verts[0], verts[1], verts[2]);

        vectScaleTo(vec, .2, 3);

        if(push) {
            vectAdd(ter->verts[verts[0]].loc, vec, ter->verts[verts[0]].loc, 3);
            vectAdd(ter->verts[verts[1]].loc, vec, ter->verts[verts[1]].loc, 3);
            vectAdd(ter->verts[verts[2]].loc, vec, ter->verts[verts[2]].loc, 3);
        } else {
            vectSub(ter->verts[verts[0]].loc, vec, ter->verts[verts[0]].loc, 3);
            vectSub(ter->verts[verts[1]].loc, vec, ter->verts[verts[1]].loc, 3);
            vectSub(ter->verts[verts[2]].loc, vec, ter->verts[verts[2]].loc, 3);
        }

//        arrayListAddUnique(ter->movedVerts, &verts[0]);
//        arrayListAddUnique(ter->movedVerts, &verts[1]);
//        arrayListAddUnique(ter->movedVerts, &verts[2]);

        arrayListAddEnd(ter->movedVerts, &verts[0]);
        arrayListAddEnd(ter->movedVerts, &verts[1]);
        arrayListAddEnd(ter->movedVerts, &verts[2]);
        // Move the 3 verts attached to this face. Add all 3 to the list.

//        int numMovedVerts = arrayListGetLength(ter->movedVerts);
//        printf("numMovedVerts: %d\n", numMovedVerts);
    }
}

void terEditNew(float *loc, float *vec, struct jalbGravity *gravMem, int push, int editType) {
    int lowFace = -1;
    int lowChunk = -1;

    float lowScale = FLT_MAX;

    float lowBarys[2];

    int c;
    for(c = 0; c < 7; c++) {
        struct jalbTerrain *ter = gravMem->terArray[c];
        int f;
        for (f = 0; f < ter->numFaces; f++) {
            if(f == gravMem->faceIndex) {
                // Not going to collide with the face unit is standing on.
                continue;
            }

            // New Face Verts.
            float *v0 = ter->verts[ter->faces[f *3 + 0]].loc;
            float *v1 = ter->verts[ter->faces[f *3 + 1]].loc;
            float *v2 = ter->verts[ter->faces[f *3 + 2]].loc;

            float wx = 0;
            float wy = 0;
            terToWorldNew(chunkArrayCoords[c * 2], chunkArrayCoords[c * 2 + 1], &wx, &wy, terScale);

            float relPos[3];
            relPos[0] = loc[0] - wx;
            relPos[1] = loc[1];
            relPos[2] = loc[2] - wy;

            int retType = -1;
            float retBarys[2];

            float scale = newerCircRayFaceScalar(relPos, vec, detectRadius, v0, v1, v2, retBarys, &retType);

            // Could be positive, negative, nan. Sometimes [1] is -1.
            if(scale > 0 && scale < lowScale) {
                lowScale = scale;
                lowFace = f;
                lowChunk = c;
                lowBarys[0] = retBarys[0];
                lowBarys[1] = retBarys[1];
            }
        }
    }

    if(lowFace != -1) {
        printf("lowFace: %d (lowChunk: %d)\n", lowFace, lowChunk);

        // There is a collision.
        struct jalbTerrain *ter = gravMem->terArray[lowChunk];

        int verts[3];
        verts[0] = ter->faces[lowFace * 3];
        verts[1] = ter->faces[lowFace * 3 + 1];
        verts[2] = ter->faces[lowFace * 3 + 2];

//        sayVect("barys", lowBarys, 2);

        float v[3];
        vectSub(ter->verts[verts[1]].loc, ter->verts[verts[0]].loc, v, 3);
//        sayVect("v", v, 3);
        float u[3];
        vectSub(ter->verts[verts[2]].loc, ter->verts[verts[0]].loc, u, 3);
//        sayVect("u", u, 3);

//        sayVect("ter->verts[verts[0]].loc", ter->verts[verts[0]].loc, 3);

        float point[3];
        point[0] = ter->verts[verts[0]].loc[0] + v[0] * lowBarys[0] + u[0] * lowBarys[1];
        point[1] = ter->verts[verts[0]].loc[1] + v[1] * lowBarys[0] + u[1] * lowBarys[1];
        point[2] = ter->verts[verts[0]].loc[2] + v[2] * lowBarys[0] + u[2] * lowBarys[1];

        sayVect("point", point, 3);

        float norm[3];
        crossProd3(v, u, norm);
        vectScaleTo(norm, .6, 3);
        vec[0] = norm[0];
        vec[1] = norm[1];
        vec[2] = norm[2];

        if(!push) {
            vec[0] *= -1;
            vec[1] *= -1;
            vec[2] *= -1;
        }

        // Iterate every vert of this + surrounding chunks.

        // Iterate every chunk, find the chunks around this new chunk.
        // Once i have them, iterate verts to find if they are in range.
        // If they are in range, and on edge.
//        if(lowChunk == 0) {
//            brush0(point, vec, gravMem->terArray);
//        }
        if(editType == 0) {
//            brush1(point, vec, ter->chunkCoords);
//            brushNormal(point, ter->chunkCoords, 8, push);
            brushNormalInOut(point, lowFace, ter->chunkCoords, 8);
        } else if (editType <= 4) {
            brushTexi(point, ter->chunkCoords, editType-1);
        } else if (editType == 5) {
            brushSmooth(point, lowFace, ter->chunkCoords, 8);
        }
    }
}

void brush0(float *loc, float *editVect, void *terArray) {
    int i;
    for(i = 0; i < 7; i++) {
        struct jalbTerrain *ter = ((struct jalbTerrain**)terArray)[i];

        float wx = 0;
        float wy = 0;
        terToWorldNew(chunkArrayCoords[i * 2], chunkArrayCoords[i * 2 + 1], &wx, &wy, terScale);

        int j;
        for(j = 0; j < ter->numVerts; j++) {
            float delta[3];
            delta[0] = loc[0] - (ter->verts[j].loc[0] + wx);
            delta[1] = loc[1] - ter->verts[j].loc[1];
            delta[2] = loc[2] - (ter->verts[j].loc[2] + wy);
            if(i == 0) {
                sayVect("delta", delta, 3);
            }
//            vectSub(loc, ter->verts[j].loc, delta, 3);
            float mag = vectNorm(delta, 3);
            if(mag < 8) {
//                printf("mag: %f (chunk: %d)\n", mag, i);
                float scalar = sigmoid2(mag);
                float scaledVec[3];
                scaledVec[0] = editVect[0] * scalar;
                scaledVec[1] = editVect[1] * scalar;
                scaledVec[2] = editVect[2] * scalar;

                // untested.
                ter->verts[j].loc[0] += scaledVec[0];
                ter->verts[j].loc[1] += scaledVec[1];
                ter->verts[j].loc[2] += scaledVec[2];
//                vertEdit(ter, j, scaledVec);
            }
        }
    }
}

// Iterate every chunk around chunkCoords.
// Iterate every vert to see if it is in range. add it to array list.
void brush1(float *loc, float *editVect, int *chunkCoords) {
    int numChunks = arrayListGetLength(terList);
    int i;
    for(i = 0; i < numChunks; i++) {
        struct jalbTerrain *ter = arrayListGetPointer(terList, i);

        int radius = terRadius(chunkCoords[0] - ter->chunkCoords[0], chunkCoords[1] - ter->chunkCoords[1]);
        if(radius > 2) {
            continue;
        }

        float wx = 0;
        float wy = 0;
        terToWorldNew(chunkCoords[0] - ter->chunkCoords[0], chunkCoords[1] - ter->chunkCoords[1], &wx, &wy, terScale);

        int j;
        for(j = 0; j < ter->numVerts; j++) {
            float delta[3];
            delta[0] = loc[0] - ter->verts[j].loc[0] + wx;
            delta[1] = loc[1] - ter->verts[j].loc[1];
            delta[2] = loc[2] - ter->verts[j].loc[2] + wy;

            float mag = vectNorm(delta, 3);

            if(j == 164) {
//                sayVect("delta", delta, 3);
                printf("mag: %f\n", mag);
            }

            if(mag < 4) {
                printf("mag: %f (chunk: %d)\n", mag, i);
                float scalar = 2 * sigmoid2(mag);
                float scaledVec[3];
                scaledVec[0] = editVect[0] * scalar;
                scaledVec[1] = editVect[1] * scalar;
                scaledVec[2] = editVect[2] * scalar;
//                vertEdit(ter, j, scaledVec);")
                ter->verts[j].loc[0] += scaledVec[0];
                ter->verts[j].loc[1] += scaledVec[1];
                ter->verts[j].loc[2] += scaledVec[2];
                arrayListAddEnd(ter->movedVerts, &j);
            }
        }
    }
}

void brushTexi(float *loc, int *chunkCoords, int texi) {
    int numChunks = arrayListGetLength(terList);
    int i;
    for(i = 0; i < numChunks; i++) {
        struct jalbTerrain *ter = arrayListGetPointer(terList, i);

        int radius = terRadius(chunkCoords[0] - ter->chunkCoords[0], chunkCoords[1] - ter->chunkCoords[1]);
        if(radius > 2) {
            continue;
        }

        float wx = 0;
        float wy = 0;
        terToWorldNew(chunkCoords[0] - ter->chunkCoords[0], chunkCoords[1] - ter->chunkCoords[1], &wx, &wy, terScale);

        int j;
        for(j = 0; j < ter->numVerts; j++) {
            float delta[3];
            delta[0] = loc[0] - ter->verts[j].loc[0] + wx;
            delta[1] = loc[1] - ter->verts[j].loc[1];
            delta[2] = loc[2] - ter->verts[j].loc[2] + wy;

            float mag = vectNorm(delta, 3);

            if(j == 164) {
//                sayVect("delta", delta, 3);
                printf("mag: %f\n", mag);
            }

            if(mag < 4) {
//                printf("mag: %f (chunk: %d)\n", mag, i);
//                float scalar = 2 * sigmoid2(mag);
//                float scaledVec[3];
//                scaledVec[0] = editVect[0] * scalar;
//                scaledVec[1] = editVect[1] * scalar;
//                scaledVec[2] = editVect[2] * scalar;
////                vertEdit(ter, j, scaledVec);")
//                ter->verts[j].loc[0] += scaledVec[0];
//                ter->verts[j].loc[1] += scaledVec[1];
//                ter->verts[j].loc[2] += scaledVec[2];
                ter->verts[j].texi = texi;
                arrayListAddEnd(ter->movedVerts, &j);
            }
        }
    }
}

// Brush that moves the vert by the average of the normals of the face it touches.
void brushNormalInOut(float *loc, int faceIndex, int *chunkCoords, float radius) {
    printf("\nbrushSmooth (face: %d) (chunkCoords %d, %d)\n", faceIndex, chunkCoords[0], chunkCoords[1]);
    // I aught to save vertNum, and then pointer.
    ArrayList *todo = initArrayList(10, 3 * sizeof(int));
    ArrayList *doneEdges = initArrayList(10, 3 * sizeof(int)); // This is used to stop a 2nd recure of verts that are on chunk edge, and were already dealt with by that chunk.
    int doneVert = 0;

    int temp[3]; // Used as a buffer for filling the arrayListUnique.
    struct jalbTerrain *thisTer = coordsToTer(chunkCoords, terList);

    // Add the 3 verts of the collision face to the todo list.
    temp[0] = thisTer->faces[faceIndex * 3];
    temp[1] = thisTer->chunkCoords[0];
    temp[2] = thisTer->chunkCoords[1];
    arrayListAddUnique(todo, temp);

    temp[0] = thisTer->faces[faceIndex * 3 + 1];
    arrayListAddUnique(todo, temp);

    temp[0] = thisTer->faces[faceIndex * 3 + 2];
    arrayListAddUnique(todo, temp);

//    int numToDo = arrayListGetLength(todo); // Only 3.
    // Keep track of this number manually.

    for(; doneVert < arrayListGetLength(todo); doneVert++) {
        // VertId, ChunkX, ChunkY.
        int *todoVert = arrayListDataPointer(todo, doneVert);
//        printf("\tdoneVert: %d (todoVert %d, %d, %d)\n", doneVert, todoVert[0], todoVert[1], todoVert[2]);

        struct jalbTerrain *todoTer = coordsToTer(&todoVert[1], terList);

//        sayVect("todoTer->verts[todoVert[0]].loc", todoTer->verts[todoVert[0]].loc, 3);

        int edge = isOnEdge(todoTer->verts[todoVert[0]].loc[0], todoTer->verts[todoVert[0]].loc[2], terScale);
//        printf("Edge: %d\n", edge);

        // Fill the end of the buffer so i can add all relevant verts to todo / doneEdge.
        temp[1] = todoVert[1];
        temp[2] = todoVert[2];

        // Iterate all faces, if the face contains this vertIndex, then save the other 2 to the adj list.

        float normSum[3] = {0};
        int normNums = 0;

        // This is what i want to turn into a function.
        normNums += brushNormChunk(todoTer, todo, doneEdges, chunkCoords, todoVert, loc, temp, radius, normSum);


        struct jalbTerrain *adjTer0 = NULL;
        struct jalbTerrain *adjTer1 = NULL;
        int adjTerVert0 = -1;
        int adjTerVert1 = -1;

        if(edge > -1) {
            // I need to get the index of this vert, but on the other chunk.
            /// Copy
            if(edge % 2 == 1) {
                // Odd, meaning edge. Meaning 1 other chunks.
                // Use (edge + 1) to access chunkArrayCoords.
                // Temp [1+2] are currently todoVerts.
                temp[1] += chunkArrayCoords[edge + 1];
                temp[2] += chunkArrayCoords[edge + 2];
            } else {
                temp[1] += chunkArrayCoords[edge + 2];
                temp[2] += chunkArrayCoords[edge + 3];
            }
            adjTer0 = coordsToTer(&temp[1], terList);
            // Iterate entire adjTer, find the vert that overlaps with todoVert.

            // I think there is a problem here.
            float wx, wz;
            terToWorldNew(temp[1] - todoVert[1], temp[2] - todoVert[2], &wx, &wz, terScale);
            float delta[3];

//            printf("tempCoords (%d, %d)\n", temp[1], temp[2]);
//            printf("todoCoords (%d, %d)\n", todoVert[1], todoVert[2]);

            // When getting delta.
            // adjV - todoV + (adjC - todoC)

            // Convert it from adj to todo.
            // adjV + (adjC - todoC)

//            sayVect("todoLoc", todoTer->verts[todoVert[0]].loc, 3);
            for(adjTerVert0 = 0; adjTerVert0 < adjTer0->numVerts; adjTerVert0++) {
//                sayVect("adjTer0->verts[f].loc", adjTer0->verts[f].loc, 3);
                delta[0] = adjTer0->verts[adjTerVert0].loc[0] - todoTer->verts[todoVert[0]].loc[0] + wx;
                delta[1] = adjTer0->verts[adjTerVert0].loc[1] - todoTer->verts[todoVert[0]].loc[1];
                delta[2] = adjTer0->verts[adjTerVert0].loc[2] - todoTer->verts[todoVert[0]].loc[2] + wz;
//                printf("[2]: (%f, %f, %f)\n")
//                sayVect("delta", delta, 3);
                if(vectNorm(delta, 3) < .001) {
//                    printf("worldLoc: (%f, %f)\n", wx, wz);
//                    sayVect("adjTer0->verts[adjTerVert0].loc", adjTer0->verts[adjTerVert0].loc, 3);
//                    sayVect("todoTer->verts[todoVert[0]].loc", todoTer->verts[todoVert[0]].loc, 3);
                    // This is the overlap. Save it so i can use it for comparisons + the final loc set. Well this means i do need to save this pointer for the final set.
                    // I dont need to add it to any list because this 1 todo iteration handles every chunk.
                    break;
                }
            }
            if(adjTerVert0 != adjTer0->numVerts) {
                // Bug? cant find the vert on this chunk.
//                printf("adjTerVert0: %d\n", adjTerVert0);

                int temp1[3];
                temp1[0] = adjTerVert0;
                temp1[1] = adjTer0->chunkCoords[0];
                temp1[2] = adjTer0->chunkCoords[1];
                normNums += brushNormChunk(adjTer0, todo, doneEdges, chunkCoords, temp1, loc, temp, radius, normSum);
            }

            if(edge % 2 == 0) {
                /// Copy
                if(edge == 0) {
                    // If edge is 0, then im looking for
                    temp[1] = todoVert[1] + chunkArrayCoords[12];
                    temp[2] = todoVert[2] + chunkArrayCoords[13];
                } else {
                    temp[1] = todoVert[1] + chunkArrayCoords[edge];
                    temp[2] = todoVert[2] + chunkArrayCoords[edge + 1];
                }
                adjTer1 = coordsToTer(&temp[1], terList);
                // Iterate entire adjTer, find the vert that overlaps with todoVert.
                float wx, wz;
                terToWorldNew(temp[1] - todoVert[1], temp[2] - todoVert[2], &wx, &wz, terScale);

                for(adjTerVert1 = 0; adjTerVert1 < adjTer1->numVerts; adjTerVert1++) {
                    delta[0] = adjTer1->verts[adjTerVert1].loc[0] - todoTer->verts[todoVert[0]].loc[0] + wx;
                    delta[1] = adjTer1->verts[adjTerVert1].loc[1] - todoTer->verts[todoVert[0]].loc[1];
                    delta[2] = adjTer1->verts[adjTerVert1].loc[2] - todoTer->verts[todoVert[0]].loc[2] + wz;
                    if(vectNorm(delta, 3) < .001) {
                        // This is the overlap. Save it so i can use it for comparisons + the final loc set. Well this means i do need to save this pointer for the final set.
                        // I dont need to add it to any list because this 1 todo iteration handles every chunk.
                        break;
                    }
                }
                if(adjTerVert1 != adjTer1->numVerts) {
                    // Bug? cant find the vert on this chunk.
//                    printf("adjTerVert1: %d\n", adjTerVert1);

                    int temp1[3];
                    temp1[0] = adjTerVert1;
                    temp1[1] = adjTer1->chunkCoords[0];
                    temp1[2] = adjTer1->chunkCoords[1];
                    normNums += brushNormChunk(adjTer1, todo, doneEdges, chunkCoords, temp1, loc, temp, radius, normSum);
                }
            }
        }

//        sayVect("normalSumPre", normSum, 3);

        float wx = 0;
        float wz = 0;
        // chunkArrayCoords is delta from colChunk -> thisChunk (this - col)
        // cLoc - tLoc + (cChunk - tChunk)
        terToWorldNew(chunkCoords[0] - todoVert[1], chunkCoords[1] - todoVert[2], &wx, &wz, terScale);
        float delta[3];
        delta[0] = loc[0] - todoTer->verts[todoVert[0]].loc[0] + wx;
        delta[1] = loc[1] - todoTer->verts[todoVert[0]].loc[1];
        delta[2] = loc[2] - todoTer->verts[todoVert[0]].loc[2] + wz;

        float distToCol = vectNorm(delta, 3);

        float scalar = sigmoid2(distToCol);
        vectMultScalar(normSum, scalar / normNums, 3);

//        printf("scalar: %f\n", scalar);
//        sayVect("normSum", normSum, 3);

        if(edge == -1) {
            todoTer->verts[todoVert[0]].loc[0] += normSum[0];
            todoTer->verts[todoVert[0]].loc[1] += normSum[1];
            todoTer->verts[todoVert[0]].loc[2] += normSum[2];
        } else {
            todoTer->verts[todoVert[0]].loc[1] += normSum[1];
        }
        arrayListAddUnique(todoTer->movedVerts, &todoVert[0]);

//        sayVect("todoTer->verts[todoVert[0]].loc", todoTer->verts[todoVert[0]].loc, 3);
//        arrayListAddUnique(todoTer->movedVerts, &todoVert[0]);

        // I can loop these by using an array.
        if(adjTer0 != NULL) {
//            printf("otherChunk\n");
            // Now average the adjAvg and todoVert.
            // I need to convert adjAvg to chunkCoords.
            float wx, wz;
            terToWorldNew(adjTer0->chunkCoords[0] - todoVert[1], adjTer0->chunkCoords[1] - todoVert[2], &wx, &wz, terScale);
            // adjAvg centered on todoVertCoords.
            // I need to get the delta to center it onto adjTerCoords
//            adjTer0->verts[adjTerVert0].loc[0] += normSum[0];
            adjTer0->verts[adjTerVert0].loc[1] += normSum[1];
//            adjTer0->verts[adjTerVert0].loc[2] += normSum[2];

//            sayVect("adjTerVert", adjTer0->verts[adjTerVert0].loc, 3);

            arrayListAddUnique(adjTer0->movedVerts, &adjTerVert0);
        }
        if(adjTer1 != NULL) {
            // Now average the adjAvg and todoVert.
//            adjTer1->verts[adjTerVert1].loc[0] += normSum[0];
            adjTer1->verts[adjTerVert1].loc[1] += normSum[1];
//            adjTer1->verts[adjTerVert1].loc[2] += normSum[2];

            arrayListAddUnique(adjTer1->movedVerts, &adjTerVert1);
        }
    }
}

// Return the number of normals added to normSum.
// Ter is the chunks faces im iterating.
// int todoVert[3] is (vertIndex, vertCoords)
// Collision loc.
int brushNormChunk(struct jalbTerrain *ter, ArrayList *todo, ArrayList *doneEdges, int *chunkCoords, int *todoVert, float *loc, int *temp, float radius, float *normSum){
    printf("brushNormChunk() (temp: %d, %d, %d)\n", temp[0], temp[1], temp[2]);
    // This uses the same chunk as the todoVert. Which is not the same as inChunkCoords.
    float vVec[3];
    float uVec[3];
    float normal[3];

    int numReturn = 0;
    int f;
    for(f = 0; f < ter->numFaces; f++) {
        // If todoVert is part of this face, add the other 2 verts to the adjList.
        // I need to know which one it is, so i can add them using the complicated function.
        // I could just add them all. The arraylist search isnt bad.
        if(ter->faces[f * 3] == todoVert[0]) {
            // Do the calcs here.
            temp[0] = ter->faces[f * 3 +1];
            brushNormVert(ter, todo, doneEdges, chunkCoords, todoVert, loc, temp, radius);
            temp[0] = ter->faces[f * 3 +2];
            brushNormVert(ter, todo, doneEdges, chunkCoords, todoVert, loc, temp, radius);
        } else if(ter->faces[f * 3 +1] == todoVert[0]) {
            temp[0] = ter->faces[f * 3];
            brushNormVert(ter, todo, doneEdges, chunkCoords, todoVert, loc, temp, radius);
            temp[0] = ter->faces[f * 3 +2];
            brushNormVert(ter, todo, doneEdges, chunkCoords, todoVert, loc, temp, radius);
//            faceVerts[0] = ter->faces[f * 3];
//            faceVerts[1] = ter->faces[f * 3 + 2];
        } else if(ter->faces[f * 3 +2] == todoVert[0]) {
            temp[0] = ter->faces[f * 3];
            brushNormVert(ter, todo, doneEdges, chunkCoords, todoVert, loc, temp, radius);
            temp[0] = ter->faces[f * 3 +1];
            brushNormVert(ter, todo, doneEdges, chunkCoords, todoVert, loc, temp, radius);
//            faceVerts[0] = ter->faces[f * 3];
//            faceVerts[1] = ter->faces[f * 3 + 1];
        } else {
            // this face dosnt contain todo.
            continue;
        }
        // Add to normalSum.
        // Add this normal.
        vectSub(ter->verts[ter->faces[f * 3 +1]].loc, ter->verts[ter->faces[f * 3]].loc, vVec, 3);
        vectSub(ter->verts[ter->faces[f * 3 +2]].loc, ter->verts[ter->faces[f * 3]].loc, uVec, 3);
        crossProd3(vVec, uVec, normal);

        vectScaleTo(normal, .5, 3);

//        sayVect("normal", normal, 3);

        normSum[0] += normal[0];
        normSum[1] += normal[1];
        normSum[2] += normal[2];

//        sayVect("normSum", normSum, 3);

        numReturn += 1;
    }
    return numReturn;
}

void brushNormVert(struct jalbTerrain *ter, ArrayList *todo, ArrayList *doneEdges, int *chunkCoords, int *todoVert, float *loc, int *temp, float radius) {
//    printf("brushNormVert: (%d, %d, %d)\n", temp[0], temp[1], temp[2]);
    // What else do i need.
    // I need the loc of this vert relative to the todoVert. Since its all the same chunk i dont need terToWorld.
    // Delta from collision loc to this vert.
    float wx = 0;
    float wz = 0;
    terToWorldNew(chunkCoords[0] - temp[1], chunkCoords[1] - temp[2], &wx, &wz, terScale);
    float delta[3];
    // Delta = (Point - Vert) + (pointCoords - VertCoord);
    delta[0] = loc[0] - ter->verts[temp[0]].loc[0] + wx;
    delta[1] = loc[1] - ter->verts[temp[0]].loc[1];
    delta[2] = loc[2] - ter->verts[temp[0]].loc[2] + wz;

//    sayVect("delta", delta, 3);

    if(vectNorm(delta, 3) < radius) {
        // This vert is in range, add it to todo list.
        addTerVertUnique(todo, doneEdges, temp);
    }
}

void brushSmooth(float *loc, int faceIndex, int *chunkCoords, float radius) {
    printf("\nbrushSmooth (face: %d) (chunkCoords %d, %d)\n", faceIndex, chunkCoords[0], chunkCoords[1]);
    // I aught to save vertNum, and then pointer.
    ArrayList *todo = initArrayList(10, 3 * sizeof(int));
    ArrayList *doneEdges = initArrayList(10, 3 * sizeof(int)); // This is used to stop a 2nd recure of verts that are on chunk edge, and were already dealt with by that chunk.
    int doneVert = 0;

    int temp[3]; // Used as a buffer for filling the arrayListUnique.
    struct jalbTerrain *thisTer = coordsToTer(chunkCoords, terList);

    // Add the 3 verts of the collision face to the todo list.
    temp[0] = thisTer->faces[faceIndex * 3];
    temp[1] = thisTer->chunkCoords[0];
    temp[2] = thisTer->chunkCoords[1];
    arrayListAddUnique(todo, temp);

    temp[0] = thisTer->faces[faceIndex * 3 + 1];
    arrayListAddUnique(todo, temp);

    temp[0] = thisTer->faces[faceIndex * 3 + 2];
    arrayListAddUnique(todo, temp);

    int numToDo = arrayListGetLength(todo); // Only 3.
    // Keep track of this number manually.

    for(; doneVert < numToDo; doneVert++) {
        // VertId, ChunkX, ChunkY.
        int *todoVert = arrayListDataPointer(todo, doneVert);
        printf("doneVert: %d (todoVert %d, %d, %d)\n", doneVert, todoVert[0], todoVert[1], todoVert[2]);

        struct jalbTerrain *todoTer = coordsToTer(&todoVert[1], terList);

//        sayVect("todoTer->verts[todoVert[0]].loc", todoTer->verts[todoVert[0]].loc, 3);

        int edge = isOnEdge(todoTer->verts[todoVert[0]].loc[0], todoTer->verts[todoVert[0]].loc[2], terScale);
//        printf("Edge: %d\n", edge);

        // Fill the end of the buffer so i can add all relevant verts to todo / doneEdge.
        temp[1] = todoVert[1];
        temp[2] = todoVert[2];

        // Iterate all faces, if the face contains this vertIndex, then save the other 2 to the adj list.

        float adjAvg[3] = {0};
        int numAdj = 0;

        int faceVerts[2]; // Buffers the indexes of the

        // This uses the same chunk as the todoVert. Which is not the same as inChunkCoords.
        int f;
        for(f = 0; f < todoTer->numFaces; f++) {
            faceVerts[0] = -1;
            // If todoVert is part of this face, add the other 2 verts to the adjList.
            if(todoTer->faces[f * 3] == todoVert[0]) {
                // Do the calcs here.
                faceVerts[0] = todoTer->faces[f * 3 + 1];
                faceVerts[1] = todoTer->faces[f * 3 + 2];
            } else if(todoTer->faces[f * 3 +1] == todoVert[0]) {
                faceVerts[0] = todoTer->faces[f * 3];
                faceVerts[1] = todoTer->faces[f * 3 + 2];
            } else if(todoTer->faces[f * 3 +2] == todoVert[0]) {
                faceVerts[0] = todoTer->faces[f * 3];
                faceVerts[1] = todoTer->faces[f * 3 + 1];
            }
            int i;
            for(i = 0; i < 2; i++) {
                if(faceVerts[i] == -1) {
                    break;
                }
                temp[0] = faceVerts[i];

//                printf("avgCalc (%d, %d, %d)\n", temp[0], temp[1], temp[2]);

                // Add to adjAverage. Increment the counter for devision.
                adjAvg[0] += todoTer->verts[temp[0]].loc[0];
                adjAvg[1] += todoTer->verts[temp[0]].loc[1];
                adjAvg[2] += todoTer->verts[temp[0]].loc[2];
                numAdj += 1;

                // What else do i need.
                // I need the loc of this vert relative to the todoVert. Since its all the same chunk i dont need terToWorld.

                // Delta from collision loc to this vert.
                float wx = 0;
                float wz = 0;
                terToWorldNew(chunkCoords[0] - todoVert[1], chunkCoords[1] - todoVert[2], &wx, &wz, terScale);
                float delta[3];
                // Delta = (Point - Vert) + (pointCoords - VertCoord);
                delta[0] = loc[0] - todoTer->verts[temp[0]].loc[0] + wx;
                delta[1] = loc[1] - todoTer->verts[temp[0]].loc[1];
                delta[2] = loc[2] - todoTer->verts[temp[0]].loc[2] + wz;

                if(vectNorm(delta, 3) < radius) {
                    // This vert is in range, add it to todo list.
                    // How do i check the other verts, the overlap vert doesnt get mathed or added multiple times.
                    // I would normally check if on edge. But this is the start chunk so just add. Am i confused?
                    // When would i not, when the chunk i am searching is
                    int added = addTerVertUnique(todo, doneEdges, temp);
                    if(added) {
                        numToDo += 1;
                    }
                }
            }
        }

        struct jalbTerrain *adjTer0 = NULL;
        struct jalbTerrain *adjTer1 = NULL;
        int adjTerVert0 = -1;
        int adjTerVert1 = -1;

        if(edge > -1) {
            // I need to get the index of this vert, but on the other chunk.
            /// Copy
            if(edge % 2 == 1) {
                // Odd, meaning edge. Meaning 1 other chunks.
                // Use (edge + 1) to access chunkArrayCoords.
                // Temp [1+2] are currently todoVerts.
                temp[1] += chunkArrayCoords[edge + 1];
                temp[2] += chunkArrayCoords[edge + 2];
            } else {
                temp[1] += chunkArrayCoords[edge + 2];
                temp[2] += chunkArrayCoords[edge + 3];
            }
            adjTer0 = coordsToTer(&temp[1], terList);
            // Iterate entire adjTer, find the vert that overlaps with todoVert.

            // I think there is a problem here.
            float wx, wz;
            terToWorldNew(temp[1] - todoVert[1], temp[2] - todoVert[2], &wx, &wz, terScale);
            float delta[3];

//            printf("tempCoords (%d, %d)\n", temp[1], temp[2]);
//            printf("todoCoords (%d, %d)\n", todoVert[1], todoVert[2]);

            // When getting delta.
            // adjV - todoV + (adjC - todoC)

            // Convert it from adj to todo.
            // adjV + (adjC - todoC)

//            sayVect("todoLoc", todoTer->verts[todoVert[0]].loc, 3);
//            printf("worldLoc: (%f, %f)\n", wx, wz);
            for(f = 0; f < adjTer0->numVerts; f++) {
//                sayVect("adjTer0->verts[f].loc", adjTer0->verts[f].loc, 3);
                delta[0] = adjTer0->verts[f].loc[0] - todoTer->verts[todoVert[0]].loc[0] + wx;
                delta[1] = adjTer0->verts[f].loc[1] - todoTer->verts[todoVert[0]].loc[1];
                delta[2] = adjTer0->verts[f].loc[2] - todoTer->verts[todoVert[0]].loc[2] + wz;
//                printf("[2]: (%f, %f, %f)\n")
//                sayVect("delta", delta, 3);
                if(vectNorm(delta, 3) < .001) {
                    // This is the overlap. Save it so i can use it for comparisons + the final loc set. Well this means i do need to save this pointer for the final set.
                    // I dont need to add it to any list because this 1 todo iteration handles every chunk.
                    adjTerVert0 = f;
                    break;
                }
            }
//            printf("adjTerVert0: %d\n", adjTerVert0);

            if(adjTerVert0 != -1) {
            for(f = 0; f < adjTer0->numFaces; f++) {
                faceVerts[0] = -1;
                // If todoVert is part of this face, add the other 2 verts to the adjList.
                if(adjTer0->faces[f * 3] == adjTerVert0) {
                    // Do the calcs here.
                    faceVerts[0] = adjTer0->faces[f * 3 + 1];
                    faceVerts[1] = adjTer0->faces[f * 3 + 2];
                } else if(adjTer0->faces[f * 3 +1] == adjTerVert0) {
                    faceVerts[0] = adjTer0->faces[f * 3];
                    faceVerts[1] = adjTer0->faces[f * 3 + 2];
                } else if(adjTer0->faces[f * 3 +2] == adjTerVert0) {
                    faceVerts[0] = adjTer0->faces[f * 3];
                    faceVerts[1] = adjTer0->faces[f * 3 + 1];
                }
                int i;
                for(i = 0; i < 2; i++) {
                    if(faceVerts[i] == -1) {
                        break;
                    }
                    temp[0] = faceVerts[i];

//                    printf("avgCalc adj0 (%d, %d, %d)\n", temp[0], temp[1], temp[2]);

                    // Add to adjAverage. Increment the counter for devision.
                    adjAvg[0] += adjTer0->verts[temp[0]].loc[0] + wx;
                    adjAvg[1] += adjTer0->verts[temp[0]].loc[1];
                    adjAvg[2] += adjTer0->verts[temp[0]].loc[2] + wz;
                    numAdj += 1;

                    // What else do i need.
                    // I need the loc of this vert relative to the todoVert. Since its all the same chunk i dont need terToWorld.

                    // Delta from collision loc to this vert.
                    // inCoordsWorldDelta
                    float wx0 = 0;
                    float wz0 = 0;
                    terToWorldNew(chunkCoords[0] - temp[1], chunkCoords[1] - temp[2], &wx0, &wz0, terScale);
                    // Delta = (Point - Vert) + (pointCoords - VertCoord);
                    delta[0] = loc[0] - adjTer0->verts[temp[0]].loc[0] + wx0;
                    delta[1] = loc[1] - adjTer0->verts[temp[0]].loc[1];
                    delta[2] = loc[2] - adjTer0->verts[temp[0]].loc[2] + wz0;

                    if(vectNorm(delta, 3) < radius) {
                        // This vert is in range, add it to todo list.
                        // How do i check the other verts, the overlap vert doesnt get mathed or added multiple times.
                        // I would normally check if on edge. But this is the start chunk so just add. Am i confused?
                        // When would i not, when the chunk i am searching is
                        int added = addTerVertUnique(todo, doneEdges, temp);
//                        printf("added: %d (temp: %d, %d, %d)\n", added, temp[0], temp[1], temp[2]);
                        if(added) {
                            numToDo += 1;
                        }
                    }
                }
            }
            }

            if(edge % 2 == 0) {
                /// Copy
                if(edge == 0) {
                    // If edge is 0, then im looking for
                    temp[1] = todoVert[1] + chunkArrayCoords[12];
                    temp[2] = todoVert[2] + chunkArrayCoords[13];
                } else {
                    temp[1] = todoVert[1] + chunkArrayCoords[edge];
                    temp[2] = todoVert[2] + chunkArrayCoords[edge + 1];
                }
                adjTer1 = coordsToTer(&temp[1], terList);
                // Iterate entire adjTer, find the vert that overlaps with todoVert.
                float wx, wz;
                terToWorldNew(temp[1] - todoVert[1], temp[2] - todoVert[2], &wx, &wz, terScale);

                for(f = 0; f < adjTer1->numVerts; f++) {
                    delta[0] = adjTer1->verts[f].loc[0] - todoTer->verts[todoVert[0]].loc[0] + wx;
                    delta[1] = adjTer1->verts[f].loc[1] - todoTer->verts[todoVert[0]].loc[1];
                    delta[2] = adjTer1->verts[f].loc[2] - todoTer->verts[todoVert[0]].loc[2] + wz;
                    if(vectNorm(delta, 3) < .001) {
                        // This is the overlap. Save it so i can use it for comparisons + the final loc set. Well this means i do need to save this pointer for the final set.
                        // I dont need to add it to any list because this 1 todo iteration handles every chunk.
                        adjTerVert1 = f;
                        break;
                    }
                }
//                printf("adjTerVert1: %d\n", adjTerVert1);
                if(adjTerVert1 != -1) {
                    for(f = 0; f < adjTer1->numFaces; f++) {
                        faceVerts[0] = -1;
                        // If todoVert is part of this face, add the other 2 verts to the adjList.
                        if(adjTer1->faces[f * 3] == adjTerVert1) {
                            // Do the calcs here.
                            faceVerts[0] = adjTer1->faces[f * 3 + 1];
                            faceVerts[1] = adjTer1->faces[f * 3 + 2];
                        } else if(adjTer1->faces[f * 3 +1] == adjTerVert1) {
                            faceVerts[0] = adjTer1->faces[f * 3];
                            faceVerts[1] = adjTer1->faces[f * 3 + 2];
                        } else if(adjTer1->faces[f * 3 +2] == adjTerVert1) {
                            faceVerts[0] = adjTer1->faces[f * 3];
                            faceVerts[1] = adjTer1->faces[f * 3 + 1];
                        }
                        int i;
                        for(i = 0; i < 2; i++) {
                            if(faceVerts[i] == -1) {
                                break;
                            }
                            temp[0] = faceVerts[i];

//                            printf("avgCalc adj1 (%d, %d, %d)\n", temp[0], temp[1], temp[2]);

                            // Add to adjAverage. Increment the counter for devision.
                            adjAvg[0] += adjTer1->verts[temp[0]].loc[0] + wx;
                            adjAvg[1] += adjTer1->verts[temp[0]].loc[1];
                            adjAvg[2] += adjTer1->verts[temp[0]].loc[2] + wz;
                            numAdj += 1;

                            // What else do i need.
                            // I need the loc of this vert relative to the todoVert. Since its all the same chunk i dont need terToWorld.

                            // Delta from collision loc to this vert.
                            // inCoordsWorldDelta
                            float wx0 = 0;
                            float wz0 = 0;
                            terToWorldNew(chunkCoords[0] - temp[1], chunkCoords[1] - temp[2], &wx0, &wz0, terScale);
                            // Delta = (Point - Vert) + (pointCoords - VertCoord);
                            delta[0] = loc[0] - adjTer1->verts[temp[0]].loc[0] + wx0;
                            delta[1] = loc[1] - adjTer1->verts[temp[0]].loc[1];
                            delta[2] = loc[2] - adjTer1->verts[temp[0]].loc[2] + wz0;

                            if(vectNorm(delta, 3) < radius) {
                                // This vert is in range, add it to todo list.
                                // How do i check the other verts, the overlap vert doesnt get mathed or added multiple times.
                                // I would normally check if on edge. But this is the start chunk so just add. Am i confused?
                                // When would i not, when the chunk i am searching is
                                int added = addTerVertUnique(todo, doneEdges, temp);
//                                printf("added: %d (temp: %d, %d, %d)\n", added, temp[0], temp[1], temp[2]);
                                if(added) {
                                    numToDo += 1;
                                }
                            }
                        }
                    }
                }
            }
        }

        adjAvg[0] /= numAdj;
        adjAvg[1] /= numAdj;
        adjAvg[2] /= numAdj;

//        sayVect("adjAvg", adjAvg, 3);
//        sayVect("todoTer->verts[todoVert[0]].loc", todoTer->verts[todoVert[0]].loc, 3);

        // Now average the adjAvg and todoVert.
        if(edge == -1) {
        todoTer->verts[todoVert[0]].loc[0] += adjAvg[0];
        todoTer->verts[todoVert[0]].loc[1] += adjAvg[1];
        todoTer->verts[todoVert[0]].loc[2] += adjAvg[2];

        // Instead of getting getting avg. I should decide the weight of each depending on the distance of this point to the collisionLoc.
        todoTer->verts[todoVert[0]].loc[0] /= 2;
        todoTer->verts[todoVert[0]].loc[1] /= 2;
        todoTer->verts[todoVert[0]].loc[2] /= 2;
        } else {
            todoTer->verts[todoVert[0]].loc[1] += adjAvg[1];
            todoTer->verts[todoVert[0]].loc[1] /= 2;
        }
//        sayVect("todoTer->verts[todoVert[0]].loc", todoTer->verts[todoVert[0]].loc, 3);

        arrayListAddUnique(todoTer->movedVerts, &todoVert[0]);

        // I can loop these by using an array.
        if(adjTer0 != NULL) {
//            printf("otherChunk\n");
            // Now average the adjAvg and todoVert.
            // I need to convert adjAvg to chunkCoords.
            float wx, wz;
            terToWorldNew(adjTer0->chunkCoords[0] - todoVert[1], adjTer0->chunkCoords[1] - todoVert[2], &wx, &wz, terScale);
            // adjAvg centered on todoVertCoords.
            // I need to get the delta to center it onto adjTerCoords
//            sayVect("adjTerVert", adjTer0->verts[adjTerVert0].loc, 3);
//            printf("thisWorld (%f, %f)\n", wx, wz);

//            adjTer0->verts[adjTerVert0].loc[0] += adjAvg[0] - wx;
            adjTer0->verts[adjTerVert0].loc[1] += adjAvg[1];
//            adjTer0->verts[adjTerVert0].loc[2] += adjAvg[2] - wz;

//            adjTer0->verts[adjTerVert0].loc[0] /= 2;
            adjTer0->verts[adjTerVert0].loc[1] /= 2;
//            adjTer0->verts[adjTerVert0].loc[2] /= 2;

//            sayVect("adjTerVert", adjTer0->verts[adjTerVert0].loc, 3);

            arrayListAddUnique(adjTer0->movedVerts, &adjTerVert0);
        }
        if(adjTer1 != NULL) {
            // Now average the adjAvg and todoVert.
            float wx, wz;
            terToWorldNew(adjTer1->chunkCoords[0] - todoVert[1], adjTer1->chunkCoords[1] - todoVert[2], &wx, &wz, terScale);

//            adjTer1->verts[adjTerVert1].loc[0] += adjAvg[0] - wx;
            adjTer1->verts[adjTerVert1].loc[1] += adjAvg[1];
//            adjTer1->verts[adjTerVert1].loc[2] += adjAvg[2] - wz;

//            adjTer1->verts[adjTerVert1].loc[0] /= 2;
            adjTer1->verts[adjTerVert1].loc[1] /= 2;
//            adjTer1->verts[adjTerVert1].loc[2] /= 2;

            arrayListAddUnique(adjTer1->movedVerts, &adjTerVert1);
        }
    }
}

// Return numAvg increment. Or increment by 1 no matter what.
void adjMath(struct jalbTerrain *todoTer, float *adjAvg, int f, int *todoCoords, int *chunkCoords, float *loc, float radius){
    // Do the calcs here.
//    temp[0] = todoTer->faces[f * 3 + 1];

    // Add to adjAverage. Increment the counter for devision.
    adjAvg[0] += todoTer->verts[todoTer->faces[f * 3 + 1]].loc[0];
    adjAvg[1] += todoTer->verts[todoTer->faces[f * 3 + 1]].loc[1];
    adjAvg[2] += todoTer->verts[todoTer->faces[f * 3 + 1]].loc[2];
//    numAdj += 1;

    // What else do i need.
    // I need the loc of this vert relative to the todoVert. Since its all the same chunk i dont need terToWorld.

    // Delta from collision loc to this vert.
    float wx, wz;
    terToWorld(chunkCoords[0] - todoCoords[0], chunkCoords[1] - todoCoords[1], &wx, &wz, terScale);
    float delta[3];
    // Delta = (Point - Vert) + (pointCoords - VertCoord);
    delta[0] = loc[0] - todoTer->verts[todoTer->faces[f * 3 + 1]].loc[0] + wx;
    delta[1] = loc[1] - todoTer->verts[todoTer->faces[f * 3 + 1]].loc[1];
    delta[2] = loc[2] - todoTer->verts[todoTer->faces[f * 3 + 1]].loc[2] + wz;

    if(vectNorm(delta, 3) < radius) {
        // This vert is in range, add it to todo list.
        // How do i check the other verts, the overlap vert doesnt get mathed or added multiple times.
        // Is this vert on edge,
        // The 1st time i call this, i dont need to do this check, just add blindly.
    }
}

void offChunkCheck(struct jalbGravity *gravMem) {
    // If the position is off edge of chunk:
    // Change coords, refill ter array, and change position.
    int edge = isOnEdgeNew(gravMem->position[0], gravMem->position[2], terScale);
    if(edge != -1) {
        // type did not include corners, this converts it to skip (0, 0) of chunkArrayCoords..
        edge += 1;

        gravMem->chunkCoords[0] += chunkArrayCoords[edge * 2];
        gravMem->chunkCoords[1] += chunkArrayCoords[edge * 2 +1];

        /// Fix it: 2nd var, radius
        embedSpawnTer(terList, 4, gravMem->chunkCoords[0], gravMem->chunkCoords[1], terScale, terEdge);

        fillTerArray(gravMem, terList);

        float wx = 0;
        float wy = 0;
        terToWorldNew(chunkArrayCoords[edge * 2], chunkArrayCoords[edge * 2 + 1], &wx, &wy, terScale);

        gravMem->position[0] -= wx;
        gravMem->position[2] -= wy;
    }
}

float sigmoid2(float x) {
    x -= 4;
    return 1.0 / (1 + pow(2, x));
}

// 2 unique arraylists, inVert is 3 ints (vertId, chunkCoords)
// Return 1 if it is added.
int addTerVertUnique(ArrayList *todo, ArrayList *doneEdge, int *inVert) {
//    printf("addTerVertUnique: inVert (%d, %d, %d)\n", inVert[0], inVert[1], inVert[2]);
    // First check todo or doneEdge?
    if(arrayListContain(todo, inVert)) {
        // It is already saved.
//        printf("alreadySaved: todo\n");
        return 0;
    }
    if(arrayListContain(todo, doneEdge)) {
//        printf("alreadySaved: doneEdge\n");
        // It is already saved.
        return 0;
    }
    // It is not clearly contained by either list.

    // Fuck this means i need to rerun chunkToCoord every tic.
    // I should just make 3 lists one for each chunk. But this doesnt work either.
    // Iterate todo list, check if the vert is on.

    struct jalbTerrain *inTer = coordsToTer(&inVert[1], terList);

//    sayVect("inTer->verts[inVert[0]].loc", inTer->verts[inVert[0]].loc, 3);

    int edge = isOnEdge(inTer->verts[inVert[0]].loc[0], inTer->verts[inVert[0]].loc[2], terScale);
//    printf("edge: %d\n", edge);
    if(edge != -1) {
        // It is an edge, check the touching chunks, find the overlap vert index.
        // Then: If that (thoes) indexes are not in either list, then add inVert to todo, and thoes other to doneEdge.

        // This finds the first chunk it shares.
        // Iterate it and compare locs.
        float wx = 0;
        float wz = 0;
        struct jalbTerrain *adjChunk = NULL;
        int adjChunkVert[3];
        if(edge % 2 == 1) {
            // Odd, meaning edge. Meaning 1 other chunks.
            // Use (edge + 1) to access chunkArrayCoords.
            // Temp [1+2] are currently todoVerts.
            terToWorldNew(chunkArrayCoords[edge + 1], chunkArrayCoords[edge + 2], &wx, &wz, terScale);
            adjChunkVert[1] = inVert[1] +chunkArrayCoords[edge + 1];
            adjChunkVert[2] = inVert[2] +chunkArrayCoords[edge + 2];
        } else {
            terToWorldNew(chunkArrayCoords[edge + 2], chunkArrayCoords[edge + 3], &wx, &wz, terScale);
            adjChunkVert[1] = inVert[1] +chunkArrayCoords[edge + 2];
            adjChunkVert[2] = inVert[2] +chunkArrayCoords[edge + 3];
        }
        adjChunk = coordsToTer(&adjChunkVert[1], terList);
//        printf("adjChunkCoords0 (%d, %d)\n", adjChunkVert[1], adjChunkVert[2]);

        // Now i need to iterate the list of this chunk,
        int j;
        for(j = 0; j < adjChunk->numVerts; j++) {
            // Subtract world loc because it is currently additive, not delta.
            // inLoc - thisVert - worldDelta (thisCoord - inCoords)
            // (inLoc + inCoords) - (thisVert - thisCoord) : Commutative / Distributive property.
            float delta[3];
            delta[0] = inTer->verts[inVert[0]].loc[0] - adjChunk->verts[j].loc[0] - wx;
            delta[1] = inTer->verts[inVert[0]].loc[1] - adjChunk->verts[j].loc[1];
            delta[2] = inTer->verts[inVert[0]].loc[2] - adjChunk->verts[j].loc[2] - wz;
            if(vectNorm(delta, 3) < .001) {
//                printf("overlap0 (vertIndex: %d)\n", j);
                // This is the vertIndex im looking for.
                adjChunkVert[0] = j;
                if(arrayListContain(todo, adjChunkVert)) {
//                    printf("todo Contain\n");
                    // This is on todo, so add inVert to doneEdge.
                    arrayListAddEnd(doneEdge, inVert);
                    return 0;
                    // This ignores the 3rd possible overlap.
                }
                if(arrayListContain(doneEdge, adjChunkVert)) {
//                    printf("doneEdge Contain\n");
                    // This is on doneEdge, so add inVert to doneEdge. (since this is on doneEdge it means the 3rd overlap is already on todo).
                    arrayListAddEnd(doneEdge, inVert);
                    return 0;
                    // This ignores the 3rd possible overlap.
                }
                arrayListAddEnd(doneEdge, adjChunkVert);
                // If neither list contains thisVert.
                // Then i need to continue searching the next touching chunk.
                // Lets say the next also doesnt have inVert.
                // then i want to add this and that to doneEdge, and in to todo.
                // Lets say the next does have inVert.
                // then i want to add in to todo.
                // There is no chance the next is on done. the only way for that to happen would be, this or in are part of todo (which they are not)
            }
        }
        if(edge % 2 == 0) {
//            printf("2ndTer check start\n");
            if(edge == 0) {
                // If edge is 0, then im looking for
                terToWorldNew(chunkArrayCoords[12], chunkArrayCoords[13], &wx, &wz, terScale);
                adjChunkVert[1] = inVert[1] + chunkArrayCoords[12];
                adjChunkVert[2] = inVert[2] + chunkArrayCoords[13];
            } else {
                terToWorldNew(chunkArrayCoords[edge], chunkArrayCoords[edge + 1], &wx, &wz, terScale);
                adjChunkVert[1] = inVert[1] + chunkArrayCoords[edge];
                adjChunkVert[2] = inVert[2] + chunkArrayCoords[edge + 1];
            }
            adjChunk = coordsToTer(&adjChunkVert[1], terList);
//            printf("adjChunkCoords1 (%d, %d)\n", adjChunkVert[1], adjChunkVert[2]);

            /// Copy
            // Now i need to iterate the list of this chunk,
            int j;
            for(j = 0; j < adjChunk->numVerts; j++) {
                // Subtract world loc because it is currently additive, not delta.
                // inLoc - thisVert - worldDelta (thisCoord - inCoords)
                // (inLoc + inCoords) - (thisVert - thisCoord) : Commutative / Distributive property.
                float delta[3];
//                sayVect("inTer->verts[inVert[0]].loc", inTer->verts[inVert[0]].loc, 3);
//                sayVect("adjChunk->verts[j].loc", adjChunk->verts[j].loc, 3);
//                printf("world (%f, %f)\n", wx, wz);
                delta[0] = inTer->verts[inVert[0]].loc[0] - adjChunk->verts[j].loc[0] - wx;
                delta[1] = inTer->verts[inVert[0]].loc[1] - adjChunk->verts[j].loc[1];
                delta[2] = inTer->verts[inVert[0]].loc[2] - adjChunk->verts[j].loc[2] - wz;
//                sayVect("delta", delta, 3);
                if(vectNorm(delta, 3) < .001) {
//                    printf("overlap1 (vertIndex: %d)\n", j);
                    // This is the vertIndex im looking for.
                    // If this vert is on on todo, then add in to doneEdge.
                    // If this vert is NOT on todo, then add in to todo, and this to doneEdge
                    adjChunkVert[0] = j;
                    if(arrayListContain(todo, adjChunkVert)) {
                        // This is on todo, so add inVert to doneEdge.
                        arrayListAddEnd(doneEdge, inVert);
                        return 0;
                    } else {
                        arrayListAddEnd(doneEdge, adjChunkVert);
                    }
                }
            }
        }
        // Dont need unique add because i already know its not on the list.
        arrayListAddEnd(todo, inVert);
        return 1;
    } else {
        // Its not an edge. So the first checks were enough.
        // So just add it.
        return arrayListAddUnique(todo, inVert);
    }
}

