#include <stdlib.h>
#include <stdio.h>

#include "jama.h"
#include "rayFace.h"

float newerCircRayFaceScalar(float *rPoint, float *rVect, float radius, float *f0, float *f1, float *f2, float *retBarys, int *retType) {
    // What side of the face is the ray start on? Move the face perpendicular in that direction.
    // Then do ray face detection. If it there is no intersect. then using the bary coords to figure out which (side/ 2sides) the ray plane collision is on.
    // Then do ray line detection and ray circ detection to get the correct answer.

    float v[3];
    vectSub(f1, f0, v, 3);
    float u[3];
    vectSub(f2, f0, u, 3);

    float v2[3];
    vectSub(rPoint, f0, v2, 3);

    // Compute dot products
    float dot00 = dotProd(v, v, 3);
    float dot01 = dotProd(v, u, 3);
    float dot02 = dotProd(v, v2, 3);
    float dot11 = dotProd(u, u, 3);
    float dot12 = dotProd(u, v2, 3);

    // Compute barycentric coordinates.
    float invDenom = 1 / (dot00 * dot11 - dot01 * dot01);
    retBarys[0] = (dot11 * dot02 - dot01 * dot12) * invDenom;
    retBarys[1] = (dot00 * dot12 - dot01 * dot02) * invDenom;

    // |(v * b0 + u * b1) - v| = dist from face.

//    sayVect("v", v, 3);
//    sayVect("u", u, 3);
//    sayVect("v2", v2, 3);

    // Point on face - rPoint.
    float temp1[3];
    temp1[0] = v[0] * retBarys[0] + u[0] * retBarys[1] - v2[0];
    temp1[1] = v[1] * retBarys[0] + u[1] * retBarys[1] - v2[1];
    temp1[2] = v[2] * retBarys[0] + u[2] * retBarys[1] - v2[2];

//    sayVect("temp1", temp1, 3);

    float distFromFace = vectNorm(temp1, 3);
//    printf("distFromFace: %f\n", distFromFace);

//    printf("distFromFace: %f\n", distFromFace);

    distFromFace -= radius;

    float normal[3];
    crossProd3(v, u, normal);

    vectNormalize(normal, 3);

//    sayVect("normal", normal, 3);

//    float RdN = dotProd(rVect, normal, 3);
//    printf("RnD: %f\n", RdN);

    float ratio = dotProd(rVect, normal, 3);

//    printf("ratio: %f\n", ratio);

//    printf("Ratio: %f\n", ratio);

    float rayScalar = -distFromFace / ratio;

//    printf("rayScalar: %f\n", rayScalar);

    // point on face from rPoint + rRay * rScalar
    // But off by normal.
    temp1[0] = rPoint[0] + rVect[0] * rayScalar;
    temp1[1] = rPoint[1] + rVect[1] * rayScalar;
    temp1[2] = rPoint[2] + rVect[2] * rayScalar;

//    sayVect("temp1", temp1, 3);
    /// Get new barys
    // Set new v2
    vectSub(temp1, f0, v2, 3);
    // Recompute variables involving v2.
    dot02 = dotProd(v, v2, 3);
    dot12 = dotProd(u, v2, 3);
    // Compute barycentric coordinates.
    retBarys[0] = (dot11 * dot02 - dot01 * dot12) * invDenom;
    retBarys[1] = (dot00 * dot12 - dot01 * dot02) * invDenom;

//    sayVect("faceCollisionBarys", retBarys, 2);

    // If on the face.
    if(retBarys[0] >= 0 && retBarys[1] >= 0 && (1 - retBarys[0] - retBarys[1]) >= 0) {
        // Return the scalar.
        *retType = 0;
        return rayScalar;
        // The scalar is t.
    }
    // The collision is not on the face.
    else {
        // I need to do edge detection.
        // One or two of the bary coords are negative.

        float scalar0;
        float scalar1;

        if(retBarys[0] < 0) {
            // Do edge.
            float b0;
            int type0 = -1;
            scalar0 = newerCircRaySegScalar(f0, f2, rPoint, rVect, radius, &b0, &type0);
//            printf("Type0: %d\n", type0);

            // Since the last didnt return corner, this shouldnt either, atmost edge.
            if(retBarys[1] < 0) {
                float b1;
                int t1;
                scalar1 = newerCircRaySegScalar(f0, f1, rPoint, rVect, radius, &b1, &t1);
//                printf("scals (%f, %f)\n", scalar0, scalar1);
//                printf("retType: %d\n", retType);
                if(scalar1 < scalar0) {
                    if(t1 == 0) {
                        *retType = 1;
                        retBarys[0] = 0;
                        retBarys[1] = b1;
                        // E0
                    } else if (t1 == 1) {
                        *retType = 4;
                        // Corner. 1
                    } else if (t1 == 2) {
                        *retType = 5;
                        // Corner. 2
                    }
                    return scalar1;
                }
            } else if (1 - retBarys[0] - retBarys[1] < 0) {

                float bary0;
                int type1;

                scalar1 = newerCircRaySegScalar(f1, f2, rPoint, rVect, radius, &bary0, &type1);
                if(scalar1 < scalar0) {
                    retBarys[0] = 1 - retBarys[1];
                    if(type1 == 0) {
                        *retType = 3;
                        retBarys[1] = bary0;
                        retBarys[0] = 1 - retBarys[1];
                    } else if (type1 == 1) {
                        *retType = 5;
                    } else if (type1 == 2) {
                        *retType = 6;
                    }

                    return scalar1;
                }
            }

            if(type0 == 0) {
                *retType = 2;
                retBarys[0] = b0;
                retBarys[1] = 0;
            } else if (type0 == 1) {
                *retType = 4;
            } else if (type0 == 2) {
                *retType = 6;
            }

            return scalar0;

        } else if(retBarys[1] < 0) {
            float b0;
            int type0 = -1;

            scalar0 = newerCircRaySegScalar(f0, f1, rPoint, rVect, radius, &b0, &type0);


            if (1 - retBarys[0] - retBarys[1] < 0) {
                float b1;
                int type1;

                scalar1 = newerCircRaySegScalar(f1, f2, rPoint, rVect, radius, &b1, &type1);
                if(scalar1 < scalar0) {
                    if(type1 == 0) {
                        *retType = 3;
                        retBarys[1] = b1;
                        retBarys[0] = 1 - retBarys[1];
                    } else if (type1 == 1) {
                        *retType = 5;
                    } else if (type1 == 2) {
                        *retType = 6;
                    }
                    return scalar1;
                }
            }

            if(type0 == 0) {
                *retType = 1;
                retBarys[1] = b0;
                retBarys[0] = 1 - retBarys[1];
            } else if (type0 == 1) {
                *retType = 4;
            } else if (type0 == 2) {
                *retType = 5;
            }

            return scalar0;

        } else {
            // Only 3

            float b0;
            int type0 = -1;

            scalar0 = newerCircRaySegScalar(f1, f2, rPoint, rVect, radius, &b0, &type0);

            if(type0 == 0) {
                *retType = 3;
                retBarys[1] = b0;
                retBarys[0] = 1 - retBarys[1];
            } else if (type0 == 1) {
                *retType = 5;
            } else if (type0 == 2) {
                *retType = 6;
            }

            return scalar0;
        }
    }
}

float newerCircRaySegScalar(float *s0, float *s1, float *r0, float *rv, float radius, float *edgeScalar, int *retType) {
    float sv[3];
    vectSub(s1, s0, sv, 3);
    float svMag = vectNorm(sv, 3);
    vectMultScalar(sv, 1/svMag, 3);
    float rRel[3];
    vectSub(r0, s0, rRel, 3);

    float t = dotProd(rv, sv, 3);
//    printf("t: %f\n", t);

    float x[3];
    x[0] = sv[0] * t - rv[0];
    x[1] = sv[1] * t - rv[1];
    x[2] = sv[2] * t - rv[2];
//    sayVect("x", x, 3);

    t = dotProd(rRel, sv, 3);
//    printf("t: %f\n", t);

    float y[3];
    y[0] = sv[0] * t - rRel[0];
    y[1] = sv[1] * t - rRel[1];
    y[2] = sv[2] * t - rRel[2];
//    sayVect("y", y, 3);

    float dot00 = dotProd(x, x, 3);
    float dot01 = dotProd(x, y, 3);
    float dot11 = dotProd(y, y, 3);

//    printf("dots (%f, %f, %f)\n", dot00, dot01, dot11);

    float v3 = dot11 - radius * radius;

    float scals[2];
    quadEq(dot00, 2 * dot01, v3, scals);

//    sayVect("qEqScals", scals, 2);

    int lowest;

    if(scals[0] >= 0) {
        if(scals[1] >= 0) {
            if(scals[1] < scals[0]) {
                // s1
                lowest = 1;
            } else {
                lowest = 0;
            }
        } else {
            lowest = 0;
        }
    } else if(scals[1] >= 0) {
        // s1
        lowest = 1;
    } else {
        // All of the scals are negative.
//        printf("bothScalsAreNegative.\n");
        return -1;
    }

    float tempPoint[3];
    tempPoint[0] = rRel[0] + scals[lowest] * rv[0];
    tempPoint[1] = rRel[1] + scals[lowest] * rv[1];
    tempPoint[2] = rRel[2] + scals[lowest] * rv[2];

    // So this is the magnitude of edgeLoc.
    *edgeScalar = dotProd(sv, tempPoint, 3);
//    *edgeScalar /= dotProd(sv, sv, 3); /** sv is normalized so this doenst matter */
    *edgeScalar /= svMag;

//    printf("*edgeScalar: %f\n", *edgeScalar);

    if(*edgeScalar >= 0 && *edgeScalar <= 1) {
        // This scalar is off the seg so invalid. Check the next. If the next is invalid then quit.
        *retType = 0;
        return scals[lowest];
    } else {
        float scalar;
        if(*edgeScalar < 0) {
            *retType = 1;
            scalar = newCircRayPointScalar(s0, r0, rv, radius);
            *edgeScalar = 0;
        } else {
            *retType = 2;
            scalar = newCircRayPointScalar(s1, r0, rv, radius);
            *edgeScalar = 1;
        }
        return scalar;
    }
}

float newCircRayPointScalar(float *cPoint, float *rPoint, float *rVect, float radius) {
//    sayVect("cPoint", cPoint, 3);
//    sayVect("rPoint", rPoint, 3);
//    sayVect("rVect", rVect, 3);
    float dv[3];
    vectSub(rPoint, cPoint, dv, 3);

    float a = dotProd(rVect, rVect, 3);
    float b = 2 * dotProd(rVect, dv, 3);
    float c = dotProd(dv, dv, 3) - radius * radius;

    float qEq[2];
    quadEq(a, b, c, qEq);

//    sayVect("qEqCirc", qEq, 2);

    // Pick the lowest of the 2 and return.

    if(qEq[0] >= 0) {
        if(qEq[1] >= 0) {
            if(qEq[1] < qEq[0]) {
                return qEq[1];
            }
        }
        // [1] < 0 _OR_
        // [0] <= [1]
        return qEq[0];
    } else if(qEq[1] >= 0) {
        return qEq[1];
    } else {
        return -1;
    }
}

float pointFaceDist(float *pPoint, float *f0, float *f1, float *f2) {
//    printf("pointFaceDist()\n");
    float *v = calloc(3, sizeof(float));
    vectSub(f1, f0, v, 3);
    float *u = calloc(3, sizeof(float));
    vectSub(f2, f0, u, 3);

    /// Fix It
    float *normal = crossProduct3(v, u);

//    float m = rayFaceCollisionMag(f0, f1, f2, rPoint, rVect);

    vectNormalize(normal, 3);

    /** Old
      *  float d = -dotProd(f1, normal, 3);
      *  /// I just made the denom normal squared. Also remove negative.
      *  float t  = (dotProd(pPoint, normal, 3) + d) / dotProd(normal, normal, 3);
      */

    /// I dont know the direction of normal, how do i account for this?

    // t is the scalar of the ray(rPoint, normal) to the plane.
    // rPoint - f0
    float d = dotProd(f0, normal, 3);
    float t  = (dotProd(pPoint, normal, 3) - d);

    float *point = malloc(sizeof(float) * 3);

    point[0] = normal[0];
    point[1] = normal[1];
    point[2] = normal[2];

    free(normal);

    vectMultScalar(point, t, 3);
    vectAdd(point, pPoint, point, 3);

    // Point
    float *v2 = malloc(sizeof(float) * 3);

    vectSub(f1, f0, v, 3);
    vectSub(f2, f0, u, 3);
    vectSub(point, f0, v2, 3);

    free(point);

//    sayVect("v", v, 3);
//    sayVect("u", u, 3);

    // Compute dot products
    float dot00 = dotProd(v, v, 3);
    float dot01 = dotProd(v, u, 3);
    float dot02 = dotProd(v, v2, 3);
    float dot11 = dotProd(u, u, 3);
    float dot12 = dotProd(u, v2, 3);

    free(v);
    free(u);
    free(v2);

    // Compute barycentric coordinates.
    float invDenom = 1 / (dot00 * dot11 - dot01 * dot01);
    float scale1 = (dot11 * dot02 - dot01 * dot12) * invDenom;
    float scale2 = (dot00 * dot12 - dot01 * dot02) * invDenom;
//    float scale3 = 1 - (scale1 + scale2);

//    printf("BaryCoords (%f, %f)\n", scale1, scale2);

    if(scale1 >= 0 && scale2 >= 0 && (scale1 + scale2) < 1) {
        if(t < 0) {
            return -t;
        }
        return t;
    }
    // If only one is negative do a point line distance. ITs going to be one the line.
    // If two are negative do a point point distance.
    if(scale1 < 0) {
        return pointSegDist(f0, f2, pPoint);
    } else if(scale2 < 0) {
        return pointSegDist(f0, f1, pPoint);
    } else {
        return pointSegDist(f1, f2, pPoint);
    }
}

float pointSegDist(float *f0, float *f1, float *pPoint) {
//    printf("pointSegDist()\n");
    // Project and reject.
    float *v = malloc(3 * sizeof(float));
    vectSub(f1, f0, v, 3);
    float segDist = vectNorm(v, 3);
    float *pv = malloc(3 * sizeof(float));
    vectSub(pPoint, f0, pv, 3);

    vectNormalize(v, 3);

    float scalar = dotProd(pv, v, 3);
//    printf("segScalar: %f\n", scalar);
    float norm;
    if(scalar > segDist) {
        vectSub(f1, pPoint, v, 3);
//        printf("vectNorm(v, 3: %f\n", vectNorm(v, 3));
        norm = vectNorm(v, 3);
    } else if (scalar < 0) {
        vectSub(f0, pPoint, v, 3);
//        printf("vectNorm(v, 3: %f\n", vectNorm(v, 3));
        norm = vectNorm(v, 3);
    } else {
        vectMultScalar(v, scalar, 3);
        vectSub(v, pv, v, 3);
//        printf("vectNorm(v, 3: %f\n", vectNorm(v, 3));
        norm = vectNorm(v, 3);
    }
    free(v);
    free(pv);
    return norm;
}

void baryFacePoint(float *point, float *f0, float *f1, float *f2, float *barys) {
    float v[3];
    vectSub(f1, f0, v, 3);
    float u[3];
    vectSub(f2, f0, u, 3);

    point[0] = f0[0] + v[0] * barys[0] + u[0] * barys[1];
    point[1] = f0[1] + v[1] * barys[0] + u[1] * barys[1];
    point[2] = f0[2] + v[2] * barys[0] + u[2] * barys[1];
}

